package com.gic.enterprise.constant;

import java.util.ArrayList;
import java.util.List;

/**
 * @author guojx
 * @date 2019/7/18 11:22 AM
 */
public enum  EnterpriseInitCodeEnum {
    CREATE_MEMBER_CARD("createMemberCard", "至少新建一张会员卡", 3),
    CREATE_MEMBER_GRADE("createMemberGrade", "至少新建一个会员等级", 3),
    CREATE_BELONG_STRATEGY("createBelongStrategy", "至少新建一个归属策略", 3),
    CREATE_INTEGRAL_STRATEGY("createIntegralStrategy", "至少新建一个积分策略", 3),
    CONFIG_USER_MERGE_STRATEGY("configUserMergeStrategy", "配置并保存用户合并策略", 3),
    BIND_SERVICE_NUMBER("bindServiceNumber", "至少绑定一个服务号", 5),
    PERFECT_AUTO_REPLY_CONFIG("perfectAutoReplyConfig", "完善自动回复配置", 5),
    PERFECT_MENU_CONFIG("perfectMenuConfig", "完善自定义菜单配置", 5),
    BIND_WECHAT("bindWeChat", "至少绑定一个小程序", 4),
    PERFECT_WECHAT_PAGE("perfectWeChatPage", "完善小程序页面装修", 4),
    SHORT_MESSAGE_SERVICE_CONFIG("shortMessageServiceConfig", "完成短信业务配置", 1),
    COMPLETE_STORE_CONFIG("completeStoreConfig", "完成门店基础配置", 2),
    CREATE_STORE_REGION("createStoreRegion", "至少新建一个门店域", 2),
    SET_STORE_STATUS_AND_TYPE("setStoreStatusAndType", "配置本商户需要的门店状态、门店类型", 2),
    CREATE_STORE_BRAND("createStoreBrand", "至少新建一个店招品牌", 2),
    CREATE_STORE_STATUS_STRATEGY("createStoreStatusStrategy", "至少新建一个门店启用状态策略", 2),
    COMPLETE_MEMBER_CARD_CONFIG("completeMemberCardConfig", "完成会员卡配置", 3),
    COMPLETE_SERVICE_NUMBER_CONFIG("completeServiceNumberConfig", "完成服务号配置", 5),
    COMPLETE_WECHAT_CONFIG("completeWeChatConfig", "完成小程序配置", 4),
    COMPLETE_DATA_COUNT_CONFIG("completeDataCountConfig", "完成数据统计配置", 6),
    CONFIG_EFFECTIVE_ORDER("configEffectiveOrder", "完成有效订单的配置", 6),
    CONFIG_PAY("configPay", "完成实付的配置", 6);

    private String code;
    private String message;
    private int type;

    private EnterpriseInitCodeEnum(String code, String message, int type) {
        this.code = code;
        this.message = message;
        this.type = type;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public int getType() {
        return type;
    }

    public void setType(int type) {
        this.type = type;
    }

    public static boolean isRightCode(String code) {
        for (EnterpriseInitCodeEnum codeEnum : values()) {
            if (isNotInclude(code) && codeEnum.getCode().equals(code)) {
                return true;
            }
        }
        return false;
    }

    private static boolean isNotInclude(String code) {
        if (EnterpriseInitCodeEnum.COMPLETE_STORE_CONFIG.getCode().equals(code)) {
            return false;
        }
        if (EnterpriseInitCodeEnum.COMPLETE_MEMBER_CARD_CONFIG.getCode().equals(code)) {
            return false;
        }
        if (EnterpriseInitCodeEnum.COMPLETE_SERVICE_NUMBER_CONFIG.getCode().equals(code)) {
            return false;
        }
        if (EnterpriseInitCodeEnum.COMPLETE_WECHAT_CONFIG.getCode().equals(code)) {
            return false;
        }
        if (EnterpriseInitCodeEnum.COMPLETE_DATA_COUNT_CONFIG.getCode().equals(code)) {
            return false;
        }
        return true;
    }

    public static List<String> getListByParentCode(String code) {
        if (isNotInclude(code)) {
            return new ArrayList<>();
        }
        int type = 0;
        for (EnterpriseInitCodeEnum codeEnum : values()) {
            if (codeEnum.getCode().equals(code)) {
                type = codeEnum.getType();
                break;
            }
        }
        List<String> result = new ArrayList<>();
        for (EnterpriseInitCodeEnum codeEnum : values()) {
            if (!codeEnum.getCode().equals(code) && codeEnum.getType() == type) {
                result.add(codeEnum.getCode());
            }
        }
        return result;
    }
}
