package com.gic.enterprise.service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.BillingAccountDTO;

import java.util.Map;

/**
 * 计费中心账户
 * @ClassName: BillingAccountApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/8/16 3:16 PM 
 */
public interface BillingAccountApiService {

    /**
     * 根据商户ID查询余额账户
     * @Title: getByEnterpriseId 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.BillingAccountDTO> 
      */
    ServiceResponse<BillingAccountDTO> getByEnterpriseId(Integer enterpriseId);

    /**
     * 关闭自动续订短信套餐包配置
     * @Title: closeAutoRechargeConfig 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> closeAutoRechargeConfig(Integer enterpriseId);

    /**
     * 自动续订设置
     * @Title: setAutoRecharge 
     * @Description:
      * @author guojuxing
     * @param smsCode
     * @param preNumber 
     * @param enterpriseId
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> setAutoRecharge(String smsCode, Integer preNumber, Integer enterpriseId);
    /**
     * 获取自动续订短信套餐包配置
     * @Title: getAutoRechargeConfig 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.Map<java.lang.String,java.lang.Object>> 
      */
    ServiceResponse<Map<String, Object>> getAutoRechargeConfig(Integer enterpriseId);

    /**
     * 自动续订短信套餐包定时任务
     * @Title: rechargeSmsPackageTimingTask 
     * @Description:
     * @param param
      * @author guojuxing  
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Double> 
      */
    ServiceResponse<Void> rechargeSmsPackageTimingTask(String param);

    /**
     * 账户充值后回调
     * @Title: updateAccountAfterRecharge
     * @Description:
     * @author zhiwj
     * @param totalFee
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> updateAccountAfterRecharge(double totalFee, Integer enterpriseId);

    /**
     * 支付完成后对余额扣费
     * @Title: deductAccount
     * @Description:
     * @author zhiwj
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> deductAccount(double totalFee, Integer enterpriseId);

    /**
     * 余额欠费提醒发短信
     * @Title: balanceTask 
     * @Description:
     * @author guojuxing
     * @param param 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> balanceTask(String param);
}
