package com.gic.enterprise.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.BillingOrderDTO;
import com.gic.enterprise.dto.BillingPayInfoDTO;
import com.gic.enterprise.dto.SmsPackageOrderItemDTO;

/**
 * 订单
 * @ClassName: OrderApiService
 * @Description:
 * @author zhiwj
 * @date 2019/8/14 15:27
 */
public interface BillingOrderApiService {

    /**
     * 添加一个订单，立即购买功能
     * @Title: addOrder
     * @Description:
     * @author zhiwj
     * @param billingOrderDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> saveSmsOrderBuyNow(BillingOrderDTO billingOrderDTO, SmsPackageOrderItemDTO smsPackageOrderItemDTO);

    ServiceResponse<BillingPayInfoDTO> paySmsPackageOrder(Integer enterpriseId, Integer orderId, Integer payType, String authCode, String userName);

    /**
     * 条件分页查询订单状态
     * @Title: listOrder
     * @Description: 
     * @author zhiwj 
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param orderStatus 订单状态
     * @param search 订单信息
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.enterprise.dto.BillingOrderDTO>>
     * @throws 
     */
    ServiceResponse<Page<BillingOrderDTO>> listOrder(String startTime, String endTime, Integer orderStatus,
                                                     String search, int pageNum, int pageSize, Integer enterpriseId);

    /**
     * 通过id查询订单详情
     * @Title: getOrderById
     * @Description: 
     * @author zhiwj 
     * @param orderId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.BillingOrderDTO>
     * @throws 
     */
    ServiceResponse<BillingOrderDTO> getOrderById(Integer orderId);

    /**
     * 通过流水号查询订单详情
     * @Title: getOrderBySerialNumber
     * @Description: 
     * @author zhiwj 
     * @param serialNumber
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.BillingOrderDTO>
     * @throws 
     */
    ServiceResponse<BillingOrderDTO> getOrderBySerialNumber(String serialNumber);
    
    /**
     * 取消申请
     * @Title: cancelOrderApproval
     * @Description: 
     * @author zhiwj 
     * @param orderId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws 
     */
    ServiceResponse<Integer> cancelOrderApproval(Integer orderId);
}
