package com.gic.enterprise.service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.data.*;

import java.util.List;

public interface DataConfigApiService {
    /**
     * 会员消费配置
     * @Title: configMemberConsumeConfig 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     */
    ServiceResponse<Integer> configMemberConsume(DataMemberConsumeConfigDTO dto);

    /**
     * 会员消费配置信息
     * @Title: getMemberConsumeConfigByEnterpriseId 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.data.DataMemberConsumeConfigDTO> 
     */
    ServiceResponse<DataMemberConsumeConfigDTO> getMemberConsumeConfigByEnterpriseId(Integer enterpriseId);

    /**
     * 新增商户初始化会员消费配置
     * @Title: initMemberConsumeConfig 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> initMemberConsumeConfig(Integer enterpriseId);

    /**
     * 新增
     * @Title: configActuallyPaid 
     * @Description:
     * @author guojuxing
     * @param dtoList 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> configActuallyPaid(List<DataActuallyPaidConfigDTO> dtoList);

    /**
     * 实付配置信息
     * @Title: listActuallyPaidConfig 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.enterprise.dto.data.DataActuallyPaidConfigDTO>> 
     */
    ServiceResponse<List<DataActuallyPaidConfigDTO>> listActuallyPaidConfig(Integer enterpriseId);

    /**
     * 新增商户的时候设置实付配置默认数据
     * @Title: initActuallyPaidData 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> initActuallyPaidData(Integer enterpriseId);

    /**
     * 单效取值配置
     * @Title: configSingleEffect 
     * @Description:
     * @author guojuxing
     * @param dtoList
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> configSingleEffect(List<DataSingleEffectConfigDTO> dtoList);

    /**
     * 获取单效取值配置
     * @Title: getSingleEffectConfig 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.data.DataSingleEffectConfigDTO> 
     */
    ServiceResponse<List<DataSingleEffectConfigDTO>> getSingleEffectConfig(Integer enterpriseId);

    /**
     * 新增商户的时候设置单效取值配置默认数据
     * @Title: initSingleEffectConfig 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> initSingleEffectConfig(Integer enterpriseId);

    /**
     * 业绩指标配置
     * @Title: configPerformance 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> configPerformance(DataPerformanceConfigDTO dto);

    /**
     * 获取业绩指标配置
     * @Title: getPerformanceConfig 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.data.DataPerformanceConfigDTO> 
     */
    ServiceResponse<DataPerformanceConfigDTO> getPerformanceConfig(Integer enterpriseId);

    /**
     * 新增商户的时候设置业绩指标配置默认数据
     * @Title: initPerformanceConfig 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> initPerformanceConfig(Integer enterpriseId);

    /**
     * 年龄段配置
     * @Title: saveGenerationConfig 
     * @Description:
     * @author guojuxing
     * @param dtoList 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> saveGenerationConfig(List<DataGenerationConfigDTO> dtoList);

    /**
     * 查询年龄段配置信息
     * @Title: listGenerationConfig 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.enterprise.dto.data.DataGenerationConfigDTO>> 
     */
    ServiceResponse<List<DataGenerationConfigDTO>> listGenerationConfig(Integer enterpriseId);

    /**
     * 新增商户初始化年龄段配置数据
     * @Title: initGenerationConfig 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> initGenerationConfig(Integer enterpriseId);

}
