package com.gic.enterprise.service;

import java.util.List;
import java.util.Map;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.EnterpriseLicenseDTO;
import com.gic.enterprise.dto.EnterpriseListDTO;
import com.gic.enterprise.dto.EnterpriseResourceRelDTO;
import com.gic.enterprise.dto.market.MarketServiceDTO;
import com.gic.enterprise.dto.market.OrderGicDTO;
import com.gic.enterprise.qo.BuyForGicQO;
import com.gic.enterprise.qo.EnterpriseQO;
import com.gic.enterprise.qo.OrderServiceQO;

/**
 * 商户相关操作
 * @ClassName: EnterpriseApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/8/8 4:39 PM 
 */
public interface EnterpriseApiService {

    /**
     * 新增商户和超级管理员
     * @Title: saveEnterprise 
     * @Description:
      * @author guojuxing
     * @param enterpriseDTO
     * @param
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> saveEnterprise(EnterpriseDTO enterpriseDTO);


    /**
     * 编辑保存
     * @Title: editEnterprise 
     * @Description:
      * @author guojuxing
     * @param enterpriseDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> editEnterprise(EnterpriseDTO enterpriseDTO);

    ServiceResponse<String> editResourceSwitch(Integer enterpriseId, String resources);

    /**
     * 前置资源字典数据
     * @Title: getResourceDict 
     * @Description:
     * @author guojuxing  
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.Map<java.lang.String,java.lang.String>> 
     */
    ServiceResponse<Map<String, String>> getResourceDict();

    /**
     * 修改商户信息
     * @Title: editEnterpriseInfo 
     * @Description:
      * @author guojuxing
     * @param enterpriseDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> editEnterpriseInfo(EnterpriseDTO enterpriseDTO);

    /**
     * 修改商户logo
     * @Title: modifyEnterpriseLogo 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 商户ID
     * @param logoUrl logo的图片地址
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
     */
    ServiceResponse<String> modifyEnterpriseLogo(Integer enterpriseId, String logoUrl);

    /**
     * 查询单条数据
     * @Title: getEnterpriseById 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.EnterpriseDTO> 
      */
    ServiceResponse<EnterpriseDTO> getEnterpriseById(Integer enterpriseId);

    /**
     * 获取商户前置资源开关数据
     * @Title: getBusinessFrontRes 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.enterprise.dto.EnterpriseResourceRelDTO>> 
     */
    ServiceResponse<List<EnterpriseResourceRelDTO>> getBusinessFrontRes(Integer enterpriseId);

    /**
     * 商户停用
     * @Title: disableEnterprise 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse 
      */
    ServiceResponse disableEnterprise(Integer enterpriseId);

    /**
     * 商户启用
     * @Title: enableEnterprise 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse 
      */
    ServiceResponse enableEnterprise(Integer enterpriseId);

    /**
     * 分页查询商户信息
     * @Title: listEnterprise 
     * @Description:
      * @author guojuxing
     * @param search 公司名称/品牌名称
     * @param serviceVersion 服务版本 versionCode
     * @param status 0无效，1有效，2停用，3逾期 4：即将到期
     * @param orderBy  1:创建时间正序 2:创建时间倒叙 3:到期时间正序 4:到期时间倒叙  用EnterpriseListOrderByEnum.getOrderBy(orderBy)解析成sql，可以为null
     * @param pageNum 第几页
     * @param pageSize  每页多少条
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.enterprise.dto.EnterpriseListDTO>> 
      */
    @Deprecated
    ServiceResponse<Page<EnterpriseListDTO>> listEnterprise(String search, Integer serviceVersion,
                                                                Integer status, String orderBy,
                                                                Integer pageNum, Integer pageSize);

    /**
     * 最新版本的查询接口
     * @Title: listEnterprise 
     * @Description:
     * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.enterprise.dto.EnterpriseListDTO>> 
     */
    ServiceResponse<Page<EnterpriseListDTO>> listEnterprise(EnterpriseQO params);

    /**
     * 模糊查询商户列表
     * @Title: listEnterpriseByName 
     * @Description:
      * @author guojuxing
     * @param search 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.enterprise.dto.EnterpriseDTO>> 
      */
    ServiceResponse<List<EnterpriseDTO>> listEnterpriseByName(String search);

    /**
     * 商户初始化标志改成已初始化状态
     * @Title: initEnterprise 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse 
      */
    ServiceResponse initEnterprise(Integer enterpriseId);

    ServiceResponse<List<EnterpriseDTO>> listEnterpriseByIds(List<Integer> enterpriseIdList);

    ServiceResponse<EnterpriseDTO> getEnterpriseByName(String enterpriseName);

    ServiceResponse<List<EnterpriseDTO>> listEnterpriseByPhoneNumber(String phoneNumber);

    /**
     * @Title: buyForGicForNew 
     * @Description: 服务订购
      * @author guojuxing
     * @param basicParams
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<MarketServiceDTO> buyForGicForNew(BuyForGicQO basicParams);

    /**
     * 商户license信息
     * @Title: listEnterpriseLicense 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.EnterpriseLicenseDTO> 
      */
    ServiceResponse<List<EnterpriseLicenseDTO>> listEnterpriseLicense(Integer enterpriseId);

    /**
     * 服务订购列表数据
     * @Title: listOrder 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Object> 
      */
    ServiceResponse<Page<OrderGicDTO>> listOrder(OrderServiceQO params);

    /**
     * 取消订单
     * @Title: cancelOrder 
     * @Description:
      * @author guojuxing
     * @param orderId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> cancelOrder(Long orderId);

    /**
     * 服务订购成功，改变商户状态
     * @Title: buyServiceSuccess 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @param json ApplicationTokenDTO
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> buyServiceSuccess(Integer enterpriseId, String json);

    
    /** @Description: 商户过期提醒
     * @author taogs
     * @Date 17:13 2020/3/9
     * @Param 
     * @return 
    */
    ServiceResponse<Void> overdueReminder(String param);

    /**
     * 商户基础服务到期提醒
     * @Title: expireReminder 
     * @Description:
     * @author guojuxing
     * @param param 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> expireReminder(String param);
}
