package com.gic.enterprise.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.wm.WmStoreSyncLogDTO;

import java.util.List;

public interface WmStoreSyncLogApiService {

    /**
     * 新增微盟同步日志
     * @Title: saveWmStoreSyncLog 
     * @Description: 
     * @author guojuxing 
     * @param storeIdList 门店
     * @param wmMallStoreId 微盟店铺ID
     * @param type 1：总店 2：门店同步（一对一）
     * @param enterpriseId 商户ID
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> saveWmStoreSyncLog(List<Integer> storeIdList, Integer wmMallStoreId,
                                             Integer type, Integer enterpriseId);

    /**
     * 分页查询门店同步日志
     * @Title: pageWmStoreSyncLog 
     * @Description:
     * @author guojuxing
     * @param search 门店code/名称
     * @param type 1：总店 2：门店同步（一对一）
     * @param wmMallStoreId 店铺ID
     * @param pageNum 第几页
     * @param pageSize 每页多少条
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.enterprise.dto.wm.WmStoreSyncLogDTO>> 
     */
    ServiceResponse<Page<WmStoreSyncLogDTO>> pageWmStoreSyncLog(String search, Integer type, Integer wmMallStoreId,
                                                                Integer pageNum, Integer pageSize);

    /**
     * 已经同步过的门店
     * @Title: getStoreList 
     * @Description:
     * @author guojuxing
     * @param type
     * @param wmMallStoreId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.enterprise.dto.wm.WmStoreSyncLogDTO>> 
     */
    ServiceResponse<List<WmStoreSyncLogDTO>> getStoreList(List<Integer> type, Integer wmMallStoreId);

    /**
     * 解除门店关联日志
     * @Title: deleteLogic 
     * @Description:
     * @author guojuxing
     * @param storeId
     * @param wmMallStoreId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> deleteLogic(Integer storeId, Integer wmMallStoreId);

    /**
     * 查询所有店铺
     * @Title: listByStoreId 
     * @Description:
     * @author guojuxing
     * @param storeId
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.enterprise.dto.wm.WmStoreSyncLogDTO>> 
     */
    ServiceResponse<List<WmStoreSyncLogDTO>> listByStoreId(Integer storeId, Integer enterpriseId);

    ServiceResponse<List<WmStoreSyncLogDTO>> listByStoreId(List<Integer> storeIdList, Integer wmMallStoreId);
}
