package com.gic.enterprise.dao.mapper;

import java.util.List;

import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.qo.EnterpriseQO;
import org.apache.ibatis.annotations.Param;

import com.gic.enterprise.dto.EnterpriseListDTO;
import com.gic.enterprise.entity.TabEnterprise;

public interface TabEnterpriseMapper {
    /**
     * 根据主键删除
     *
     * @param enterpriseId 主键
     * @return 更新条目数
     */
    int deleteByPrimaryKey(Integer enterpriseId);

    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabEnterprise record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabEnterprise record);

    /**
     * 根据主键查询
     *
     * @param enterpriseId 主键
     * @return 实体对象
     */
    TabEnterprise selectByPrimaryKey(Integer enterpriseId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabEnterprise record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabEnterprise record);

    /**
     * 根据商户名称相同的个数
     * @Title: countByName 
     * @Description:
      * @author guojuxing
     * @param enterpriseName
     * @param enterpriseId 
     * @return  int 
      */
    int countByEnterpriseName(@Param("enterpriseName") String enterpriseName,
                              @Param("enterpriseId") Integer enterpriseId);

    /**
     * 根据公司名称相同的个数
     * @Title: countByName 
     * @Description:
      * @author guojuxing
     * @param companyName
     * @param enterpriseId 
     * @return  int 
      */
    int countByCompanyName(@Param("companyName") String companyName,
                              @Param("enterpriseId") Integer enterpriseId);

    /**
     * 查询列表
     * @param search 商户名称或者公司名称
     * @param serviceVersion
     * @param serviceStatus
     * @return
     */
    List<EnterpriseListDTO> listEnterpriseId(@Param("search") String search,
                                             @Param("serviceVersion") Integer serviceVersion,
                                             @Param("serviceStatus") Integer serviceStatus,
                                             @Param("orderBy") String orderBy);


    /**
     * 查询列表最新
     * @Title: listEnterpriseIdOfNew 
     * @Description:
     * @author guojuxing
     * @param params 
     * @return  java.util.List<com.gic.enterprise.dto.EnterpriseListDTO> 
     */
    List<EnterpriseListDTO> listEnterpriseIdOfNew(EnterpriseQO params);

    /**
     * 商户名称/公司名称模糊查询商户
     * @Title: listEnterpriseByName 
     * @Description:
      * @author guojuxing
     * @param search 
     * @return  java.util.List<com.gic.enterprise.dto.EnterpriseDTO> 
      */
    List<TabEnterprise> listEnterpriseByName(@Param("search")String search);


    /**
     * 查询商户列表数据
     * @Title: listEnterprsie 
     * @Description:
      * @author guojuxing  
     * @return  java.util.List<com.gic.enterprise.dto.EnterpriseListDTO> 
      */
    List<TabEnterprise> listEnterprise();

    List<TabEnterprise> listEnterpriseByIds(@Param("enterpriseIdList") List<Integer> enterpriseIdlist);

    List<TabEnterprise> getEnterpriseByName(@Param("enterpriseName") String enterpriseName);
}