package com.gic.enterprise.service;

import java.util.List;

import com.gic.enterprise.dto.rule.RuleEnterpriseDTO;
import com.gic.enterprise.entity.TabConfigRuleEnterprise;
import com.gic.enterprise.entity.TabConfigRuleEnterpriseRel;
import com.gic.enterprise.entity.TabConfigRuleRel;
import com.github.pagehelper.Page;

public interface ConfigRuleEnterpriseService {

    /**
     * 新增商户规则
     * @Title: save 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  java.lang.Integer 
     */
    Integer save(RuleEnterpriseDTO dto);

    void update(TabConfigRuleEnterprise record);

    TabConfigRuleEnterprise getByEnterpriseRuleId(Integer enterpriseRuleId);

    /**
     * 是否重复名称
     * @Title: isRepeatName 
     * @Description:
     * @author guojuxing
     * @param enterpriseRuleId
     * @param ruleName 
     * @return  boolean 
     */
    boolean isRepeatName(Integer enterpriseRuleId, String ruleName);

    /**
     * 批量新增商户和规则关联数据
     * @Title: insertBatch 
     * @Description:
     * @author guojuxing
     * @param list 
     * @return  void 
     */
    void insertBatch(List<TabConfigRuleEnterpriseRel> list);

    /**
     * 解绑规则的旧关联
     * @Title: deleteByEnterpriseRuleId 
     * @Description:
     * @author guojuxing
     * @param enterpriseRuleId 
     * @return  void 
     */
    void deleteByEnterpriseRuleId(Integer enterpriseRuleId);

    /**
     * 解绑商户之前的旧关联
     * @Title: deleteByEnterpriseIdList 
     * @Description:
     * @author guojuxing
     * @param enterpriseIdList 
     * @return  void 
     */
    void deleteByEnterpriseIdList(List<Integer> enterpriseIdList);

    /**
     * 解绑规则树和商户规则的关联
     * @Title: deleteByEnterpriseRuleIdForRuleRel 
     * @Description:
     * @author guojuxing
     * @param enterpriseRuleId 
     * @return  void 
     */
    void deleteByEnterpriseRuleIdForRuleRel(Integer enterpriseRuleId);

    /**
     * 批量新增规则树和商户规则的关联
     * @Title: insertBatchForRuleRel 
     * @Description:
     * @author guojuxing
     * @param list 
     * @return  void 
     */
    void insertBatchForRuleRel(List<TabConfigRuleRel> list);

    /**
     * 规则列表
     * @Title: listRule 
     * @Description:
     * @author guojuxing  
     * @param search 规则名称/商户名称/公司名称
     * @param pageNum
     * @param pageSize
     * @return  java.util.List<com.gic.enterprise.dto.rule.RuleEnterpriseDTO> 
     */
    Page<RuleEnterpriseDTO> listRule(String search, Integer pageNum, Integer pageSize);
    /**
     * 商户列表
     * @Title: listEnterpriseRule 
     * @Description:
     * @author guojuxing  
     * @param search 商户名称/公司名称
     * @param enterpriseRuleId 商户规则ID
     * @param pageNum
     * @param pageSize
     * @return  java.util.List<com.gic.enterprise.dto.rule.RuleEnterpriseDTO> 
     */
    Page<RuleEnterpriseDTO> listEnterpriseRule(String search, Integer enterpriseRuleId, Integer pageNum, Integer pageSize);

    /**
     * 查询规则树数据
     * @Title: listRuleRel 
     * @Description:
     * @author guojuxing
     * @param enterpriseRuleId 
     * @return  java.util.List<com.gic.enterprise.entity.TabConfigRuleRel> 
     */
    List<TabConfigRuleRel> listRuleRel(Integer enterpriseRuleId);

    /**
     * 查询所有的商户及所在规则
     * @Title: listEnterprise 
     * @Description:
     * @author guojuxing  
     * @return  java.util.List<com.gic.enterprise.entity.TabConfigRuleEnterpriseRel> 
     */
    List<TabConfigRuleEnterpriseRel> listEnterprise();

    /**
     * 查询所有的商户规则
     * @Title: listEnterpriseRule 
     * @Description:
     * @author guojuxing  
     * @return  java.util.List<com.gic.enterprise.dto.rule.RuleEnterpriseDTO> 
     */
    List<TabConfigRuleEnterprise> listEnterpriseRule();

    /**
     * 查询商户配置的规则
     * @Title: getByEnterpriseId 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.enterprise.entity.TabConfigRuleEnterpriseRel 
     */
    TabConfigRuleEnterpriseRel getByEnterpriseId(Integer enterpriseId);
}
