package com.gic.enterprise.service;

import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.EnterpriseListDTO;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.qo.EnterpriseQO;
import com.github.pagehelper.Page;

import java.util.List;

/**
 * 商户
 * @ClassName: EnterpriseService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/8/8 4:43 PM 
 */
public interface EnterpriseService {

    /**
     * 新建商户
     * @Title: saveEnterprise 
     * @Description:
      * @author guojuxing
     * @param enterpriseDTO 
     * @return  int 
      */
    int saveEnterprise(EnterpriseDTO enterpriseDTO);

    /**
     * 编辑保存
     * @Title: editEnterprise 
     * @Description:
      * @author guojuxing
     * @param enterpriseDTO 
     * @return  void 
      */
    void editEnterprise(EnterpriseDTO enterpriseDTO);

    /**
     * 根据主键获取
     * @Title: getEnterpriseById 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.enterprise.entity.TabEnterprise 
      */
    TabEnterprise getEnterpriseById(Integer enterpriseId);

    /**
     * 商户名称是否重复
     * @Title: isRepeatEnterpriseName 
     * @Description:
      * @author guojuxing
     * @param enterpriseName
     * @param enterpriseId 
     * @return  boolean 
      */
    boolean isRepeatEnterpriseName(String enterpriseName, Integer enterpriseId);

    /**
     * 公司名称是否重复
     * @Title: isRepeatEnterpriseName 
     * @Description:
      * @author guojuxing
     * @param companyName
     * @param enterpriseId 
     * @return  boolean 
      */
    boolean isRepeatCompanyName(String companyName, Integer enterpriseId);

    /**
     * 商户启用、停用
     * @Title: changeEnterpriseStatus 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param status 
     * @return  void 
      */
    void changeEnterpriseStatus(Integer enterpriseId, Integer status);

    /**
     * 分页查询商户
     * @Title: listEnterprise 
     * @Description:
      * @author guojuxing
     * @param search
     * @param serviceVersion
     * @param status
     * @param orderBy
     * @param pageNum
     * @param pageSize 
     * @return  com.github.pagehelper.Page<com.gic.enterprise.dto.EnterpriseListDTO> 
      */
    Page<EnterpriseListDTO> listEnterprise(String search, Integer serviceVersion,
                                           Integer status, String orderBy,
                                           Integer pageNum, Integer pageSize);

    /**
     * 最新版本的查询
     * @Title: listEnterprise 
     * @Description:
     * @author guojuxing
     * @param params 
     * @return  com.github.pagehelper.Page<com.gic.enterprise.dto.EnterpriseListDTO> 
     */
    Page<EnterpriseListDTO> listEnterprise(EnterpriseQO params);

    /**
     * 商户名称/公司名称模糊查询
     * @Title: listEnterpriseByName 
     * @Description:
      * @author guojuxing
     * @param search 
     * @return  java.util.List<com.gic.enterprise.dto.EnterpriseDTO> 
      */
    List<EnterpriseDTO> listEnterpriseByName(String search);

    /**
     * 查询所有的商户
     * @Title: listEnterprise 
     * @Description:
      * @author guojuxing  
     * @return  java.util.List<com.gic.enterprise.dto.EnterpriseDTO> 
      */
    List<EnterpriseDTO> listEnterprise();

    /**
     * 商户初始化字段改为已初始化状态
     * @Title: initEnterprise 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  void 
      */
    void initEnterprise(Integer enterpriseId);

    List<TabEnterprise> listEnterpriseByIds(List<Integer> enterpriseIdList);

    TabEnterprise getEnterpriseByName(String enterpriseName);
}
