package com.gic.enterprise.service.impl;

import java.util.Date;
import java.util.List;

import com.gic.enterprise.constant.applet.AppletPageTypeEnum;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.dao.mapper.TabAppletCustomPageMapper;
import com.gic.enterprise.dto.AppletCustomPageDTO;
import com.gic.enterprise.entity.TabAppletCustomPage;
import com.gic.enterprise.qo.AppletPageQO;
import com.gic.enterprise.service.AppletCustomPageService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;

@Service("appletCustomPageService")
public class AppletCustomPageServiceImpl implements AppletCustomPageService{
    @Autowired
    private TabAppletCustomPageMapper tabAppletCustomPageMapper;

    @Override
    public Integer save(AppletCustomPageDTO dto) {
        dto.setCreateTime(new Date());
        dto.setUpdateTime(new Date());
        TabAppletCustomPage record = EntityUtil.changeEntityNew(TabAppletCustomPage.class, dto);
        tabAppletCustomPageMapper.insert(record);
        return record.getPageId();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Integer save(TabAppletCustomPage record) {
        record.setCreateTime(new Date());
        record.setUpdateTime(new Date());
        tabAppletCustomPageMapper.insert(record);
        return record.getPageId();
    }

    @Override
    public void edit(TabAppletCustomPage record) {
        record.setUpdateTime(new Date());
        tabAppletCustomPageMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public void updateStatus(Integer pageId, Integer status) {
        TabAppletCustomPage record = new TabAppletCustomPage();
        record.setPageId(pageId);
        record.setStatus(status);
        tabAppletCustomPageMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public Page<TabAppletCustomPage> page(AppletPageQO params) {
        PageHelper.startPage(params.getCurrentPage(), params.getPageSize());
        List<TabAppletCustomPage> list = tabAppletCustomPageMapper.listByParams(params);
        return (Page<TabAppletCustomPage>) list;
    }

    @Override
    public TabAppletCustomPage getByPageId(Integer pageId) {
        return tabAppletCustomPageMapper.selectByPrimaryKey(pageId);
    }

    @Override
    public boolean hasCreateMemberCenterPage(Integer enterpriseId) {
        AppletPageQO pageQO = new AppletPageQO();
        pageQO.setEnterpriseId(enterpriseId);
        pageQO.setPageType(AppletPageTypeEnum.MEMBER_CENTER.getCode());
        List<TabAppletCustomPage> list = tabAppletCustomPageMapper.listByParams(pageQO);
        if (CollectionUtils.isNotEmpty(list)) {
            for (TabAppletCustomPage temp : list) {
                if (temp.getStatus().intValue() == 1) {
                    return true;
                }
            }
        }
        return false;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int disableFetch(Integer enterpriseId, Integer appType, String appId, Integer pageType) {
        return tabAppletCustomPageMapper.disableFetch(enterpriseId, appType, appId, pageType);
    }

    @Override
    public List<TabAppletCustomPage> listEnablePage(Integer enterpriseId, Integer appType, String appId, Integer pageType) {
        return tabAppletCustomPageMapper.listEnablePage(enterpriseId, appType, appId, pageType);
    }

    @Override
    public int count(Integer enterpriseId, Integer appType, String appId, Integer pageType) {
        return tabAppletCustomPageMapper.count(enterpriseId, appType, appId, pageType);
    }

    @Override
    public TabAppletCustomPage getDefault(Integer pageType) {
        return tabAppletCustomPageMapper.getDefault(pageType);
    }

    @Override
    public void deleteFetch(List<Integer> pageIdList) {
        tabAppletCustomPageMapper.deleteFetch(pageIdList);
    }
}
