package com.gic.enterprise.service.impl;

import com.alibaba.fastjson.JSON;
import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.dao.mapper.TabBillingSmsPackageRecordMapper;
import com.gic.enterprise.dto.BillingOrderDTO;
import com.gic.enterprise.dto.HomeStatisticsDTO;
import com.gic.enterprise.dto.SmsPackageOrderItemDTO;
import com.gic.enterprise.entity.TabBillingSmsPackageRecord;
import com.gic.enterprise.qo.HomeStatisticsQO;
import com.gic.enterprise.qo.RecordQO;
import com.gic.enterprise.service.BillingSmsPackageRecordService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service("billingSmsPackageRecordService")
public class BillingSmsPackageRecordServiceImpl implements BillingSmsPackageRecordService{
    @Autowired
    private TabBillingSmsPackageRecordMapper tabBillingSmsPackageRecordMapper;
    @Override
    public List<HomeStatisticsDTO> listHomeStatistics(HomeStatisticsQO params) {
        List<HomeStatisticsDTO> list = tabBillingSmsPackageRecordMapper.listHomeStatistics(params);
        if (CollectionUtils.isNotEmpty(list)) {
            return list;
        }
        return new ArrayList<>();
    }

    @Override
    public double getFee(Integer enterpriseId, Integer dateType) {
        return tabBillingSmsPackageRecordMapper.getFee(enterpriseId, dateType);
    }

    @Override
    public Integer saveSmsPackageRecord(BillingOrderDTO billingOrderDTO) {
        TabBillingSmsPackageRecord packageRecord = new TabBillingSmsPackageRecord();
        packageRecord.setProductName(billingOrderDTO.getOrderContent());
        SmsPackageOrderItemDTO smsPackageOrderItemDTO = JSON.parseObject(billingOrderDTO.getItemJson(), SmsPackageOrderItemDTO.class);
        packageRecord.setItemCount(smsPackageOrderItemDTO.getBuyCount());
        packageRecord.setItemFee(new BigDecimal(smsPackageOrderItemDTO.getCost() + "").multiply(new BigDecimal(smsPackageOrderItemDTO.getSmsAmount() + "")).doubleValue());
        packageRecord.setTotalFee(billingOrderDTO.getTotalFee());
        packageRecord.setPayType(billingOrderDTO.getPayType());
        packageRecord.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        packageRecord.setOrderId(billingOrderDTO.getOrderId());
        packageRecord.setDeductionTime(new Date());
        packageRecord.setEnterpriseId(billingOrderDTO.getEnterpriseId());

        packageRecord.setCardName(billingOrderDTO.getCouponCardName());
        packageRecord.setCardAmount(billingOrderDTO.getCouponCardContent());

        tabBillingSmsPackageRecordMapper.insertSelective(packageRecord);
        return packageRecord.getSmsPackageRecordId();
    }

    @Override
    public Page<TabBillingSmsPackageRecord> listSmsPackageRecord(RecordQO recordQO) {
        PageHelper.startPage(recordQO.getCurrentPage(), recordQO.getPageSize());
        return this.tabBillingSmsPackageRecordMapper.listSmsPackageRecord(recordQO);
    }

    @Override
    public Integer countSmsPackageRecord(RecordQO recordQO) {
        return tabBillingSmsPackageRecordMapper.countSmsPackageRecord(recordQO);
    }

    @Override
    public Double getTotalFee(Integer enterpriseId, String startTime, String endTime) {
        return tabBillingSmsPackageRecordMapper.getTotalFee(enterpriseId, startTime, endTime);
    }
}
