package com.gic.enterprise.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.dao.mapper.TabBillingSmsRecordMapper;
import com.gic.enterprise.dto.HomeStatisticsDTO;
import com.gic.enterprise.dto.SmsRecordDTO;
import com.gic.enterprise.entity.TabBillingSmsRecord;
import com.gic.enterprise.qo.HomeStatisticsQO;
import com.gic.enterprise.qo.RecordQO;
import com.gic.enterprise.service.BillingSmsRecordService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service("billingSmsRecordService")
public class BillingSmsRecordServiceImpl implements BillingSmsRecordService{
    @Autowired
    private TabBillingSmsRecordMapper tabBillingSmsRecordMapper;
    @Override
    public List<HomeStatisticsDTO> listHomeStatistics(HomeStatisticsQO params) {
        List<HomeStatisticsDTO> list = tabBillingSmsRecordMapper.listHomeStatistics(params);
        if (CollectionUtils.isNotEmpty(list)) {
            return list;
        }
        return new ArrayList<>();
    }

    @Override
    public double getFee(Integer enterpriseId, Integer dateType) {
        return tabBillingSmsRecordMapper.getFee(enterpriseId, dateType);
    }

    @Override
    public Page<TabBillingSmsRecord> listSmsPackageRecord(String search, String startDate, String endDate, Integer smsPackageId) {
        return this.tabBillingSmsRecordMapper.listSmsPackageRecord(search, startDate, endDate, smsPackageId);
    }

    @Override
    public Page<TabBillingSmsRecord> listSmsRecord(RecordQO recordQO) {
        PageHelper.startPage(recordQO.getCurrentPage(), recordQO.getPageSize());
        return this.tabBillingSmsRecordMapper.listSmsRecord(recordQO);
    }

    @Override
    public Integer countSmsRecord(RecordQO recordQO) {
        return tabBillingSmsRecordMapper.countSmsRecord(recordQO);
    }

    @Override
    public Double getTotalFee(Integer enterpriseId, String startTime, String endTime) {
        return tabBillingSmsRecordMapper.getTotalFee(enterpriseId, startTime, endTime);
    }

    @Override
    public Long saveSmsRecord(SmsRecordDTO smsRecordDTO) {
        TabBillingSmsRecord billingSmsRecord = EntityUtil.changeEntityByJSON(TabBillingSmsRecord.class, smsRecordDTO);
        billingSmsRecord.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        billingSmsRecord.setCreateTime(new Date());
        billingSmsRecord.setBillNumber(1);
        tabBillingSmsRecordMapper.insertSelective(billingSmsRecord);
        return billingSmsRecord.getSmsRecordId();
    }

    @Override
    public void updateCacheStatus(Long smsRecordId) {
        TabBillingSmsRecord smsRecord = new TabBillingSmsRecord();
        smsRecord.setSmsRecordId(smsRecordId);
        smsRecord.setCacheStatus(1);
        tabBillingSmsRecordMapper.updateByPrimaryKeySelective(smsRecord);
    }

    @Override
    public void updateDatabaseStatus(SmsRecordDTO smsRecordDTO) {
        TabBillingSmsRecord smsRecord = EntityUtil.changeEntityByJSON(TabBillingSmsRecord.class, smsRecordDTO);
        // 不去修改缓存状态
        smsRecord.setCacheStatus(null);
        smsRecord.setDatabaseStatus(1);
        smsRecord.setDeductionTime(new Date());
        tabBillingSmsRecordMapper.updateByPrimaryKeySelective(smsRecord);
    }
}