package com.gic.enterprise.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.dao.mapper.TabConfigRuleEnterpriseMapper;
import com.gic.enterprise.dao.mapper.TabConfigRuleEnterpriseRelMapper;
import com.gic.enterprise.dao.mapper.TabConfigRuleRelMapper;
import com.gic.enterprise.dto.rule.RuleEnterpriseDTO;
import com.gic.enterprise.entity.TabConfigRuleEnterprise;
import com.gic.enterprise.entity.TabConfigRuleEnterpriseRel;
import com.gic.enterprise.entity.TabConfigRuleRel;
import com.gic.enterprise.service.ConfigRuleEnterpriseService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Service("configRuleEnterpriseService")
public class ConfigRuleEnterpriseServiceImpl implements ConfigRuleEnterpriseService{
    @Autowired
    private TabConfigRuleEnterpriseMapper tabConfigRuleEnterpriseMapper;
    @Autowired
    private TabConfigRuleEnterpriseRelMapper tabConfigRuleEnterpriseRelMapper;
    @Autowired
    private TabConfigRuleRelMapper tabConfigRuleRelMapper;

    @Override
    public Integer save(RuleEnterpriseDTO dto) {
        dto.setCreateTime(new Date());
        dto.setUpdateTime(new Date());
        dto.setStatus(1);
        TabConfigRuleEnterprise record = EntityUtil.changeEntityNew(TabConfigRuleEnterprise.class, dto);
        tabConfigRuleEnterpriseMapper.insert(record);
        return record.getEnterpriseRuleId();
    }

    @Override
    public void update(TabConfigRuleEnterprise record) {
        record.setUpdateTime(new Date());
        tabConfigRuleEnterpriseMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public TabConfigRuleEnterprise getByEnterpriseRuleId(Integer enterpriseRuleId) {
        return tabConfigRuleEnterpriseMapper.selectByPrimaryKey(enterpriseRuleId);
    }

    @Override
    public boolean isRepeatName(Integer enterpriseRuleId, String ruleName) {
        int count = tabConfigRuleEnterpriseMapper.countRepeatName(enterpriseRuleId, ruleName);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public void insertBatch(List<TabConfigRuleEnterpriseRel> list) {
        tabConfigRuleEnterpriseRelMapper.insertBatch(list);
    }

    @Override
    public void deleteByEnterpriseRuleId(Integer enterpriseRuleId) {
        tabConfigRuleEnterpriseRelMapper.deleteByEnterpriseRuleId(enterpriseRuleId);
    }

    @Override
    public void deleteByEnterpriseIdList(List<Integer> enterpriseIdList) {
        if (CollectionUtils.isEmpty(enterpriseIdList)) {
            return;
        }
        tabConfigRuleEnterpriseRelMapper.deleteByEnterpriseIdList(enterpriseIdList);
    }

    @Override
    public void deleteByEnterpriseRuleIdForRuleRel(Integer enterpriseRuleId) {
        tabConfigRuleRelMapper.deleteByEnterpriseRuleId(enterpriseRuleId);
    }

    @Override
    public void insertBatchForRuleRel(List<TabConfigRuleRel> list) {
        tabConfigRuleRelMapper.insertBatch(list);
    }

    @Override
    public Page<RuleEnterpriseDTO> listRule(String search, Integer pageNum, Integer pageSize) {
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        PageHelper.startPage(pageNum, pageSize);
        List<RuleEnterpriseDTO> list = tabConfigRuleEnterpriseMapper.listRule(search);
        return (Page<RuleEnterpriseDTO>) list;
    }

    @Override
    public Page<RuleEnterpriseDTO> listEnterpriseRule(String search, Integer enterpriseRuleId, Integer pageNum, Integer pageSize) {
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        PageHelper.startPage(pageNum, pageSize);
        List<RuleEnterpriseDTO> list = tabConfigRuleEnterpriseMapper.listEnterpriseRule(enterpriseRuleId, search);
        return (Page<RuleEnterpriseDTO>) list;
    }

    @Override
    public List<TabConfigRuleRel> listRuleRel(Integer enterpriseRuleId) {
        return tabConfigRuleRelMapper.listRuleRel(enterpriseRuleId);
    }

    @Override
    public List<TabConfigRuleEnterpriseRel> listEnterprise() {
        return tabConfigRuleEnterpriseRelMapper.listEnterprise();
    }

    @Override
    public List<TabConfigRuleEnterprise> listEnterpriseRule() {
        return tabConfigRuleEnterpriseMapper.listEnterpriseRuleAll();
    }

    @Override
    public TabConfigRuleEnterpriseRel getByEnterpriseId(Integer enterpriseId) {
        return tabConfigRuleEnterpriseRelMapper.getByEnterpriseId(enterpriseId);
    }
}
