package com.gic.enterprise.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.gic.enterprise.constant.EnterpriseStatusTypeEnum;
import com.gic.enterprise.qo.EnterpriseQO;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.dao.mapper.TabEnterpriseMapper;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.EnterpriseListDTO;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.service.EnterpriseService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;

/**
 * @author guojx
 * @date 2019/7/16 4:32 PM
 */
@Service("enterpriseService")
public class EnterpriseServiceImpl implements EnterpriseService{
    @Autowired
    private TabEnterpriseMapper tabEnterpriseMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int saveEnterprise(EnterpriseDTO enterpriseDTO) {
        //无服务
        enterpriseDTO.setStatus(1);
        enterpriseDTO.setServiceStatus(EnterpriseStatusTypeEnum.INVALID.getCode());
        enterpriseDTO.setCreateTime(new Date());
        enterpriseDTO.setUpdateTime(new Date());
        enterpriseDTO.setIsInitComplete(0);
        TabEnterprise tabEnterprise = EntityUtil.changeEntityNew(TabEnterprise.class, enterpriseDTO);
        tabEnterpriseMapper.insertSelective(tabEnterprise);
        return tabEnterprise.getEnterpriseId();
    }

    @Override
    public void editEnterprise(EnterpriseDTO enterpriseDTO) {
        enterpriseDTO.setUpdateTime(new Date());
        TabEnterprise tabEnterprise = EntityUtil.changeEntityNew(TabEnterprise.class, enterpriseDTO);
        tabEnterpriseMapper.updateByPrimaryKeySelective(tabEnterprise);
    }

    @Override
    public TabEnterprise getEnterpriseById(Integer enterpriseId) {
        return tabEnterpriseMapper.selectByPrimaryKey(enterpriseId);
    }

    @Override
    public boolean isRepeatEnterpriseName(String enterpriseName, Integer enterpriseId) {
        int count = tabEnterpriseMapper.countByEnterpriseName(enterpriseName, enterpriseId);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public boolean isRepeatCompanyName(String companyName, Integer enterpriseId) {
        int count = tabEnterpriseMapper.countByCompanyName(companyName, enterpriseId);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public void changeEnterpriseStatus(Integer enterpriseId, Integer status) {
        TabEnterprise tabEnterprise = new TabEnterprise();
        tabEnterprise.setEnterpriseId(enterpriseId);
        tabEnterprise.setServiceStatus(status);
        tabEnterpriseMapper.updateByPrimaryKeySelective(tabEnterprise);
    }

    @Override
    public Page<EnterpriseListDTO> listEnterprise(String search, Integer serviceVersion, Integer status,
                                                  String orderBy, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<EnterpriseListDTO> list = tabEnterpriseMapper.listEnterpriseId(search, serviceVersion, status, orderBy);
        return (Page<EnterpriseListDTO>) list;
    }

    @Override
    public Page<EnterpriseListDTO> listEnterprise(EnterpriseQO params) {
        PageHelper.startPage(params.getCurrentPage(), params.getPageSize());
        List<EnterpriseListDTO> list = tabEnterpriseMapper.listEnterpriseIdOfNew(params);
        return (Page<EnterpriseListDTO>) list;
    }

    @Override
    public List<EnterpriseDTO> listEnterpriseByName(String search) {
        List<TabEnterprise> list = tabEnterpriseMapper.listEnterpriseByName(search);
        if (CollectionUtils.isNotEmpty(list)) {
            return EntityUtil.changeEntityListNew(EnterpriseDTO.class, list);
        }
        return new ArrayList<>();
    }

    @Override
    public List<EnterpriseDTO> listEnterprise() {
        List<TabEnterprise> list = tabEnterpriseMapper.listEnterprise();
        if (CollectionUtils.isNotEmpty(list)) {
            return EntityUtil.changeEntityListNew(EnterpriseDTO.class, list);
        }
        return new ArrayList<>();
    }

    @Override
    public void initEnterprise(Integer enterpriseId) {
        TabEnterprise tabEnterprise = new TabEnterprise();
        tabEnterprise.setEnterpriseId(enterpriseId);
        tabEnterprise.setIsInitComplete(1);
        tabEnterpriseMapper.updateByPrimaryKeySelective(tabEnterprise);
    }

    @Override
    public List<TabEnterprise> listEnterpriseByIds(List<Integer> enterpriseIdList) {
        if(CollectionUtils.isEmpty(enterpriseIdList)){
            return null;
        }
        return this.tabEnterpriseMapper.listEnterpriseByIds(enterpriseIdList);
    }

    @Override
    public TabEnterprise getEnterpriseByName(String enterpriseName) {
        List<TabEnterprise> list = this.tabEnterpriseMapper.getEnterpriseByName(enterpriseName);
        if(CollectionUtils.isNotEmpty(list)){
            return list.get(0);
        }
        return null;
    }
}
