package com.gic.enterprise.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.dao.mapper.TabIndexModuleMapper;
import com.gic.enterprise.dto.IndexModuleDTO;
import com.gic.enterprise.entity.TabIndexModule;
import com.gic.enterprise.service.IndexModuleService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-13 15:56
 */
@Service
public class IndexModuleServiceImpl implements IndexModuleService {

    @Autowired
    TabIndexModuleMapper tabIndexModuleMapper;

    @Override
    public boolean validModuleNameIsRepeat(String indexModuleName) {
        TabIndexModule indexModule = new TabIndexModule();
        indexModule.setIndexModuleName(indexModuleName);
        TabIndexModule result = tabIndexModuleMapper.getBySelective(indexModule);
        return result != null;
    }

    @Override
    public boolean validModuleCodeIsRepeat(String indexModuleCode) {
        TabIndexModule indexModule = new TabIndexModule();
        indexModule.setIndexModuleCode(indexModuleCode);
        TabIndexModule result = tabIndexModuleMapper.getBySelective(indexModule);
        return result != null;
    }

    @Override
    public Integer saveIndexModule(IndexModuleDTO indexModuleDTO) {
        TabIndexModule indexModule = EntityUtil.changeEntityByJSON(TabIndexModule.class, indexModuleDTO);
        indexModule.setCreateTime(new Date());
        indexModule.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        tabIndexModuleMapper.insertSelective(indexModule);
        return indexModule.getIndexModuleId();
    }

    @Override
    public double getMinSort() {
        Double sort = tabIndexModuleMapper.getMinSort();
        return sort == null ? 0d : sort;
    }

    @Override
    public void updateIndexModule(IndexModuleDTO indexModuleDTO) {
        TabIndexModule indexModule = EntityUtil.changeEntityByJSON(TabIndexModule.class, indexModuleDTO);
        indexModule.setStatus(null);
        tabIndexModuleMapper.updateByPrimaryKeySelective(indexModule);
    }

    @Override
    public void delIndexModule(Integer indexModuleId) {
        TabIndexModule tabIndexModule = new TabIndexModule();
        tabIndexModule.setIndexModuleId(indexModuleId);
        tabIndexModule.setStatus(GlobalInfo.DATA_STATUS_DELETE);
        tabIndexModuleMapper.updateByPrimaryKeySelective(tabIndexModule);
    }

    @Override
    public List<TabIndexModule> listIndexModule() {
        return tabIndexModuleMapper.listIndexModule();
    }

    private void sortTopping(Integer indexModuleId) {
        Double sort = tabIndexModuleMapper.getMinSort();
        TabIndexModule tabIndexModule = new TabIndexModule();
        tabIndexModule.setIndexModuleId(indexModuleId);
        tabIndexModule.setSort(sort - 1d);
        tabIndexModuleMapper.updateByPrimaryKeySelective(tabIndexModule);
    }

    private void sortBottom(Integer indexModuleId) {
        Double sort = tabIndexModuleMapper.getMaxSort();
        TabIndexModule tabIndexModule = new TabIndexModule();
        tabIndexModule.setIndexModuleId(indexModuleId);
        tabIndexModule.setSort(sort + 1d);
        tabIndexModuleMapper.updateByPrimaryKeySelective(tabIndexModule);
    }

    @Override
    public void sortIndexModule(Integer indexModuleId, Integer seq) {
        if (seq <= 1) {
            // 相当于置顶
            sortTopping(indexModuleId);
        } else {
            int start = seq - 2;
            int size = 2;

            List<TabIndexModule> list = tabIndexModuleMapper.listTwoElementExceptId(start, size, indexModuleId);
            if (CollectionUtils.isEmpty(list)) {
                // 置底
                sortBottom(indexModuleId);
            } else {
                Double sort;
                if (list.size() == 1) {
                    // 置底
                    sort = list.get(0).getSort() + 1d;
                } else {
                    sort = (list.get(0).getSort() + list.get(1).getSort()) / 2d;
                }
                TabIndexModule tabIndexModule = new TabIndexModule();
                tabIndexModule.setIndexModuleId(indexModuleId);
                tabIndexModule.setSort(sort);
                tabIndexModuleMapper.updateByPrimaryKeySelective(tabIndexModule);
            }
        }
    }

    @Override
    public List<TabIndexModule> listIndexModuleEnterprise(Integer enterpriseId) {
        return tabIndexModuleMapper.listIndexModuleEnterprise(enterpriseId);
    }

    @Override
    public TabIndexModule getIndexModule(Integer indexModuleId) {
        return tabIndexModuleMapper.selectByPrimaryKey(indexModuleId);
    }
}
