package com.gic.enterprise.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constant.LinkEnterpriseTypeEnum;
import com.gic.enterprise.constant.LinkSortEnum;
import com.gic.enterprise.constant.LinkTypeEnum;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.dao.mapper.TabLinkMapper;
import com.gic.enterprise.dto.LinkDTO;
import com.gic.enterprise.entity.TabLink;
import com.gic.enterprise.qo.LinkQO;
import com.gic.enterprise.service.LinkService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-25 14:22
 */
@Service
public class LinkServiceImpl implements LinkService {

    @Autowired
    private TabLinkMapper tabLinkMapper;

    @Override
    public boolean validNameIsRepeat(String name, Integer linkId) {
        TabLink tabLink = new TabLink();
        tabLink.setName(name);
        tabLink.setLinkId(linkId);
        return tabLinkMapper.getSelective(tabLink) != null;
    }

    @Override
    public Integer saveLinkType(String name, Integer type) {
        TabLink tabLink = new TabLink();
        tabLink.setCreateTime(new Date());
        tabLink.setParentId(0);
        tabLink.setStatus(Constants.NORMAL_STATUS);
        tabLink.setType(type);
        tabLink.setName(name);
        tabLink.setIsCustom(Constants.NORMAL_STATUS);
        tabLink.setEnterpriseType(LinkEnterpriseTypeEnum.ALL.getCode());
        tabLinkMapper.insertSelective(tabLink);
        return tabLink.getLinkId();
    }

    @Override
    public int updateLinkType(Integer linkId, String name, Integer type) {
        TabLink tabLink = new TabLink();
        tabLink.setLinkId(linkId);
        tabLink.setName(name);
        tabLink.setType(type);
        return tabLinkMapper.updateByPrimaryKeySelective(tabLink);
    }

//    @Override
//    public Integer saveLinkMenu(Integer parentId, String name, Integer type) {
//        TabLink tabLink = new TabLink();
//        tabLink.setCreateTime(new Date());
//        tabLink.setParentId(parentId);
//        tabLink.setStatus(Constants.NORMAL_STATUS);
//        tabLink.setType(type);
//        tabLink.setName(name);
//        tabLink.setIsCustom(Constants.NORMAL_STATUS);
//        tabLink.setEnterpriseType(LinkEnterpriseTypeEnum.ALL.getCode());
//        tabLinkMapper.insertSelective(tabLink);
//        return tabLink.getLinkId();
//    }
//
//    @Override
//    public int updateLinkMenu(Integer linkId, String name, Integer type) {
//        TabLink tabLink = new TabLink();
//        tabLink.setLinkId(linkId);
//        tabLink.setName(name);
//        tabLink.setType(type);
//        return tabLinkMapper.updateByPrimaryKeySelective(tabLink);
//    }

    @Override
    public void updateLinkMenu(LinkDTO linkDTO) {
        TabLink tabLink = EntityUtil.changeEntityByJSON(TabLink.class, linkDTO);
        tabLinkMapper.updateByPrimaryKeySelective(tabLink);
    }

    @Override
    public Integer saveLinkChildMenu(LinkDTO linkDTO) {
        TabLink tabLink = EntityUtil.changeEntityByJSON(TabLink.class, linkDTO);
        tabLink.setCreateTime(new Date());
        tabLink.setParentId(linkDTO.getParentId());
        tabLink.setStatus(Constants.NORMAL_STATUS);

        tabLink.setIsCustom(Constants.NORMAL_STATUS);
        tabLink.setEnterpriseType(LinkEnterpriseTypeEnum.ALL.getCode());
        if (StringUtils.isBlank(linkDTO.getAppletContent()) && StringUtils.isBlank(linkDTO.getH5Content())) {
            tabLink.setObjectType(null);
            tabLink.setType(LinkTypeEnum.HIERARCHICAL.getCode());
        } else {
            tabLink.setObjectType(linkDTO.getObjectType());
            tabLink.setType(LinkTypeEnum.HIERARCHICAL.getCode());
        }
        tabLinkMapper.insertSelective(tabLink);
        return tabLink.getLinkId();
    }

    @Override
    public int updateLinkChildMenu(LinkDTO linkDTO) {
        TabLink tabLink = new TabLink();
        tabLink.setLinkId(linkDTO.getLinkId());
        tabLink.setObjectType(linkDTO.getObjectType());
        tabLink.setName(linkDTO.getName());
        tabLink.setH5Content(linkDTO.getH5Content());
        tabLink.setAppletContent(linkDTO.getAppletContent());
        if (StringUtils.isBlank(linkDTO.getAppletContent()) && StringUtils.isBlank(linkDTO.getH5Content())) {
            tabLink.setObjectType(null);
            tabLink.setType(LinkTypeEnum.HIERARCHICAL.getCode());
        } else {
            tabLink.setObjectType(linkDTO.getObjectType());
            TabLink oldLink = tabLinkMapper.selectByPrimaryKey(linkDTO.getLinkId());
            if (LinkTypeEnum.HIERARCHICAL.getCode().equals(oldLink.getType())) {
                tabLink.setType(LinkTypeEnum.COMMON_LINK.getCode());
            }
        }
        return tabLinkMapper.updateByPrimaryKeySelective(tabLink);
    }

    @Override
    public List<Integer> getAllLinkIdsByParentIds(List<Integer> linkIdList) {
        List<Integer> allLinkIdList = new ArrayList<>();
        getAllLinkIdsByParentIds(linkIdList, allLinkIdList);
        return allLinkIdList;
    }

    private void getAllLinkIdsByParentIds(List<Integer> parentIdList, List<Integer> resultList) {
        if (CollectionUtils.isNotEmpty(parentIdList)) {
            resultList.addAll(parentIdList);
            List<Integer> linkIds = tabLinkMapper.getLinkIdsByParentIds(parentIdList);
            getAllLinkIdsByParentIds(linkIds, resultList);
        }
    }

    @Override
    public int delLinkByIdList(List<Integer> allIdList) {
        return tabLinkMapper.delLinkByIdList(allIdList);
    }

    @Override
    public List<TabLink> listLinkType() {
        return tabLinkMapper.listLinkType();
    }

    @Override
    public Page<TabLink> listLinkMenu(LinkQO linkQO) {
        if (linkQO.getSortColumn() != null && linkQO.getSortType() != null) {
            String sortColumn = LinkSortEnum.LinkSortColumn.parse(linkQO.getSortColumn());
            String sortType = LinkSortEnum.LinkSortType.parse(linkQO.getSortType());
            if (StringUtils.isNotBlank(sortColumn) && StringUtils.isNotBlank(sortType)) {
                linkQO.setOrderBy(sortColumn + " " + sortType);
            }
        }
        PageHelper.startPage(linkQO);
        return tabLinkMapper.listLinkMenu(linkQO);
    }

    @Override
    public List<TabLink> getLinkByParentIds(List<Integer> linkIdList) {
        return tabLinkMapper.getLinkByParentIds(linkIdList);
    }

    @Override
    public TabLink getById(Integer linkId) {
        return tabLinkMapper.selectByPrimaryKey(linkId);
    }

    @Override
    public List<TabLink> listByGic(Integer enterpriseId, Integer parentId) {
        return tabLinkMapper.listByGic(enterpriseId, parentId);
    }

    @Override
    public List<TabLink> treeByGic(Integer enterpriseId, Integer objectType) {
        return tabLinkMapper.treeByGic(enterpriseId, objectType);
    }

}
