package com.gic.enterprise.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.dao.mapper.TabProjectItemMapper;
import com.gic.enterprise.dto.ProjectItemDTO;
import com.gic.enterprise.entity.TabProjectItem;
import com.gic.enterprise.qo.ProjectItemQO;
import com.gic.enterprise.service.ProjectItemService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-09-17 10:10
 */
@Service
public class ProjectItemServiceImpl implements ProjectItemService {

    @Autowired
    private TabProjectItemMapper tabProjectItemMapper;

    @Override
    public TabProjectItem getFirstProjectItem(ProjectItemDTO projectItemDTO) {
        return tabProjectItemMapper.getFirstProjectItem(projectItemDTO);
    }

    @Override
    public Integer save(ProjectItemDTO copy) {
        TabProjectItem projectItem = EntityUtil.changeEntityByJSON(TabProjectItem.class, copy);
        projectItem.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        projectItem.setCreateTime(new Date());
        Double sort = tabProjectItemMapper.getMinSort();
        projectItem.setSort(sort == null ? 1d : sort - 1d);
        tabProjectItemMapper.insertSelective(projectItem);
        return projectItem.getProjectItemId();
    }

    @Override
    public Integer update(ProjectItemDTO copy) {
        TabProjectItem projectItem = EntityUtil.changeEntityByJSON(TabProjectItem.class, copy);
        return tabProjectItemMapper.updateByPrimaryKeySelective(projectItem);
    }

    @Override
    public boolean validProjectItemNameIsRepeat(String projectItemName, Integer projectId, Integer projectItemId) {
        ProjectItemDTO projectItem = new ProjectItemDTO();
        projectItem.setProjectItemName(projectItemName);
        projectItem.setProjectId(projectId);
        projectItem.setProjectItemId(projectItemId);
        TabProjectItem item = tabProjectItemMapper.getFirstProjectItem(projectItem);
        return item != null;
    }

    @Override
    public boolean validProjectItemCodeIsRepeat(String projectItemCode, Integer projectItemId) {
        ProjectItemDTO projectItem = new ProjectItemDTO();
        projectItem.setProjectItemCode(projectItemCode);
        projectItem.setProjectItemId(projectItemId);
        TabProjectItem item = tabProjectItemMapper.getFirstProjectItem(projectItem);
        return item != null;
    }

    @Override
    public Integer del(Integer projectItemId) {
        TabProjectItem projectItem = new TabProjectItem();
        projectItem.setProjectItemId(projectItemId);
        projectItem.setStatus(GlobalInfo.DATA_STATUS_DELETE);
        return tabProjectItemMapper.updateByPrimaryKeySelective(projectItem);
    }

    @Override
    public Page<TabProjectItem> listProjectItem(ProjectItemQO projectItemQO) {
        PageHelper.startPage(projectItemQO.getCurrentPage(), projectItemQO.getPageSize());
        return tabProjectItemMapper.listProjectItem(projectItemQO);
    }

    @Override
    public Map<Integer, Integer> getProjectItemCountByProjectIds(List<Integer> projectIds, Integer useStatus) {
        List<Map<String, Object>> list = tabProjectItemMapper.getProjectItemCountByProjectIds(projectIds, useStatus);
        Map<Integer, Integer> result = new HashMap<>(list.size());
        for (Map<String, Object> map : list) {
            result.put(Integer.valueOf(map.get("projectId").toString()), Integer.valueOf(map.get("itemCount").toString()));
        }
        return result;
    }

    @Override
    public List<TabProjectItem> listAllProjectItem() {
        return tabProjectItemMapper.listProjectItem(new ProjectItemQO());
    }

    @Override
    public TabProjectItem getById(Integer projectItemId) {
        TabProjectItem projectItem = tabProjectItemMapper.selectByPrimaryKey(projectItemId);
        return (projectItem != null && projectItem.getStatus() == GlobalInfo.DATA_STATUS_NORMAL) ? projectItem : null;
    }

    @Override
    public TabProjectItem getByProjectItemCode(String projectItemCode) {
        return tabProjectItemMapper.getByProjectItemCode(projectItemCode);
    }

    @Override
    public void setProjectItemSort(Integer projectItemId, Integer seq) {
        if (seq <= 1) {
            // 相当于置顶
            sortTopping(projectItemId);
        } else {
            int start = seq - 2;
            int size = 2;

            List<TabProjectItem> list = tabProjectItemMapper.listTwoElementExceptId(start, size, projectItemId);
            if (CollectionUtils.isEmpty(list)) {
                // 置底
                sortBottom(projectItemId);
            } else {
                Double sort;
                if (list.size() == 1) {
                    // 置底
                    sort = list.get(0).getSort() + 1d;
                } else {
                    sort = (list.get(0).getSort() + list.get(1).getSort()) / 2d;
                }
                TabProjectItem project = new TabProjectItem();
                project.setProjectItemId(projectItemId);
                project.setSort(sort);
                tabProjectItemMapper.updateByPrimaryKeySelective(project);
            }
        }
    }

    private void sortTopping(Integer projectItemId) {
        Double sort = tabProjectItemMapper.getMinSort();
        TabProjectItem projectItem = new TabProjectItem();
        projectItem.setProjectItemId(projectItemId);
        projectItem.setSort(sort - 1d);
        tabProjectItemMapper.updateByPrimaryKeySelective(projectItem);
    }

    private void sortBottom(Integer projectItemId) {
        Double sort = tabProjectItemMapper.getMaxSort();
        if (sort != null) {
            TabProjectItem projectItem = new TabProjectItem();
            projectItem.setProjectItemId(projectItemId);
            projectItem.setSort(sort + 1d);
            tabProjectItemMapper.updateByPrimaryKeySelective(projectItem);
        }
    }

}
