package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.applet.AppletAppStatusEnum;
import com.gic.enterprise.dto.AppletAppDTO;
import com.gic.enterprise.entity.TabAppletApp;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.AppletAppApiService;
import com.gic.enterprise.service.AppletAppService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("appletAppApiService")
public class AppletAppApiServiceImpl implements AppletAppApiService {
    @Autowired
    private AppletAppService appletAppService;

    @Override
    public ServiceResponse<Integer> save(AppletAppDTO dto) {
        Integer enterpriseId = dto.getEnterpriseId();
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (StringUtils.isBlank(dto.getAppName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "APP名称不能为空");
        }
        if (StringUtils.isBlank(dto.getAppId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "APPID不能为空");
        }
        if (appletAppService.isRepeatAppId(enterpriseId, null, dto.getAppId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "APPID不能重复");
        }
        if (appletAppService.isRepeatName(enterpriseId, null, dto.getAppName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "APP名称不能重复");
        }
        int count = appletAppService.countByEnterpriseId(enterpriseId);
        if (count >= 10) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "当前配置API名单数已达上限10个。若要启用该API，请先解绑当前配置中无用的API");
        }
        Integer id = appletAppService.save(dto);
        return ServiceResponse.success(id);
    }

    @Override
    public ServiceResponse<Void> edit(AppletAppDTO dto) {
        Integer appletAppId = dto.getAppletAppId();
        TabAppletApp record = appletAppService.getById(appletAppId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键ID有误，查无数据");
        }
        Integer enterpriseId = dto.getEnterpriseId();
        if (StringUtils.isBlank(dto.getAppName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "APP名称不能为空");
        }
        if (StringUtils.isBlank(dto.getAppId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "APPID不能为空");
        }
        if (appletAppService.isRepeatAppId(enterpriseId, appletAppId, dto.getAppId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "APPID不能重复");
        }
        if (appletAppService.isRepeatName(enterpriseId, appletAppId, dto.getAppName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "APP名称不能重复");
        }

        dto.setEnterpriseId(record.getEnterpriseId());
        dto.setAppStatus(dto.getAppStatus());
        appletAppService.edit(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> bind(Integer appletAppId, Integer enterpriseId) {
        TabAppletApp record = appletAppService.getById(appletAppId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键ID有误，查无数据");
        }
        int count = appletAppService.countByEnterpriseId(enterpriseId);
        if (count >= 10) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "当前配置API名单数已达上限10个。若要启用该API，请先解绑当前配置中无用的API");
        }
        appletAppService.updateAppStatus(appletAppId, AppletAppStatusEnum.BIND.getCode());
        return ServiceResponse.success(record.getAppName());
    }

    @Override
    public ServiceResponse<String> unBind(Integer appletAppId) {
        TabAppletApp record = appletAppService.getById(appletAppId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键ID有误，查无数据");
        }
        if (record.getAppStatus().intValue() != AppletAppStatusEnum.BIND.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "未绑定");
        }
        appletAppService.updateAppStatus(appletAppId, AppletAppStatusEnum.UNBIND.getCode());
        return ServiceResponse.success(record.getAppName());
    }

    @Override
    public ServiceResponse<String> delete(Integer appletAppId) {
        TabAppletApp record = appletAppService.getById(appletAppId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键ID有误，查无数据");
        }
        if (record.getAppStatus().intValue() == AppletAppStatusEnum.BIND.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "请先解绑，再删除");
        }
        appletAppService.updateAppStatus(appletAppId, AppletAppStatusEnum.DELETE.getCode());
        return ServiceResponse.success(record.getAppName());
    }

    @Override
    public ServiceResponse<Page<AppletAppDTO>> page(String search, Integer enterpriseId, Integer appStatus,
            Integer orderCode, Integer currentPage, Integer pageSize) {
        com.github.pagehelper.Page<TabAppletApp> page = appletAppService.page(search, enterpriseId, appStatus,
                orderCode, currentPage, pageSize);
        Page<AppletAppDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, AppletAppDTO.class);
        return ServiceResponse.success(resultPage);
    }
}
