package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.AppletServiceConfigDTO;
import com.gic.enterprise.entity.TabAppletServiceConfig;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.AppletServiceConfigQO;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.AppletServiceConfigApiService;
import com.gic.enterprise.service.AppletServiceConfigService;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.dto.ServiceConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-27 17:03
 */
@Service("appletServiceConfigApiService")
public class AppletServiceConfigApiServiceImpl implements AppletServiceConfigApiService {

    @Autowired
    private AppletServiceConfigService appletServiceConfigService;
    @Autowired
    AppletsConfigApiService appletsConfigApiService;
    @Autowired
    ServiceConfigApiService serviceConfigApiService;

    @Override
    public ServiceResponse<Page<AppletServiceConfigDTO>> listApplet(Integer enterpriseId, AppletServiceConfigQO appletServiceConfigQO) {
        appletServiceConfigQO.setEnterpriseId(enterpriseId);
        // 查询
        com.github.pagehelper.Page<TabAppletServiceConfig> listApplet = appletServiceConfigService.listApplet(appletServiceConfigQO);
        Page<AppletServiceConfigDTO> page = PageHelperUtils.changePageHelperToCurrentPage(listApplet, AppletServiceConfigDTO.class);
        return EnterpriseServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<Void> syncApplet(Integer enterpriseId) {
        ServiceResponse<List<AppletsConfigSimpleOutputDTO>> serviceResponse = appletsConfigApiService.listValidAppletConfigAppId(enterpriseId);
        if (serviceResponse.isSuccess()) {
            List<AppletsConfigSimpleOutputDTO> responseResult = serviceResponse.getResult();
            appletServiceConfigService.syncApplet(responseResult, enterpriseId);
            return EnterpriseServiceResponse.success();
        } else {
            return EnterpriseServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @Override
    public ServiceResponse<Page<AppletServiceConfigDTO>> listService(Integer enterpriseId, AppletServiceConfigQO appletServiceConfigQO) {
        appletServiceConfigQO.setEnterpriseId(enterpriseId);
        com.github.pagehelper.Page<TabAppletServiceConfig> listService = appletServiceConfigService.listService(appletServiceConfigQO);
        Page<AppletServiceConfigDTO> page = PageHelperUtils.changePageHelperToCurrentPage(listService, AppletServiceConfigDTO.class);
        return EnterpriseServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<Void> syncServiceConfig(Integer enterpriseId) {
        ServiceResponse<List<ServiceConfigSimpleOutputDTO>> serviceResponse = serviceConfigApiService.listValidServiceConfigAppId(enterpriseId);
        if (serviceResponse.isSuccess()) {
            List<ServiceConfigSimpleOutputDTO> responseResult = serviceResponse.getResult();
            appletServiceConfigService.syncServiceConfig(responseResult, enterpriseId);
            return EnterpriseServiceResponse.success();
        } else {
            return EnterpriseServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @Override
    public ServiceResponse<Void> delAppletService(Integer appletServiceId) {
        appletServiceConfigService.delAppletService(appletServiceId);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<AppletServiceConfigDTO> getAppletService(Integer appletServiceId) {
        TabAppletServiceConfig appletService = appletServiceConfigService.getAppletService(appletServiceId);
        if (appletService != null) {
            return EnterpriseServiceResponse.success(EntityUtil.changeEntityByJSON(AppletServiceConfigDTO.class, appletService));
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
    }
}
