package com.gic.enterprise.service.outer.impl;

import java.math.BigDecimal;
import java.util.Date;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constant.BillingPayResultEnum;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.dto.*;
import com.gic.enterprise.entity.TabBillingAccount;
import com.gic.enterprise.entity.TabBillingRefund;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.*;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.store.utils.valid.ValidUtil;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/28 10:27
 */
@Service("billingBalancePayApiService")
public class BillingBalancePayApiServiceImpl implements BillingBalancePayApiService {
    private Logger logger = LogManager.getLogger(BillingBalancePayApiServiceImpl.class);
    @Autowired
    private BillingPayInfoApiService billingPayInfoApiService;
    @Autowired
    private BillingRefundService billingRefundService;
    @Autowired
    private BillingAccountService billingAccountService;
    @Autowired
    private BillingAccountApiService billingAccountApiService;

    @Override
    public ServiceResponse<Double> queryAccount(Integer enterpriseId) {
        TabBillingAccount account = billingAccountService.getByEnterpriseId(enterpriseId);
        if (account != null) {
            return ServiceResponse.success(account.getAccountBalance());
        } else {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "不存在该账户");
        }
    }

    @Override
    public ServiceResponse<BalancePayReturnDTO> savePrePay(OutPayDTO outPayDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(outPayDTO, OutPayDTO.SavePrePay.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        outPayDTO.setPayType(PayTypeEnum.BALANCE_PAY.getCode());
        ServiceResponse<BillingPayInfoDTO> response = billingPayInfoApiService.savePrePayInfo(outPayDTO);
        if (response.isSuccess()) {
            BillingPayInfoDTO result = response.getResult();
            BalancePayReturnDTO balancePayReturnDTO = new BalancePayReturnDTO();
            balancePayReturnDTO.setTotalFeePaid(result.getTotalFeePaid());
            balancePayReturnDTO.setEnterpriseId(result.getEnterpriseId());
            balancePayReturnDTO.setPayInfoSerialNumber(result.getSerialNumber());
            balancePayReturnDTO.setAuthCode(result.getAuthCode());
            balancePayReturnDTO.setOrderSerialNumber(outPayDTO.getOrderSerialNumber());
            return ServiceResponse.success(balancePayReturnDTO);
        } else {
            return ServiceResponse.failure(response.getCode(), response.getMessage());
        }
    }

    @Override
    public ServiceResponse<BalancePayReturnDTO> confirmPay(OutPayDTO outPayDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(outPayDTO, OutPayDTO.ConfirmPay.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        outPayDTO.setPayType(PayTypeEnum.BALANCE_PAY.getCode());
        ServiceResponse<BillingPayInfoDTO> serviceResponse = billingPayInfoApiService.confirmPay(outPayDTO);
        if (serviceResponse.isSuccess()) {
            BillingPayInfoDTO payInfoDTO = serviceResponse.getResult();
            BalancePayReturnDTO balancePayReturnDTO = new BalancePayReturnDTO();
            balancePayReturnDTO.setPayInfoSerialNumber(payInfoDTO.getSerialNumber());
            balancePayReturnDTO.setOrderSerialNumber(payInfoDTO.getOrderSerialNumber());
            balancePayReturnDTO.setResultCode(payInfoDTO.getResultCode());
            balancePayReturnDTO.setTotalFeePaid(payInfoDTO.getTotalFeePaid());
            balancePayReturnDTO.setEnterpriseId(payInfoDTO.getEnterpriseId());
            return ServiceResponse.success(balancePayReturnDTO);
        } else {
            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @Override
    public ServiceResponse<BalancePayReturnDTO> savePayAndConfirmPay(OutPayDTO outPayDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(outPayDTO, OutPayDTO.ConfirmPay.class, OutPayDTO.SavePrePay.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        outPayDTO.setPayType(PayTypeEnum.BALANCE_PAY.getCode());
        ServiceResponse<BillingPayInfoDTO> serviceResponse = billingPayInfoApiService.savePrePayInfo(outPayDTO);
        if (!serviceResponse.isSuccess()) {
            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        ServiceResponse<BillingPayInfoDTO> confirmPayResponse = billingPayInfoApiService.confirmPay(outPayDTO);
        if (confirmPayResponse.isSuccess()) {
            BillingPayInfoDTO payInfoDTO = confirmPayResponse.getResult();
            BalancePayReturnDTO balancePayReturnDTO = new BalancePayReturnDTO();
            balancePayReturnDTO.setPayInfoSerialNumber(payInfoDTO.getSerialNumber());
            balancePayReturnDTO.setOrderSerialNumber(payInfoDTO.getOrderSerialNumber());
            balancePayReturnDTO.setResultCode(payInfoDTO.getResultCode());
            balancePayReturnDTO.setTotalFeePaid(payInfoDTO.getTotalFeePaid());
            balancePayReturnDTO.setEnterpriseId(payInfoDTO.getEnterpriseId());
            return ServiceResponse.success(balancePayReturnDTO);
        } else {
            return ServiceResponse.failure(confirmPayResponse.getCode(), confirmPayResponse.getMessage());
        }
    }

    @Override
    public ServiceResponse<BalancePayReturnDTO> getPayInfo(Integer enterpriseId, String orderSerialNumber, String requestCode) {
        ServiceResponse<BillingPayInfoDTO> serviceResponse = billingPayInfoApiService.getPayInfo(enterpriseId, orderSerialNumber, requestCode);
        if (serviceResponse.isSuccess()) {
            BillingPayInfoDTO payInfoDTO = serviceResponse.getResult();
            BalancePayReturnDTO balancePayReturnDTO = new BalancePayReturnDTO();
            balancePayReturnDTO.setPayInfoSerialNumber(payInfoDTO.getSerialNumber());
            balancePayReturnDTO.setOrderSerialNumber(payInfoDTO.getOrderSerialNumber());
            balancePayReturnDTO.setTotalFeePaid(payInfoDTO.getTotalFeePaid());
            balancePayReturnDTO.setEnterpriseId(payInfoDTO.getEnterpriseId());
            balancePayReturnDTO.setResultCode(payInfoDTO.getResultCode());
            balancePayReturnDTO.setProductDescription(payInfoDTO.getProductDescription());
            return ServiceResponse.success(balancePayReturnDTO);
        } else {
            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @Override
    public ServiceResponse<Void> cancelPay(OutPayDTO outPayDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(outPayDTO, OutPayDTO.CancelPay.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        outPayDTO.setPayType(PayTypeEnum.BALANCE_PAY.getCode());
        return this.billingPayInfoApiService.cancelPay(outPayDTO);
    }

    @Override
    public ServiceResponse<RefundReturnDTO> refund(RefundDTO refundDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(refundDTO, RefundDTO.RefundValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        ServiceResponse<BillingPayInfoDTO> payService = this.billingPayInfoApiService.getPayInfo(refundDTO.getEnterpriseId(), refundDTO.getOrderSerialNumber(), refundDTO.getRequestCode());
        if (!payService.isSuccess()) {
            return ServiceResponse.failure(payService.getCode(), payService.getMessage());
        }
        BillingPayInfoDTO payInfo = payService.getResult();
        if (!BillingPayResultEnum.SUCCESS.getCode().equals(payInfo.getResultCode())) {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "退款失败, 订单号不存在");
        }
        // 退款单号是否重复
        TabBillingRefund refund = billingRefundService.getRefund(refundDTO.getEnterpriseId(), refundDTO.getRefundSerialNumber(), refundDTO.getRequestCode());
        if (refund != null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "退款失败， 退款单号重复");
        }
        // 每笔退款金额（部分）超出订单总金额
        Double refundFee = refundDTO.getRefundFee();
        if (refundFee > payInfo.getTotalFeePaid()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "退款失败， 退款金额超出总金额");
        }
        // 每笔退款金额累计超出订单总金额
        Double oldTotalFee = billingRefundService.getTotalRefundFee(refundDTO.getEnterpriseId(), refundDTO.getOrderSerialNumber(), refundDTO.getRequestCode());
        if (oldTotalFee == null) {
            oldTotalFee = 0d;
        }
        double olderFee = new BigDecimal(refundFee).add(new BigDecimal(oldTotalFee)).doubleValue();
        if (olderFee > refundFee) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "退款失败， 退款金额超出总金额");
        }
        // 保存退款记录
        Date date = new Date();
        String serialNumber = CreateRandomUtils.createSerialNumber();
        refundDTO.setRefundPayNumber(serialNumber);
        refundDTO.setTimeEnd(date);
        Integer line = billingRefundService.save(refundDTO);
        if (line == 0) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            logger.info("保存退款记录失败：{}", JSON.toJSONString(refundDTO));
            return ServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), "保存退款记录失败");
        }
        // 退款
        int i = billingAccountService.updateAccountAfterRechargeOrRefund(olderFee, refundDTO.getEnterpriseId());
        if (i == 0) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            logger.info("退款失败：{}", JSON.toJSONString(refundDTO));
            return ServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), "退款失败");
        }
        // 写明细
//        AccountBalanceRecordDTO balanceRecordDTO = new AccountBalanceRecordDTO();
//        balanceRecordDTO.setProductName(payInfo.getProductDescription());
//        balanceRecordDTO.setPayType(PayTypeEnum.BALANCE_PAY.getCode());
//        balanceRecordDTO.setProductType(refundDTO.getBuyProductType());
//        balanceRecordDTO.setBillType(BillTypeEnum.REFUND.getCode());
//        balanceRecordDTO.setDeductionTime(date);
//        balanceRecordDTO.setEnterpriseId(refundDTO.getEnterpriseId());
//        balanceRecordDTO.setFee(refundDTO.getRefundFee());
//        Integer line2 = billingAccountBalanceRecordService.save(balanceRecordDTO);
//        if (line2 == 0) {
//            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
//            logger.info("写入明细失败：{}", JSON.toJSONString(refundDTO));
//            return ServiceResponse.failure(ErrorCode.UNKNOWN_ERROR.getCode(), "写入明细失败");
//        }
        // 返回应用市场
        RefundReturnDTO refundReturnDTO = EntityUtil.changeEntityByOrika(RefundReturnDTO.class, refundDTO);
        refundReturnDTO.setRefundPaySerialNumber(serialNumber);
        refundReturnDTO.setDeductionTime(date);
        return ServiceResponse.success(refundReturnDTO);
    }

    @Override
    public ServiceResponse<RefundReturnDTO> getRefund(RefundDTO refundDTO) {
        TabBillingRefund refund = billingRefundService.getRefund(refundDTO.getEnterpriseId(), refundDTO.getRefundSerialNumber(), refundDTO.getRequestCode());
        if (refund != null) {
            RefundReturnDTO returnDTO = EntityUtil.changeEntityByOrika(RefundReturnDTO.class, refundDTO);
            return ServiceResponse.success(returnDTO);
        } else {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
        }
    }
}
