package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.BillingAuditStatusEnum;
import com.gic.enterprise.constant.BuyTypeEnum;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.dto.ApprovalCallBackDTO;
import com.gic.enterprise.dto.BillingPayInfoDTO;
import com.gic.enterprise.dto.InvoiceDTO;
import com.gic.enterprise.dto.OutPayDTO;
import com.gic.enterprise.entity.TabBillingOrder;
import com.gic.enterprise.entity.TabBillingPayInfo;
import com.gic.enterprise.entity.TabBillingRechargeRecord;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.BillListQueryQO;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.*;
import com.gic.enterprise.service.outer.impl.pay.PayStrategy;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import com.gic.redis.data.util.RedisUtil;
import com.gic.thirdparty.api.dto.ScanPayResDataDTO;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Service("billingPayInfoApiService")
public class BillingPayInfoApiServiceImpl implements BillingPayInfoApiService {
    private Logger logger = LogManager.getLogger(BillingPayInfoApiServiceImpl.class);
    @Autowired
    private BillingPayInfoService billingPayInfoService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private BillingRechargeRecordService billingRechargeRecordService;
    @Autowired
    private BillingOrderService billingOrderService;
    private Map<String, PayStrategy> strategyMap = new ConcurrentHashMap<>();

    @Autowired
    public BillingPayInfoApiServiceImpl(Map<String, PayStrategy> strategyMap) {
        this.strategyMap.clear();
        this.strategyMap.putAll(strategyMap);
    }

    @Override
    public ServiceResponse<Page<BillingPayInfoDTO>> listBill(BillListQueryQO params) {
        com.github.pagehelper.Page page = billingPayInfoService.listBilling(params);
        Page<BillingPayInfoDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, BillingPayInfoDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Void> saveInvoiceAccountToRedis(InvoiceDTO invoiceDTO, Integer enterpriseId) {
        TabEnterprise enterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (enterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID有误");
        }
        RedisUtil.setCache("enterprise:billing:invoice:" + enterpriseId, invoiceDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<InvoiceDTO> getInvoiceInfoFromRedis(Integer enterpriseId) {
        TabEnterprise enterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (enterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID有误");
        }
        InvoiceDTO value = (InvoiceDTO) RedisUtil.getCache("enterprise:billing:invoice:" + enterpriseId);
        if (value == null) {
            return ServiceResponse.success(new InvoiceDTO());
        }
        return ServiceResponse.success(value);
    }

    @Override
    public ServiceResponse<BillingPayInfoDTO> savePrePayInfo(Integer enterpriseId, Double totalFee, Integer payType, Integer buyType, String orderSerialNumber) {
        OutPayDTO outPayDTO = new OutPayDTO();
        outPayDTO.setEnterpriseId(enterpriseId);
        outPayDTO.setTotalFee(totalFee);
        outPayDTO.setTotalFeePaid(totalFee);
        outPayDTO.setBuyType(buyType);
        outPayDTO.setOrderSerialNumber(orderSerialNumber);
        return strategyMap.get(payType.toString()).savePrePayInfo(outPayDTO);
    }

    @Override
    public ServiceResponse<BillingPayInfoDTO> savePrePayInfo(OutPayDTO outPayDTO) {
        return strategyMap.get(outPayDTO.getPayType().toString()).savePrePayInfo(outPayDTO);
    }

    @Override
    public ServiceResponse<BillingPayInfoDTO> confirmPay(OutPayDTO outPayDTO) {
        return strategyMap.get(outPayDTO.getPayType().toString()).confirmPay(outPayDTO);
    }

    @Override
    public ServiceResponse<Void> wxCallBack(String resDataStr) {
        ScanPayResDataDTO resData = JSON.parseObject(resDataStr, ScanPayResDataDTO.class);
        ApprovalCallBackDTO approvalCallBackDTO = new ApprovalCallBackDTO();
        approvalCallBackDTO.setPaySerialNumber(resData.getOut_trade_no());
        approvalCallBackDTO.setEnterpriseId(Integer.valueOf(resData.getAttach().split(":")[1]));
        approvalCallBackDTO.setErrorCode(resData.getErr_code());
        approvalCallBackDTO.setErrorCodeRes(resData.getErr_code_des());
        approvalCallBackDTO.setTimeEnd(resData.getTime_end());
        approvalCallBackDTO.setPrepayId(resData.getPrepay_id());
        approvalCallBackDTO.setResultCode(StringUtils.equalsIgnoreCase(resData.getResult_code(), "SUCCESS") ? 1 : 0);
        approvalCallBackDTO.setTotalFeePaid(new BigDecimal(resData.getTotal_fee()).divide(new BigDecimal("100"), 5, RoundingMode.HALF_UP).doubleValue());
        approvalCallBackDTO.setOnlineSerialNumber(resData.getTransaction_id());

        return strategyMap.get(PayTypeEnum.WE_CHAT_PAY.getCode() + "").callBack(approvalCallBackDTO);
    }

    @Override
    public ServiceResponse<Void> offlineCallBack(String resDataStr) {
        JSONObject jsonObject = JSON.parseObject(resDataStr);
        logger.info("财务审批回调:{}", resDataStr);
        ApprovalCallBackDTO approvalCallBackDTO = new ApprovalCallBackDTO();
        approvalCallBackDTO.setOrderSerialNumber(jsonObject.getString("orderNumber"));
        approvalCallBackDTO.setEnterpriseId(jsonObject.getInteger("enterpriseId"));
        approvalCallBackDTO.setBuyType(jsonObject.getInteger("orderType"));
        approvalCallBackDTO.setAuditStatus(jsonObject.getInteger("approvalStatus"));
        Date approvalTime = jsonObject.getDate("approvalTime");
        approvalCallBackDTO.setTimeEnd(DateUtil.formatString(approvalTime, DateUtil.FORMAT_DATETIME_14));
        // 银行回调取计划金额
        approvalCallBackDTO.setTotalFee(jsonObject.getDouble("plannedAmount"));
        approvalCallBackDTO.setTotalFeePaid(jsonObject.getDouble("accountAmount"));
        ServiceResponse<Void> validParam = ValidParamsUtils.allCheckValidate(approvalCallBackDTO, ApprovalCallBackDTO.ApprovalValid.class);
        if (!validParam.isSuccess()) {
            return validParam;
        }
        return strategyMap.get(PayTypeEnum.OFFLINE_PAY.getCode() + "").callBack(approvalCallBackDTO);
    }

    @Override
    public ServiceResponse<Boolean> queryAndUpdatePayStatus(Integer enterpriseId, String orderSerialNumber, Integer payType) {
        TabBillingPayInfo payInfo = billingPayInfoService.getByOrderSerialNumber(enterpriseId, null, orderSerialNumber);
        if (payInfo != null) {
            if (BuyTypeEnum.BALANCE_RECHARGE.getCode() == payInfo.getBuyType()) {
                TabBillingRechargeRecord rechargeRecord = billingRechargeRecordService.getRechargeRecordBySerialNumber(orderSerialNumber);
                return ServiceResponse.success(rechargeRecord != null && BillingAuditStatusEnum.PASS.getCode().equals(rechargeRecord.getAuditStatus()));
            } else if(BuyTypeEnum.BUY_SHORT_MESSAGE_PACKAGE.getCode() == payInfo.getBuyType()){
                TabBillingOrder order = billingOrderService.getOrderBySerialNumber(orderSerialNumber);
                return ServiceResponse.success(order != null && BillingAuditStatusEnum.PASS.getCode().equals(order.getOrderStatus()));
            }
        }
        return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
    }

    @Override
    public ServiceResponse<BillingPayInfoDTO> getPayInfoDetail(Integer payInfoId) {
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(BillingPayInfoDTO.class, this.billingPayInfoService.getById(payInfoId)));
    }

    @Override
    public ServiceResponse<BillingPayInfoDTO> getPayInfo(Integer enterpriseId, String orderSerialNumber, String requestCode) {
        BillingPayInfoDTO infoDTO = EntityUtil.changeEntityByJSON(BillingPayInfoDTO.class, this.billingPayInfoService.getByOrderSerialNumber(enterpriseId, requestCode, orderSerialNumber));
        if (infoDTO != null) {
            return ServiceResponse.success(infoDTO);
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
    }

    @Override
    public ServiceResponse<Void> cancelPay(OutPayDTO outPayDTO) {
        BillingPayInfoDTO payInfoDTO = new BillingPayInfoDTO();
        payInfoDTO.setEnterpriseId(outPayDTO.getEnterpriseId());
        payInfoDTO.setOrderSerialNumber(outPayDTO.getOrderSerialNumber());
        payInfoDTO.setRequestCode(outPayDTO.getRequestCode());
        Integer line = this.billingPayInfoService.cancelPay(payInfoDTO);
        if (line > 0) {
            return ServiceResponse.success();
        } else {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "订单不存在或已支付");
        }

    }

    @Override
    public ServiceResponse<Void> updateInvoiceStatus(String serialNumbers) {
        billingPayInfoService.updateInvoiceStatus(serialNumbers);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<BillingPayInfoDTO> getPayInfoBySerialNumber(String serialNumber) {
        TabBillingPayInfo record = billingPayInfoService.getPayInfoBySerialNumber(serialNumber);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "流水号有误，支付信息不存在");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(BillingPayInfoDTO.class, record));
    }

}
