package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.constant.BuyTypeEnum;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.dto.BillingPayInfoDTO;
import com.gic.enterprise.dto.BillingRechargeDTO;
import com.gic.enterprise.dto.OutPayDTO;
import com.gic.enterprise.entity.TabBillingRechargeRecord;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.BillingPayInfoApiService;
import com.gic.enterprise.service.BillingRechargeApiService;
import com.gic.enterprise.service.BillingRechargeRecordService;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Map;

/**
 *
 * @ClassName: BillingRechargeApiServiceImpl
 * @Description:
 * @author zhiwj
 * @date 2019/8/15 17:05
 */
@Service("billingRechargeApiService")
public class BillingRechargeApiServiceImpl implements BillingRechargeApiService {
    private Logger logger = LogManager.getLogger(BillingRechargeApiServiceImpl.class);
    @Autowired
    private BillingRechargeRecordService billingRechargeRecordService;
    @Autowired
    private BillingPayInfoApiService billingPayInfoApiService;

    @Override
    @Transactional
    public ServiceResponse<Map<String, Object>> saveAndSelectPaymentMethod(BillingRechargeDTO billingRechargeDTO) {
        String serialNumber = StringUtils.isNotBlank(billingRechargeDTO.getSerialNumber())?billingRechargeDTO.getSerialNumber(): CreateRandomUtils.createSerialNumber();
        ServiceResponse<BillingPayInfoDTO> serviceResponse = billingPayInfoApiService.savePrePayInfo(billingRechargeDTO.getEnterpriseId(), billingRechargeDTO.getTotalFee(), billingRechargeDTO.getPayType(), BuyTypeEnum.BALANCE_RECHARGE.getCode(), serialNumber);
        if (!serviceResponse.isSuccess()) {
            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        if (PayTypeEnum.ALI_PAY.getCode() == billingRechargeDTO.getPayType()) {
            // 支付宝
            return ServiceResponse.success();
        } else if (PayTypeEnum.WE_CHAT_PAY.getCode() == billingRechargeDTO.getPayType()) {
            // 微信支付
            // 生成充值记录
            BillingPayInfoDTO payInfoDTO = serviceResponse.getResult();

            if (StringUtils.isNotBlank(billingRechargeDTO.getSerialNumber())) {
                TabBillingRechargeRecord record = this.billingRechargeRecordService.getRechargeRecordBySerialNumber(billingRechargeDTO.getSerialNumber());
                if (record == null) {
                    billingRechargeDTO.setPayInfoId(payInfoDTO.getPayInfoId());
                    billingRechargeDTO.setSerialNumber(serialNumber);
                    billingRechargeRecordService.saveWxPay(billingRechargeDTO);
                } else {
                    serialNumber = record.getSerialNumber();
                }
            } else {
                billingRechargeDTO.setPayInfoId(payInfoDTO.getPayInfoId());
                billingRechargeDTO.setSerialNumber(serialNumber);
                billingRechargeRecordService.saveWxPay(billingRechargeDTO);
            }

            JSONObject jo = new JSONObject();
            jo.put("payInfoSerialNumber", payInfoDTO.getSerialNumber());
            jo.put("serialNumber", serialNumber);
            jo.put("totalFee", billingRechargeDTO.getTotalFee());
            jo.put("codeUrl", payInfoDTO.getCodeUrl());
            return ServiceResponse.success(jo);
        } else {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不存在该支付类型");
        }
    }

    @Override
    public ServiceResponse<Integer> confirmPay(BillingRechargeDTO billingRechargeDTO) {
        if (PayTypeEnum.OFFLINE_PAY.getCode() == billingRechargeDTO.getPayType()) {
            // 银行对公转账
            // 保存支付记录
            String code = RedisUtil.getCache("enterprise:billing:verificationcode:" + billingRechargeDTO.getEnterpriseId()).toString();
            if (!StringUtils.equals(code, billingRechargeDTO.getVerificationCode())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "验证码错误或已失效");
            }
            // 查询审核中的一条记录
            TabBillingRechargeRecord auditingRecord = billingRechargeRecordService.getAuditingRechargeRecord(billingRechargeDTO.getEnterpriseId());
            if (auditingRecord != null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "当前有一笔交易处于审核中状态，不可发起新的一笔银行转账。请耐心等待审核结果。");
            }
            // 保存充值记录
            TabBillingRechargeRecord billingRechargeRecord = billingRechargeRecordService.saveOffLinePay(billingRechargeDTO);

            OutPayDTO outPayDTO = new OutPayDTO();
            outPayDTO.setEnterpriseId(billingRechargeDTO.getEnterpriseId());
            outPayDTO.setTotalFee(billingRechargeDTO.getTotalFee());
            outPayDTO.setPayType(billingRechargeDTO.getPayType());
            outPayDTO.setBuyType(BuyTypeEnum.BALANCE_RECHARGE.getCode());
            outPayDTO.setOrderSerialNumber(billingRechargeRecord.getSerialNumber());
            outPayDTO.setAuthCode(billingRechargeDTO.getVerificationCode());
            outPayDTO.setInitiatorUser(billingRechargeDTO.getInitiatorUser());
            outPayDTO.setInitiatorName(billingRechargeDTO.getInitiatorName());

            billingPayInfoApiService.confirmPay(outPayDTO);
            return ServiceResponse.success();
        } else {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不存在该支付类型");
        }
    }

    @Override
    public ServiceResponse<Boolean> hasRecharge(Integer enterpriseId) {
        TabBillingRechargeRecord auditingRecord = billingRechargeRecordService.getAuditingRechargeRecord(enterpriseId);
        return ServiceResponse.success(auditingRecord != null);
    }
}
