package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.BillingAuditStatusEnum;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.dto.BillingRechargeRecordDTO;
import com.gic.enterprise.entity.TabBillingPayInfo;
import com.gic.enterprise.entity.TabBillingRechargeRecord;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.RechargeRecordQO;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.BillingPayInfoService;
import com.gic.enterprise.service.BillingRechargeRecordApiService;
import com.gic.enterprise.service.BillingRechargeRecordService;
import com.gic.finance.dto.TransferAccountsApprovalDTO;
import com.gic.finance.service.TransferAccountsApprovalApiService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/15 17:47
 */
@Service("billingRechargeRecordApiService")
public class BillingRechargeRecordApiServiceImpl implements BillingRechargeRecordApiService {

    @Autowired
    private BillingRechargeRecordService billingRechargeRecordService;
    @Autowired
    private BillingPayInfoService billingPayInfoService;
    @Autowired
    private TransferAccountsApprovalApiService transferAccountsApprovalApiService;

    @Override
    public ServiceResponse<Page<BillingRechargeRecordDTO>> listRechargeRecord(RechargeRecordQO rechargeRecordQO) {
        com.github.pagehelper.Page<TabBillingRechargeRecord> page = billingRechargeRecordService.listRechargeRecord(rechargeRecordQO);
        Page<BillingRechargeRecordDTO> dtoPage = PageHelperUtils.changePageHelperToCurrentPage(page, BillingRechargeRecordDTO.class);
        return ServiceResponse.success(dtoPage);
    }

    @Override
    public ServiceResponse<BillingRechargeRecordDTO> getRechargeRecord(Integer rechargeRecordId) {
        TabBillingRechargeRecord billingRechargeRecord = billingRechargeRecordService.getRechargeRecord(rechargeRecordId);
        if (billingRechargeRecord == null) {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
        }
        BillingRechargeRecordDTO resultRechargeRecordDTO = EntityUtil.changeEntityByOrika(BillingRechargeRecordDTO.class, billingRechargeRecord);
        // 查询交易信息
        TabBillingPayInfo payInfo = billingPayInfoService.getByOrderSerialNumber(resultRechargeRecordDTO.getSerialNumber(), resultRechargeRecordDTO.getPayType());
        if (payInfo != null) {
            resultRechargeRecordDTO.setInvoiceStatus(payInfo.getInvoiceStatus());
            resultRechargeRecordDTO.setPayInfoSerialNumber(payInfo.getSerialNumber());
        }
        ServiceResponse<TransferAccountsApprovalDTO> service = transferAccountsApprovalApiService.getByOrderNumber(resultRechargeRecordDTO.getSerialNumber());
        TransferAccountsApprovalDTO serviceResult = service.getResult();
        if (serviceResult != null) {
            resultRechargeRecordDTO.setApprovalNumber(serviceResult.getApprovalNumber());
            resultRechargeRecordDTO.setApprovalCreateTime(serviceResult.getCreateTime());
            resultRechargeRecordDTO.setApprovalTime(serviceResult.getApprovalTime());
            resultRechargeRecordDTO.setPlannedAmount(serviceResult.getPlannedAmount());
            resultRechargeRecordDTO.setAccountAmount(serviceResult.getAccountAmount());
            resultRechargeRecordDTO.setApprovalId(serviceResult.getApprovalId());
            resultRechargeRecordDTO.setApprovalName(serviceResult.getApprovalName());
        }
        // 如果是微信支付 要把交易单号设置为交易流水
        if (resultRechargeRecordDTO.getPayType() == PayTypeEnum.WE_CHAT_PAY.getCode()) {
            resultRechargeRecordDTO.setPayInfoSerialNumber(resultRechargeRecordDTO.getOnlineSerialNumber());
        }

        return ServiceResponse.success(resultRechargeRecordDTO);
    }

    @Override
    public ServiceResponse<BillingRechargeRecordDTO> getRechargeRecordForApproval(String serialNumber) {
        TabBillingRechargeRecord record = billingRechargeRecordService.getRechargeRecordBySerialNumber(serialNumber);
        BillingRechargeRecordDTO recordDTO = EntityUtil.changeEntityByOrika(BillingRechargeRecordDTO.class, record);
        if (recordDTO != null) {
            TabBillingPayInfo payInfo = billingPayInfoService.getByOrderSerialNumber(record.getSerialNumber(), record.getPayType());
            recordDTO.setPayInfoSerialNumber(payInfo.getSerialNumber());
            return ServiceResponse.success(recordDTO);
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
    }

    @Override
    public ServiceResponse<Integer> cancelRechargeApproval(Integer rechargeRecordId) {
        TabBillingRechargeRecord billingRechargeRecord = billingRechargeRecordService.getRechargeRecord(rechargeRecordId);
        if (billingRechargeRecord == null) {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
        }
        transferAccountsApprovalApiService.cancelApproval(billingRechargeRecord.getSerialNumber());
        Integer line = billingRechargeRecordService.cancelRechargeApproval(rechargeRecordId, BillingAuditStatusEnum.CANCEL.getCode());
        return ServiceResponse.success(line);
    }

}
