package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.dto.CustomChatFiledDTO;
import com.gic.enterprise.dto.CustomUdeskFieldDTO;
import com.gic.enterprise.entity.TabCustomUdeskField;
import com.gic.enterprise.service.CustomUdeskFieldApiService;
import com.gic.enterprise.service.CustomUdeskFieldService;
import com.gic.redis.data.util.RedisUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service("customUdeskFieldApiService")
public class CustomUdeskFieldApiServiceImpl implements CustomUdeskFieldApiService {
    @Autowired
    private CustomUdeskFieldService customUdeskFieldService;

    @Override
    public ServiceResponse<List<CustomChatFiledDTO>> listAllField() {
        String key = "custom:chat:field";
        Object cache = RedisUtil.getCache(key);
        if(cache != null){
            List<CustomChatFiledDTO> list = (List<CustomChatFiledDTO>) cache;
            return ServiceResponse.success(list);
        }
        List<CustomChatFiledDTO> list = new ArrayList<>();
        list.add(new CustomChatFiledDTO("集团卡号", "cliqueCard"));
        list.add(new CustomChatFiledDTO("姓名", "userName"));
        list.add(new CustomChatFiledDTO("手机号码", "phoneNumber"));
        list.add(new CustomChatFiledDTO("会员卡号", "cardNo"));
        list.add(new CustomChatFiledDTO("用户信息", "userInfo"));
        list.add(new CustomChatFiledDTO("会员卡等级", "memberGrade"));
        list.add(new CustomChatFiledDTO("积分余额", "integralBalance"));
        list.add(new CustomChatFiledDTO("最近消费时间", "latestCunsumeTime"));
        list.add(new CustomChatFiledDTO("消费次数", "consumeTimes"));
        list.add(new CustomChatFiledDTO("消费总额", "consumeTotal"));
        RedisUtil.setCache(key, list);
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<Integer> saveEnterpriseField(CustomUdeskFieldDTO dto) {
        int i = this.customUdeskFieldService.saveEnterpriseField(EntityUtil.changeEntityByJSON(TabCustomUdeskField.class, dto));
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<List<CustomUdeskFieldDTO>> listEnterpriseField(Integer enterpriseId) {
        List<TabCustomUdeskField> list = this.customUdeskFieldService.listEnterpriseField(enterpriseId);
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(CustomUdeskFieldDTO.class, list));
    }

    @Override
    public ServiceResponse<CustomUdeskFieldDTO> getDetail(Integer id) {
        TabCustomUdeskField detail = this.customUdeskFieldService.getDetail(id);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(CustomUdeskFieldDTO.class, detail));
    }

    @Override
    public ServiceResponse<Integer> deleteField(Integer id) {
        int i = this.customUdeskFieldService.deleteFiled(id);
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Integer> updateStatus(Integer id, Integer status) {
        int i = this.customUdeskFieldService.updateStatus(id, status);
        return ServiceResponse.success(i);
    }
}
