package com.gic.enterprise.service.outer.impl;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import com.gic.enterprise.dto.data.*;
import com.gic.enterprise.entity.*;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.*;

@Service("dataConfigApiService")
public class DataConfigApiServiceImpl implements DataConfigApiService {
    @Autowired
    private DataMemberConsumeConfigService dataMemberConsumeConfigService;
    @Autowired
    private DataWechatMemberConfigService dataWechatMemberConfigService;
    @Autowired
    private DataActuallyPaidConfigService dataActuallyPaidConfigService;
    @Autowired
    private DataSingleEffectConfigService dataSingleEffectConfigService;
    @Autowired
    private DataPerformanceConfigService dataPerformanceConfigService;
    @Autowired
    private DataGenerationConfigService dataGenerationConfigService;

    @Override
    public ServiceResponse<Integer> configMemberConsume(DataMemberConsumeConfigDTO dto) {
        if (dto.getEnterpriseId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        TabDataMemberConsumeConfig record = dataMemberConsumeConfigService.getByEnterpriseId(dto.getEnterpriseId());
        if (record == null) {
            //还未配置，新增
            return ServiceResponse.success(dataMemberConsumeConfigService.saveMemberConsumeConfig(dto));
        }
        dto.setMemberConsumeConfigId(record.getMemberConsumeConfigId());
        dataMemberConsumeConfigService.editMemberConsumeConfig(dto);
        return ServiceResponse.success(record.getMemberConsumeConfigId());
    }

    @Override
    public ServiceResponse<DataMemberConsumeConfigDTO> getMemberConsumeConfigByEnterpriseId(Integer enterpriseId) {
        TabDataMemberConsumeConfig record = dataMemberConsumeConfigService.getByEnterpriseId(enterpriseId);
        if (record == null) {
            TabDataMemberConsumeConfig defaultRecord = dataMemberConsumeConfigService.getByEnterpriseId(-1);
            return ServiceResponse.success(EntityUtil.changeEntityNew(DataMemberConsumeConfigDTO.class, defaultRecord));
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(DataMemberConsumeConfigDTO.class, record));
    }

    @Override
    public ServiceResponse<Void> initMemberConsumeConfig(Integer enterpriseId) {
        TabDataMemberConsumeConfig defaultRecord = dataMemberConsumeConfigService.getByEnterpriseId(-1);
        DataMemberConsumeConfigDTO dto = EntityUtil.changeEntityNew(DataMemberConsumeConfigDTO.class, defaultRecord);
        dto.setMemberConsumeConfigId(null);
        dto.setEnterpriseId(enterpriseId);
        configMemberConsume(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> configActuallyPaid(List<DataActuallyPaidConfigDTO> dtoList) {
        if (CollectionUtils.isEmpty(dtoList)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "实付配置内容为空");
        }
        for (DataActuallyPaidConfigDTO dto : dtoList) {
            if (dto.getClassify() == null) {
                dto.setClassify(1);
            }
            dataActuallyPaidConfigService.delete(dto.getEnterpriseId(), dto.getClassify());
            dto.setConfigStatus(Optional.ofNullable(dto.getConfigStatus()).orElse(0));
            dataActuallyPaidConfigService.saveActuallyPaidConfig(dto);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<DataActuallyPaidConfigDTO>> listActuallyPaidConfig(Integer enterpriseId) {
        List<TabDataActuallyPaidConfig> list = dataActuallyPaidConfigService.listByEnterpriseId(enterpriseId);
        if (CollectionUtils.isEmpty(list)) {
            //如果没有配置，默认数据
            return ServiceResponse.success(EntityUtil.changeEntityListNew(DataActuallyPaidConfigDTO.class,
                    dataActuallyPaidConfigService.listByEnterpriseId(-1)));
        }
        return ServiceResponse.success(EntityUtil.changeEntityListNew(DataActuallyPaidConfigDTO.class, list));
    }

    @Override
    public ServiceResponse<Void> initActuallyPaidData(Integer enterpriseId) {
        List<TabDataActuallyPaidConfig> list = dataActuallyPaidConfigService.listByEnterpriseId(-1);
        list = list.stream().map(e -> e.setEnterpriseId(enterpriseId)).map(e -> e.setActuallyPaidConfigId(null))
                .collect(Collectors.toList());
        configActuallyPaid(EntityUtil.changeEntityListNew(DataActuallyPaidConfigDTO.class, list));
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> configSingleEffect(List<DataSingleEffectConfigDTO> dtoList) {
        if (CollectionUtils.isEmpty(dtoList)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "单效配置内容为空");
        }
        for (DataSingleEffectConfigDTO dto : dtoList) {
            dataSingleEffectConfigService.deleteSingleEffectConfig(dto.getEnterpriseId(), dto.getOrderGrain());
            dataSingleEffectConfigService.saveSingleEffectConfig(dto);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<DataSingleEffectConfigDTO>> getSingleEffectConfig(Integer enterpriseId) {
        List<TabDataSingleEffectConfig> record = dataSingleEffectConfigService.getByEnterpriseId(enterpriseId);
        if (CollectionUtils.isEmpty(record)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(DataSingleEffectConfigDTO.class,
                    dataSingleEffectConfigService.getByEnterpriseId(-1)));
        }
        return ServiceResponse.success(EntityUtil.changeEntityListNew(DataSingleEffectConfigDTO.class, record));
    }

    @Override
    public ServiceResponse<Void> initSingleEffectConfig(Integer enterpriseId) {
        List<DataSingleEffectConfigDTO> dtoList = EntityUtil.changeEntityListNew(DataSingleEffectConfigDTO.class,
                dataSingleEffectConfigService.getByEnterpriseId(-1));
        dtoList = dtoList.stream().map(e -> e.setEnterpriseId(enterpriseId))
                .map(e -> e.setSingleEffectConfigId(null)).collect(Collectors.toList());
        configSingleEffect(dtoList);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> configPerformance(DataPerformanceConfigDTO dto) {
        TabDataPerformanceConfig record = dataPerformanceConfigService.getByEnterpriseId(dto.getEnterpriseId());
        if (record == null) {
            dataPerformanceConfigService.savePerformanceConfig(dto);
            return ServiceResponse.success();
        }
        dto.setPerformanceConfigId(record.getPerformanceConfigId());
        dataPerformanceConfigService.editPerformanceConfig(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<DataPerformanceConfigDTO> getPerformanceConfig(Integer enterpriseId) {
        TabDataPerformanceConfig record = dataPerformanceConfigService.getByEnterpriseId(enterpriseId);
        if (record == null) {
            return ServiceResponse.success(EntityUtil.changeEntityNew(DataPerformanceConfigDTO.class,
                    dataPerformanceConfigService.getByEnterpriseId(-1)));
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(DataPerformanceConfigDTO.class, record));
    }

    @Override
    public ServiceResponse<Void> initPerformanceConfig(Integer enterpriseId) {
        DataPerformanceConfigDTO dto = EntityUtil.changeEntityNew(DataPerformanceConfigDTO.class,
                dataPerformanceConfigService.getByEnterpriseId(-1));
        dto.setPerformanceConfigId(null);
        dto.setEnterpriseId(enterpriseId);
        configPerformance(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveGenerationConfig(List<DataGenerationConfigDTO> dtoList) {
        if (CollectionUtils.isEmpty(dtoList)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "年龄段配置内容为空");
        }
        dataGenerationConfigService.deleteGenerationConfig(dtoList.get(0).getEnterpriseId());
        dtoList.stream()
                .forEach(e -> dataGenerationConfigService.saveGenerationConfig(e));
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<DataGenerationConfigDTO>> listGenerationConfig(Integer enterpriseId) {
        List<TabDataGenerationConfig> list = dataGenerationConfigService.listGenerationByEnterpriseId(enterpriseId);
        return ServiceResponse.success(CollectionUtils.isEmpty(list) ?
                EntityUtil.changeEntityListNew(DataGenerationConfigDTO.class, dataGenerationConfigService.listGenerationByEnterpriseId(-1)) :
        EntityUtil.changeEntityListNew(DataGenerationConfigDTO.class, list));
    }

    @Override
    public ServiceResponse<Void> initGenerationConfig(Integer enterpriseId) {
        List<DataGenerationConfigDTO> dtoList = EntityUtil.changeEntityListNew(DataGenerationConfigDTO.class,
                dataGenerationConfigService.listGenerationByEnterpriseId(-1));
        saveGenerationConfig(dtoList.stream().map(e -> e.setEnterpriseId(enterpriseId))
                .map(e -> e.setGenerationConfigId(null)).collect(Collectors.toList()));
        return ServiceResponse.success();
    }
}
