package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.authcenter.api.dto.GicUserDTO;
import com.gic.authcenter.api.service.GicUserService;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.constant.IndexModuleObjectTypeEnum;
import com.gic.enterprise.dto.DictDTO;
import com.gic.enterprise.dto.IndexModuleDTO;
import com.gic.enterprise.entity.TabIndexModule;
import com.gic.enterprise.entity.TabIndexModuleEnterprise;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.DictApiService;
import com.gic.enterprise.service.IndexModuleApiService;
import com.gic.enterprise.service.IndexModuleEnterpriseService;
import com.gic.enterprise.service.IndexModuleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-13 15:55
 */
@Service("indexModuleApiService")
public class IndexModuleApiServiceImpl implements IndexModuleApiService {

    @Autowired
    private IndexModuleService indexModuleService;
    @Autowired
    private IndexModuleEnterpriseService indexModuleEnterpriseService;
    @Autowired
    private GicUserService gicUserService;
    @Autowired
    private DictApiService dictApiService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> saveIndexModule(IndexModuleDTO indexModuleDTO) {
        boolean nameIsRepeat = indexModuleService.validModuleNameIsRepeat(indexModuleDTO.getIndexModuleName());
        if (nameIsRepeat) {
            return EnterpriseServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), "组件名称重复");
        }
        boolean codeIsRepeat = indexModuleService.validModuleCodeIsRepeat(indexModuleDTO.getIndexModuleCode());
        if (codeIsRepeat) {
            return EnterpriseServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), "组件code重复");
        }
        if (indexModuleDTO.getIndexModuleId() == null) {
            double sort = indexModuleService.getMinSort();
            indexModuleDTO.setSort(sort);
            Integer id = indexModuleService.saveIndexModule(indexModuleDTO);
            indexModuleDTO.setIndexModuleId(id);
        } else {
            indexModuleService.updateIndexModule(indexModuleDTO);
        }
        if (IndexModuleObjectTypeEnum.PART.getCode() == indexModuleDTO.getObjectType()) {
            indexModuleEnterpriseService.save(indexModuleDTO.getIndexModuleId(), indexModuleDTO.getEnterpriseIdList());
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> delIndexModule(Integer indexModuleId) {
        indexModuleService.delIndexModule(indexModuleId);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<IndexModuleDTO> getIndexModule(Integer indexModuleId) {
        TabIndexModule indexModule = indexModuleService.getIndexModule(indexModuleId);
        IndexModuleDTO indexModuleDTO = EntityUtil.changeEntityByJSON(IndexModuleDTO.class, indexModule);
        List<TabIndexModuleEnterprise> indexModuleEnterpriseList = indexModuleEnterpriseService.listIndexModuleEnterprise(indexModuleId);
        List<Integer> enterpriseIdList = Optional.ofNullable(indexModuleEnterpriseList).orElse(Collections.emptyList()).stream().map(TabIndexModuleEnterprise::getEnterpriseId).collect(Collectors.toList());
        indexModuleDTO.setEnterpriseIdList(enterpriseIdList);
        return EnterpriseServiceResponse.success(indexModuleDTO);
    }

    @Override
    public ServiceResponse<List<IndexModuleDTO>> listIndexModule() {
        List<TabIndexModule> indexModuleList = indexModuleService.listIndexModule();
        List<IndexModuleDTO> list = EntityUtil.changeEntityListByJSON(IndexModuleDTO.class, indexModuleList);
        for (IndexModuleDTO indexModuleDTO : list) {
            GicUserDTO user = gicUserService.getUserByLoginName(indexModuleDTO.getCreateUserLoginName());
            if (user != null) {
                indexModuleDTO.setCreateUserRealName(user.getRealName());
            }
        }
        return EnterpriseServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<Void> sortIndexModule(Integer indexModuleId, Integer seq) {
        this.indexModuleService.sortIndexModule(indexModuleId, seq);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<IndexModuleDTO>> listIndexModuleEnterprise(Integer enterpriseId) {
        List<TabIndexModule> indexModuleList = this.indexModuleService.listIndexModuleEnterprise(enterpriseId);
        List<IndexModuleDTO> dtoList = EntityUtil.changeEntityListByJSON(IndexModuleDTO.class, indexModuleList);
        for (IndexModuleDTO indexModuleDTO : dtoList) {
            ServiceResponse<DictDTO> dict = dictApiService.getDict(Constants.MODULE_TEMPLATE_CODE, indexModuleDTO.getModuleTemplateCode());
            indexModuleDTO.setModuleTemplateName(dict.getResult().getKey());
        }
        return EnterpriseServiceResponse.success(dtoList);
    }
}