package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.enterprise.constant.AccountStandardEnum;
import com.gic.enterprise.constant.SmsAppEnum;
import com.gic.enterprise.dto.InternationalSmsRecordDTO;
import com.gic.enterprise.service.AccountStandardService;
import com.gic.enterprise.service.BillingAccountService;
import com.gic.enterprise.service.BillingInternationalSmsRecordService;
import com.gic.enterprise.service.InternationalSmsRecordApiService;
import com.gic.member.api.dto.MemberUserDTO;
import com.gic.member.api.dto.UserOwnerDTO;
import com.gic.member.api.service.MemberUserApiService;
import com.gic.member.api.service.UserOwnerApiService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.store.dto.StoreDTO;
import com.gic.store.service.StoreApiService;
import com.gic.store.utils.valid.ValidUtil;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-09 9:26
 */
@Service("internationalSmsRecordApiService")
public class InternationalSmsRecordApiServiceImpl implements InternationalSmsRecordApiService {

    private Logger logger = LogManager.getLogger(InternationalSmsRecordApiServiceImpl.class);

    @Autowired
    private BillingAccountService billingAccountService;
    @Autowired
    private BillingInternationalSmsRecordService billingInternationalSmsRecordService;
    @Autowired
    private MemberUserApiService memberUserApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private UserOwnerApiService userOwnerApiService;
    @Autowired
    private AccountStandardService accountStandardService;

    @Override
    public ServiceResponse<Void> saveInternationalSmsRecord(InternationalSmsRecordDTO internationalSmsRecordDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(internationalSmsRecordDTO, InternationalSmsRecordDTO.InternationalSmsRecordValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            String json = JSON.toJSONString(internationalSmsRecordDTO);
            logger.info("收到海外短信计费信息：{}", json);
            client.sendMessage("InternationalSmsMq", json);
//            this.run(json);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> run(String arg) {
        // 缓存中计费
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            logger.info("海外短信缓存扣费：{}", arg);
            InternationalSmsRecordDTO internationalSmsRecordDTO = JSON.parseObject(arg, InternationalSmsRecordDTO.class);
            // 判断是扣套餐包还是扣余额
            Integer fee = accountStandardService.queryUnitFee(internationalSmsRecordDTO.getEnterpriseId(), AccountStandardEnum.SMS.getCode());
            billingAccountService.deductAccountByCache(fee, internationalSmsRecordDTO.getEnterpriseId());
            internationalSmsRecordDTO.setFee((double)fee / 1000);

            Long smsRecordId = billingInternationalSmsRecordService.save(internationalSmsRecordDTO);
            internationalSmsRecordDTO.setSmsRecordId(smsRecordId);

            // 队列
            client.sendMessage("InternationalSmsDbMq", JSON.toJSONString(internationalSmsRecordDTO));
            billingInternationalSmsRecordService.updateCacheStatus(internationalSmsRecordDTO.getSmsRecordId());
//            this.executeDatabase(arg);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> executeDatabase(String arg) {
        logger.info("海外短信数据库扣费：{}", arg);
        InternationalSmsRecordDTO internationalSmsRecordDTO = JSON.parseObject(arg, InternationalSmsRecordDTO.class);
        internationalSmsRecordDTO.setDeductionTime(new Date());
        // 应用名字
        internationalSmsRecordDTO.setAppName(SmsAppEnum.getNameByCode(internationalSmsRecordDTO.getAppCode()));
        // 会员名字
        setMemberName(internationalSmsRecordDTO);
        // 操作人
        setUserName(internationalSmsRecordDTO);
        // 门店
        setStore(internationalSmsRecordDTO);
        billingAccountService.deductAccountByDatabase(internationalSmsRecordDTO.getFee(), internationalSmsRecordDTO.getEnterpriseId());
        billingInternationalSmsRecordService.updateDatabaseStatus(internationalSmsRecordDTO);
        return ServiceResponse.success();
    }

    private void setStore(InternationalSmsRecordDTO smsRecordDTO) {
        ServiceResponse<UserOwnerDTO> userOwnerResp = userOwnerApiService.getUserMaxOwner(smsRecordDTO.getEnterpriseId(), smsRecordDTO.getEcuId(), 1, smsRecordDTO.getMemberCardId(), null);
        if (userOwnerResp.isSuccess()) {
            UserOwnerDTO userOwnerDTO = userOwnerResp.getResult();
            if (userOwnerDTO != null) {
                ServiceResponse<StoreDTO> storeResp = storeApiService.getStoreById(smsRecordDTO.getEnterpriseId(), Integer.valueOf(userOwnerDTO.getOwnerId()));
                if (storeResp.isSuccess()) {
                    StoreDTO storeDTO = storeResp.getResult();
                    if (storeDTO != null) {
                        smsRecordDTO.setStoreId(storeDTO.getStoreId());
                        smsRecordDTO.setStoreName(storeDTO.getStoreName());
                        smsRecordDTO.setStoreGroupId(storeDTO.getStoreGroupId());
                        smsRecordDTO.setStoreGroupName(storeDTO.getStoreGroupName());
                        smsRecordDTO.setStoreBrandName(storeDTO.getStoreBrandNames());
                    }
                } else {
                    logger.info("storeApiService.getStoreById 调用失败 :{}", storeResp.getMessage());
                }
            }
        } else {
            logger.info("UserOwnerApiService.getUserMaxOwner 调用失败:{}", userOwnerResp.getMessage());
        }
    }

    /**
     * 操作人
     */
    private void setUserName(InternationalSmsRecordDTO smsRecordDTO) {
        if (smsRecordDTO.getUserId() != null) {
            ServiceResponse<UserDTO> userResp = userApiService.getUserById(smsRecordDTO.getUserId());
            if (userResp.isSuccess()) {
                UserDTO userDTO = userResp.getResult();
                if (userDTO != null) {
                    smsRecordDTO.setUserName(smsRecordDTO.getUserName());
                }
            } else {
                logger.info("userId:{}, UserApiService.getUserById 调用失败:{}", smsRecordDTO.getUserId(), userResp.getMessage());
            }
        }
    }

    /**
     * 会员名字和会员卡号
     */
    private void setMemberName(InternationalSmsRecordDTO smsRecordDTO) {
        if (smsRecordDTO.getMcuId() != null) {
            ServiceResponse<MemberUserDTO> memberUserDTOServiceResponse = memberUserApiService.getByMcuId(smsRecordDTO.getMcuId(), smsRecordDTO.getEnterpriseId());
            if (memberUserDTOServiceResponse.isSuccess()) {
                MemberUserDTO memberUserDTO = memberUserDTOServiceResponse.getResult();
                if (memberUserDTO != null) {
                    smsRecordDTO.setMemberName(memberUserDTO.getName());
                    smsRecordDTO.setMemberCardName(memberUserDTO.getCardName());
                }
            } else {
                logger.warn("mcuId:{}, enterpriseId:{}, memberUserApiService.getByMcuId调用失败:{}, ", smsRecordDTO.getMcuId(), smsRecordDTO.getEnterpriseId(), memberUserDTOServiceResponse.getMessage());
            }
        }
    }

}
