package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.LinkEnterpriseTypeEnum;
import com.gic.enterprise.dto.LinkDTO;
import com.gic.enterprise.entity.TabLink;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.LinkQO;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.LinkApiService;
import com.gic.enterprise.service.LinkSceneService;
import com.gic.enterprise.service.LinkService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-25 14:21
 */
@Service("linkApiService")
public class LinkApiServiceImpl implements LinkApiService {

    @Autowired
    private LinkService linkService;
    @Autowired
    private LinkSceneService linkSceneService;

    @Override
    public ServiceResponse<Void> saveLinkType(String name, Integer type) {
        boolean nameIsRepeat = linkService.validNameIsRepeat(name, null);
        if (nameIsRepeat) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
        }
        Integer linkId = linkService.saveLinkType(name, type);
//        linkSceneService.editLinkEnterprise(linkId, Collections.singletonList(-1));
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateLinkType(Integer linkId, String name, Integer type) {
        boolean nameIsRepeat = linkService.validNameIsRepeat(name, linkId);
        if (nameIsRepeat) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
        }
        int i = linkService.updateLinkType(linkId, name, type);
        if (i > 0) {
            return EnterpriseServiceResponse.success();
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
    }

//    @Override
//    public ServiceResponse<Void> saveLinkMenu(Integer parentId, String name, Integer type) {
//        boolean nameIsRepeat = linkService.validNameIsRepeat(name, null);
//        if (nameIsRepeat) {
//            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
//        }
//        if (type == null) {
//            type = LinkTypeEnum.HIERARCHICAL.getCode();
//        }
//        Integer linkId = linkService.saveLinkMenu(parentId, name, type);
//        linkSceneService.editLinkEnterprise(linkId, Collections.singletonList(-1));
//        return EnterpriseServiceResponse.success();
//    }
//
//    @Override
//    public ServiceResponse<Void> updateLinkMenu(Integer linkId, String name, Integer type) {
//        boolean nameIsRepeat = linkService.validNameIsRepeat(name, linkId);
//        if (nameIsRepeat) {
//            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
//        }
//        if (type == null) {
//            type = LinkTypeEnum.HIERARCHICAL.getCode();
//        }
//        int i = linkService.updateLinkMenu(linkId, name, type);
//        if (i > 0) {
//            return EnterpriseServiceResponse.success();
//        } else {
//            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
//        }
//    }

    @Override
    public ServiceResponse<Void> saveLinkChildMenu(LinkDTO linkDTO) {
        boolean nameIsRepeat = linkService.validNameIsRepeat(linkDTO.getName(), null);
        if (nameIsRepeat) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
        }
        Integer linkId = linkService.saveLinkChildMenu(linkDTO);
//        linkSceneService.editLinkEnterprise(linkId, Collections.singletonList(-1));
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateLinkChildMenu(LinkDTO linkDTO) {
        boolean nameIsRepeat = linkService.validNameIsRepeat(linkDTO.getName(), linkDTO.getLinkId());
        if (nameIsRepeat) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
        }
        int i = linkService.updateLinkChildMenu(linkDTO);
        if (i > 0) {
            return EnterpriseServiceResponse.success();
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
    }

    @Override
    public ServiceResponse<Void> delLink(List<Integer> linkIdList) {
        if (CollectionUtils.isEmpty(linkIdList)) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        List<Integer> allIdList = linkService.getAllLinkIdsByParentIds(linkIdList);
        if (CollectionUtils.isNotEmpty(allIdList)) {
            linkService.delLinkByIdList(allIdList);
            linkSceneService.delRelByLinkIdList(allIdList);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<LinkDTO>> listLinkType() {
        List<TabLink> linkList = linkService.listLinkType();
        return EnterpriseServiceResponse.success(EntityUtil.changeEntityListByJSON(LinkDTO.class, linkList));
    }

    @Override
    public ServiceResponse<Page<LinkDTO>> listLinkMenu(LinkQO linkQO) {
        linkQO.setPageNum(linkQO.getCurrentPage());
        com.github.pagehelper.Page<TabLink> page = linkService.listLinkMenu(linkQO);
        if (CollectionUtils.isNotEmpty(page)) {
            Page<LinkDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, LinkDTO.class);
            List<LinkDTO> linkList = resultPage.getResult();
            List<Integer> linkIdList = linkList.stream().map(LinkDTO::getLinkId).collect(Collectors.toList());
            List<TabLink> childList = this.linkService.getLinkByParentIds(linkIdList);
            if (CollectionUtils.isNotEmpty(childList)) {
                Map<Integer, List<LinkDTO>> listMap = EntityUtil.changeEntityListByJSON(LinkDTO.class, childList).stream().collect(Collectors.groupingBy(LinkDTO::getParentId, Collectors.toList()));
                for (LinkDTO linkDTO : linkList) {
                    List<LinkDTO> childListByLinkId = listMap.get(linkDTO.getLinkId());
                    if (childListByLinkId != null) {
                        linkDTO.setChildList(childListByLinkId);
                    }
                }
            }
            return EnterpriseServiceResponse.success(resultPage);
        } else {
            return EnterpriseServiceResponse.success(new Page<>(linkQO.getCurrentPage(), linkQO.getPageSize()));
        }
    }

    @Override
    public ServiceResponse<LinkDTO> getLink(Integer linkId) {
        TabLink tabLink = linkService.getById(linkId);
        if (tabLink == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        LinkDTO linkDTO = EntityUtil.changeEntityByJSON(LinkDTO.class, tabLink);
        List<Integer> enterpriseList = linkSceneService.listByLinkId(linkDTO.getLinkId());
        linkDTO.setEnterpriseList(enterpriseList);
        return EnterpriseServiceResponse.success(linkDTO);
    }

    @Override
    public ServiceResponse<Void> editLinkEnterprise(Integer linkId, Integer enterpriseType, List<Integer> enterpriseIdList) {
        LinkDTO linkDTO = new LinkDTO();
        linkDTO.setLinkId(linkId);
        linkDTO.setEnterpriseType(enterpriseType);
        this.linkService.updateLinkMenu(linkDTO);
        if (LinkEnterpriseTypeEnum.ALL.getCode().equals(enterpriseType)) {
            this.linkSceneService.delRelByLinkIdList(Collections.singletonList(linkId));
        } else if (LinkEnterpriseTypeEnum.PART.getCode().equals(enterpriseType)) {
            this.linkSceneService.editLinkEnterprise(linkId, enterpriseIdList);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<LinkDTO>> listByGic(Integer enterpriseId, Integer parentId) {
        List<TabLink> linkList = this.linkService.listByGic(enterpriseId, parentId);
        return EnterpriseServiceResponse.success(EntityUtil.changeEntityListByJSON(LinkDTO.class, linkList));
    }

    @Override
    public ServiceResponse<List<LinkDTO>> treeByGic(Integer enterpriseId, Integer objectType) {
        List<TabLink> linkList = this.linkService.treeByGic(enterpriseId, objectType);
        List<LinkDTO> dtoList = EntityUtil.changeEntityListByJSON(LinkDTO.class, linkList);
        List<LinkDTO> treeList = toTree(dtoList);
        return EnterpriseServiceResponse.success(treeList);
    }

    private List<LinkDTO> toTree(List<LinkDTO> list) {
        if (list == null || list.isEmpty()) {
            return null;
        }
        List<LinkDTO> treeList = new ArrayList<>();
        for (LinkDTO tree : list) {
            if (tree.getParentId() == 0) {
                treeList.add(tree);
            }
        }
        Collections.sort(treeList);
        for (LinkDTO tree : list) {
            toTreeChildren(treeList, tree);
        }
        return treeList;
    }

    /**
     * 寻找子节点
     *
     * @param treeList
     * @param tree
     */
    private void toTreeChildren(List<LinkDTO> treeList, LinkDTO tree) {
        for (LinkDTO node : treeList) {
            if (node.getLinkId().equals(tree.getParentId())) {
                if (node.getChildList() == null) {
                    node.setChildList(new ArrayList<>());
                }
                node.getChildList().add(tree);
            }
            if (node.getChildList() != null) {
                toTreeChildren(node.getChildList(), tree);
                Collections.sort(node.getChildList());
            }
        }
    }

}
