package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.ProjectDTO;
import com.gic.enterprise.dto.ProjectItemDTO;
import com.gic.enterprise.entity.TabProject;
import com.gic.enterprise.entity.TabProjectItem;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.ProjectItemQO;
import com.gic.enterprise.service.ProjectApiService;
import com.gic.enterprise.service.ProjectItemService;
import com.gic.enterprise.service.ProjectService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-09-17 9:44
 */
@Service("projectApiService")
public class ProjectApiServiceImpl implements ProjectApiService{

    private Logger logger = LogManager.getLogger(ProjectApiServiceImpl.class);

    @Autowired
    private ProjectService projectService;
    @Autowired
    private ProjectItemService projectItemService;

    @Override
    public ServiceResponse<Integer> saveOrUpdateProject(ProjectDTO projectDTO) {
        boolean repeat = projectService.validProjectNameIsRepeat(projectDTO.getProjectName(), projectDTO.getProjectId());
        if (repeat) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "项目名称不能重复");
        }
        if (projectDTO.getProjectId() == null) {
            Integer projectId = projectService.save(projectDTO);
            return ServiceResponse.success(projectId);
        } else {
            Integer line = projectService.update(projectDTO);
            return ServiceResponse.success(line);
        }
    }

    @Override
    public ServiceResponse<Integer> delProject(Integer projectId) {
        ProjectItemDTO projectItemDTO = new ProjectItemDTO();
        projectItemDTO.setProjectId(projectId);
        TabProjectItem projectItem = projectItemService.getFirstProjectItem(projectItemDTO);
        if (projectItem != null) {
            logger.info("projectId：{}, 下有审批项", projectId);
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "受审项目下有审批项");
        }
        Integer line = projectService.delProject(projectId);
        return ServiceResponse.success(line);
    }

    @Override
    public ServiceResponse<Page<ProjectDTO>> listProject(ProjectDTO projectDTO, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page<TabProject> page = projectService.listProject(projectDTO, pageNum, pageSize);
        Page<ProjectDTO> projectPage = PageHelperUtils.changePageHelperToCurrentPage(page, ProjectDTO.class);
        if (CollectionUtils.isNotEmpty(page)) {
            List<Integer> projectIds = page.stream().map(TabProject::getProjectId).collect(Collectors.toList());
            Map<Integer, Integer> totalCountMap = projectItemService.getProjectItemCountByProjectIds(projectIds, null);
            Map<Integer, Integer> countUseMap = projectItemService.getProjectItemCountByProjectIds(projectIds, GlobalInfo.DATA_STATUS_NORMAL);
            List<ProjectDTO> resultList = projectPage.getResult();
            int index = (pageNum - 1) * pageSize + 1;
            int i = 0;
            for (ProjectDTO dto : resultList) {
                Integer totalCount = totalCountMap.get(dto.getProjectId());
                dto.setItemTotalCount(totalCount == null ? 0 : totalCount);
                Integer useCount = countUseMap.get(dto.getProjectId());
                dto.setItemUseCount(useCount == null ? 0 : useCount);
                ProjectItemQO itemQO = new ProjectItemQO();
                itemQO.setProjectId(dto.getProjectId());
                itemQO.setCurrentPage(1);
                itemQO.setPageSize(3);
                com.github.pagehelper.Page<TabProjectItem> itemList = projectItemService.listProjectItem(itemQO);
                dto.setItemList(EntityUtil.changeEntityListNew(ProjectItemDTO.class, itemList));
                dto.setSeq(index + i);
                i++;
            }
        }
        return ServiceResponse.success(projectPage);
    }

    @Override
    public ServiceResponse<List<ProjectDTO>> listAllProject() {
        List<TabProject> tabProjectList = projectService.listAllProject();
        List<ProjectDTO> projectList = EntityUtil.changeEntityListByJSON(ProjectDTO.class, tabProjectList);
        if (CollectionUtils.isEmpty(projectList)) {
            return ServiceResponse.success(Collections.emptyList());
        }
        List<TabProjectItem> tabItemList = projectItemService.listAllProjectItem();
        List<ProjectItemDTO> itemList = EntityUtil.changeEntityListByJSON(ProjectItemDTO.class, tabItemList);
        Map<Integer, List<ProjectItemDTO>> group = Optional.ofNullable(itemList).orElse(Collections.emptyList()).stream()
                .filter(e -> e.getUseStatus() == 1).collect(Collectors.groupingBy(ProjectItemDTO::getProjectId, Collectors.mapping(e -> e, Collectors.toList())));
//        Map<Integer, List<ProjectItemDTO>> group = CollectionUtil.group(itemList, "projectId");
        for (ProjectDTO projectDTO : projectList) {
            projectDTO.setItemList(group.get(projectDTO.getProjectId()));
        }
        List<ProjectDTO> result = projectList.stream().filter(e -> CollectionUtils.isNotEmpty(e.getItemList())).collect(Collectors.toList());
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<Void> setProjectSort(Integer projectId, Integer seq) {
        this.projectService.setProjectSort(projectId, seq);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> setProjectItemSort(Integer projectItemId, Integer seq) {
        this.projectItemService.setProjectItemSort(projectItemId, seq);
        return ServiceResponse.success();
    }

}
