package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.PushMessageDTO;
import com.gic.enterprise.dto.PushTypeDTO;
import com.gic.enterprise.dto.UserMessageDTO;
import com.gic.enterprise.entity.TabPushClassify;
import com.gic.enterprise.entity.TabPushMessage;
import com.gic.enterprise.entity.TabPushUserMessage;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.*;
import com.gic.mq.sdk.GicMQClient;
import com.gic.utils.JobUtils;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service("pushMessageApiService")
public class PushMessageApiServiceImpl implements PushMessageApiService {
    private static final Logger log = LogManager.getLogger(PushMessageApiServiceImpl.class);
    @Autowired
    private PushMessageService pushMessageService;
    @Autowired
    private PushTypeService pushTypeService;
    @Autowired
    private PushUserMessageService pushUserMessageService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private UserApiService userApiService;

    @Override
    public ServiceResponse<Integer> savePushType(PushTypeDTO pushTypeDTO) {
        List<TabPushClassify> list = this.pushTypeService.listPushTypeByName(pushTypeDTO.getClassifyName());
        if(CollectionUtils.isNotEmpty(list)){
            if(list.size() > 1){
                return ServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), "类型名称已经重复");
            }
            if(list.size() == 1 && list.get(0).getId() != pushTypeDTO.getId()){
                return ServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), "类型名称已经重复");
            }
        }
        int i = this.pushTypeService.savePushType(EntityUtil.changeEntityByJSON(TabPushClassify.class, pushTypeDTO));
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Page<PushTypeDTO>> pagePushType(String search, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        com.github.pagehelper.Page<TabPushClassify> page = this.pushTypeService.pagePushType(search);
        Page<PushTypeDTO> pushTypeDTOPage = PageHelperUtils.changePageHelperToCurrentPage(page, PushTypeDTO.class);
        return ServiceResponse.success(pushTypeDTOPage);
    }

    @Override
    public ServiceResponse<PushTypeDTO> getPushTypeById(Integer pushTypeId) {
        TabPushClassify pushType = this.pushTypeService.getById(pushTypeId);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(PushTypeDTO.class, pushType));
    }

    @Override
    public ServiceResponse<Integer> deletePushType(Integer pushTypeId) {
        TabPushClassify pushType = this.pushTypeService.getById(pushTypeId);
        if("系统消息".equals(pushType.getClassifyName())){
            return ServiceResponse.success();
        }
        int i = this.pushTypeService.deletePushType(pushTypeId);
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Integer> savePushMessage(PushMessageDTO pushMessageDTO, int saveType) {
        if(pushMessageDTO.getMessageId() != null){
            TabPushMessage tabPushMessage = this.pushMessageService.getById(pushMessageDTO.getMessageId());
            if(tabPushMessage.getPushStatus() == 1){
                return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "已发送通知不可修改");
            }
        }
        if(saveType == 1){
            if(pushMessageDTO.getPushTimeFlag() == 1){
                pushMessageDTO.setPushStatus(1);  //已发送
            }else {
                pushMessageDTO.setPushStatus(2);  //待发送
            }
        }else {
            pushMessageDTO.setPushStatus(3);  //草稿箱
        }
        TabPushMessage tabPushMessage = EntityUtil.changeEntityByJSON(TabPushMessage.class, pushMessageDTO);
        int i = this.pushMessageService.savePushMesasge(tabPushMessage);
        if(saveType == 1){
            if(pushMessageDTO.getPushTimeFlag() == 1){
                this.pushMessage(pushMessageDTO.getPushEnterpriseIds(), tabPushMessage.getMessageId());
            }else {
                if(pushMessageDTO.getPushTime() != null){
                    String cron = new SimpleDateFormat("ss mm HH dd MM ? yyyy").format(pushMessageDTO.getPushTime());
                    JSONObject json = new JSONObject();
                    json.put("enterpriseIds", pushMessageDTO.getPushEnterpriseIds());
                    json.put("messageId", tabPushMessage.getMessageId());
                    JobUtils.addJob("通知发送", cron, "com.gic.enterprise.service.PushMessageApiService",
                            "pushMessageTask", JSON.toJSONString(json),"");
                }
            }
        }
        return ServiceResponse.success(i);
    }

    public ServiceResponse<Void> pushMessageTask(String param){
        JSONObject json = JSON.parseObject(param);
        String enterpriseIds = json.getString("enterpriseIds");
        Integer messageId = json.getInteger("messageId");
        this.pushMessage(enterpriseIds, messageId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> consumeMessageTask(String param) {
        JSONObject json = JSON.parseObject(param);
        Integer enterpriseId = json.getInteger("enterpriseId");
        Integer messageId = json.getInteger("messageId");
        TabPushMessage tabPushMessage = this.pushMessageService.getById(messageId);
        TabPushClassify pushClassify = this.pushTypeService.getById(tabPushMessage.getPushClassifyId());
        if(tabPushMessage.getPushStatus() == 1){
            ServiceResponse<List<Integer>> response = this.userApiService.listAllUser(enterpriseId);
            if(response.isSuccess() && CollectionUtils.isNotEmpty(response.getResult())){
                for(Integer userId : response.getResult()){
                    this.pushUserMessageService.save(enterpriseId, userId, messageId, pushClassify.getClassifyName(),
                            tabPushMessage.getTitle(), tabPushMessage.getContent());
                }
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<UserMessageDTO>> pageUserMessage(Integer userId, String search, Integer status,
                                                                 Date startTime, Date endTime, Integer pushClassifyId,
                                                                 Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page<UserMessageDTO> page = this.pushMessageService.pageUserMessage(search, status,
                startTime, endTime, pushClassifyId, userId, pageNum, pageSize);
        Page<UserMessageDTO> userMessageDTOPage = PageHelperUtils.changePageHelperToCurrentPage(page);
        return ServiceResponse.success(userMessageDTOPage);
    }

    @Override
    public ServiceResponse<Void> recallMessage(Integer messageId) {
        this.pushUserMessageService.reCallMessage(messageId);
        TabPushMessage tabPushMessage = new TabPushMessage();
        tabPushMessage.setMessageId(messageId);
        tabPushMessage.setPushStatus(3);
        this.pushMessageService.savePushMesasge(tabPushMessage);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> totalUserMessage(Integer userId, String search, Integer status,
                                                     Date startTime, Date endTime, Integer pushClassifyId) {
        return ServiceResponse.success(this.pushUserMessageService.totalUserMessage(userId, search, status, startTime, endTime, pushClassifyId));
    }

    @Override
    public ServiceResponse<Integer> pushUserMessage(Integer enterpriseId, Integer userId, String classify,
                                                    String title, String content) {
        if(enterpriseId == null || userId == null || StringUtils.isBlank(title) || StringUtils.isBlank(content)){
            return ServiceResponse.failure(ErrorCode.MISS_PARAMETER.getCode(), ErrorCode.MISS_PARAMETER.getErrorMsg());
        }
        this.pushTypeService.saveClassify(classify);
        int i = this.pushUserMessageService.save(enterpriseId, userId, null, classify, title, content);
        if(i > 0){
            return ServiceResponse.success(i);
        }else {
            return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), ErrorCode.SYSTEM_ERROR.getErrorMsg());
        }
    }

    @Override
    public ServiceResponse<Integer> pushMessage(Integer enterpriseId, String classify, String title, String content) {
        ServiceResponse<List<Integer>> response = this.userApiService.listAllUser(enterpriseId);
        if(response.isSuccess() && CollectionUtils.isNotEmpty(response.getResult())){
            for(Integer userId : response.getResult()){
                this.pushUserMessageService.save(enterpriseId, userId, null, classify,title, content);
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> readAll(Integer userId) {
        int i = this.pushUserMessageService.readAll(userId);
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Integer> readMessage(Integer messageId) {
        int i = this.pushUserMessageService.readMessage(messageId);
        if(i > 0){
            TabPushUserMessage message = this.pushUserMessageService.getById(messageId);
            if(message != null && message.getMessageId() != null){
                this.pushMessageService.readCount(message.getMessageId());
            }
        }
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Page<PushMessageDTO>> pagePushMessage(String search, Integer pushStatus, Date startTime, Date endTime,
                                                                 Integer pageNum, Integer pageSize, Integer pushClassifyId) {
        PageHelper.startPage(pageNum, pageSize);
        com.github.pagehelper.Page<TabPushMessage> page = this.pushMessageService.pagePushMesage(search, pushStatus, startTime, endTime, pushClassifyId);
        Page<PushMessageDTO> pushMessageDTOPage = PageHelperUtils.changePageHelperToCurrentPage(page, PushMessageDTO.class);
        return ServiceResponse.success(pushMessageDTOPage);
    }

    @Override
    public ServiceResponse<PushMessageDTO> getPushMessageById(Integer messageId) {
        TabPushMessage tabPushMessage = this.pushMessageService.getById(messageId);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(PushMessageDTO.class, tabPushMessage));
    }

    @Override
    public ServiceResponse<UserMessageDTO> getUserMessageDetail(Integer messageId){
        TabPushUserMessage message = this.pushUserMessageService.getById(messageId);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(UserMessageDTO.class, message));
    }

    @Override
    public ServiceResponse<Integer> deletePushMessage(Integer messageId) {
        int i = this.pushMessageService.delete(messageId);
        return ServiceResponse.success(i);
    }

    private void pushMessage(String enterpriseIds, Integer messageId){
        List<Integer> enterpriseIdList = new ArrayList<>();
        if("all".equals(enterpriseIds)){
            List<EnterpriseDTO> list = this.enterpriseService.listEnterprise();
            for(EnterpriseDTO dto : list){
                enterpriseIdList.add(dto.getEnterpriseId());
            }
        }else {
            String[] arr = enterpriseIds.split(",");
            for(String s : arr){
                enterpriseIdList.add(Integer.valueOf(s));
            }
        }
        TabPushMessage tabPushMessage = new TabPushMessage();
        tabPushMessage.setMessageId(messageId);
        tabPushMessage.setPushStatus(1);
        this.pushMessageService.savePushMesasge(tabPushMessage);
        for(Integer enterpriseId : enterpriseIdList){
            GicMQClient client = GICMQClientUtil.getClientInstance();
            JSONObject json = new JSONObject();
            json.put("enterpriseId", enterpriseId);
            json.put("messageId", messageId);
            log.info("通知消息发送到队列的内容：{}", JSON.toJSONString(json));
            try {
                client.sendMessage("pushMessageMq", JSON.toJSONString(json));
            } catch (Exception e) {
                log.warn(e.getStackTrace());
            }
        }
    }
}
