package com.gic.enterprise.service.outer.impl;

import java.util.Date;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.DictDTO;
import com.gic.enterprise.dto.RecordingStorageRecordDTO;
import com.gic.enterprise.service.BillingAccountService;
import com.gic.enterprise.service.BillingRecordingStorageRecordService;
import com.gic.enterprise.service.DictApiService;
import com.gic.enterprise.service.RecordingStorageRecordApiService;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.mq.sdk.GicMQClient;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-09 10:10
 */
@Service("recordingStorageRecordApiService")
public class RecordingStorageRecordApiServiceImpl implements RecordingStorageRecordApiService {

    private Logger logger = LogManager.getLogger(RecordingStorageRecordApiServiceImpl.class);

    @Autowired
    private BillingAccountService billingAccountService;
    @Autowired
    private BillingRecordingStorageRecordService billingRecordingStorageRecordService;
    @Autowired
    private DictApiService dictApiService;

    @Override
    public ServiceResponse<Void> saveRecordingStorageRecord(RecordingStorageRecordDTO recordingStorageRecordDTO) {
//        ServiceResponse paramResult = ValidUtil.allCheckValidate(smsRecordDTO, SmsRecordDTO.SmsRecordValid.class);
//        if (!paramResult.isSuccess()) {
//            return paramResult;
//        }
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            String json = JSON.toJSONString(recordingStorageRecordDTO);
            logger.info("收到语音存储计费信息：{}", json);
            client.sendMessage("RecordingStorageMq", json);
//            this.run(json);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> run(String arg) {
        // 缓存中计费
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            logger.info("语音存储缓存扣费：{}", arg);
            RecordingStorageRecordDTO recordingStorageRecordDTO = JSON.parseObject(arg, RecordingStorageRecordDTO.class);
            // 判断是扣套餐包还是扣余额
            //            Integer fee = accountStandardService.queryUnitFee(internationalSmsRecordDTO.getEnterpriseId(), AccountStandardEnum.SMS.getCode());
            billingAccountService.deductAccountByCache(recordingStorageRecordDTO.getFee(), recordingStorageRecordDTO.getEnterpriseId());

            // 队列
            client.sendMessage("RecordingStorageDbMq", arg);
//            this.executeDatabase(arg);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> executeDatabase(String arg) {
        logger.info("语音存储数据库扣费：{}", arg);
        RecordingStorageRecordDTO recordingStorageRecordDTO = JSON.parseObject(arg, RecordingStorageRecordDTO.class);
        recordingStorageRecordDTO.setDeductionTime(new Date());
        ServiceResponse<DictDTO> serviceDict = dictApiService.getDict(Constants.VOICE_APP_TYPE, recordingStorageRecordDTO.getAppCode());
        if (serviceDict.isSuccess()) {
            DictDTO dictDTO = serviceDict.getResult();
            if (dictDTO != null) {
                String key = dictDTO.getKey();
                recordingStorageRecordDTO.setAppName(key);
            }
        }
        billingAccountService.deductAccountByDatabase(recordingStorageRecordDTO.getFee(), recordingStorageRecordDTO.getEnterpriseId());
        recordingStorageRecordDTO.setPaySerialNumber(CreateRandomUtils.createSerialNumber());
        billingRecordingStorageRecordService.save(recordingStorageRecordDTO);
        return ServiceResponse.success();
    }
}
