package com.gic.enterprise.service.outer.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.util.StringUtil;
import com.gic.enterprise.constant.MallModeEnum;
import com.gic.enterprise.constant.WmStoreConfigTypeEnum;
import com.gic.enterprise.dto.WmMemberCardConfigDTO;
import com.gic.enterprise.dto.WmOrderConfigDTO;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.dto.wm.WmCouponDTO;
import com.gic.enterprise.dto.wm.WmStoreConfigDTO;
import com.gic.enterprise.dto.wm.WmStoreCouponConfigDTO;
import com.gic.enterprise.entity.TabWmStore;
import com.gic.enterprise.entity.TabWmStoreConfig;
import com.gic.enterprise.entity.TabWmStoreCouponConfig;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.wm.CouponConfigQO;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.enterprise.service.WmStoreConfigService;
import com.gic.enterprise.service.WmStoreCouponConfigService;
import com.gic.enterprise.service.WmStoreService;

@Service("wmStoreApiService")
public class WmStoreApiServiceImpl implements WmStoreApiService {
    @Autowired
    private WmStoreService wmStoreService;
    @Autowired
    private WmStoreConfigService wmStoreConfigService;
    @Autowired
    private WmStoreCouponConfigService wmStoreCouponConfigService;

    @Override
    public ServiceResponse<Integer> saveWmStore(WmStoreDTO wmStoreDTO) {
        ServiceResponse validParam = validParam(wmStoreDTO);
        if (!validParam.isSuccess()) {
            return ServiceResponse.failure(validParam.getCode(), validParam.getMessage());
        }
        return ServiceResponse.success(wmStoreService.saveWmStore(wmStoreDTO));
    }

    @Override
    public ServiceResponse<Void> modifyWmStore(WmStoreDTO wmStoreDTO) {
        ServiceResponse validParam = validParam(wmStoreDTO);
        if (!validParam.isSuccess()) {
            return ServiceResponse.failure(validParam.getCode(), validParam.getMessage());
        }
        wmStoreService.modifyWmStore(wmStoreDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> authWmStore(Integer wmMallStoreId) {
        TabWmStore record = wmStoreService.getByWmMallStoreId(wmMallStoreId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟商城店铺ID错误，查无数据");
        }
        wmStoreService.modifyWmStore(new WmStoreDTO().setWmMallStoreId(wmMallStoreId).setAuthStatus(1));
        return ServiceResponse.success(record.getWmPidName());
    }

    @Override
    public ServiceResponse<List<WmStoreDTO>> listWmStore(Integer enterpriseId) {
        List<TabWmStore> list = wmStoreService.listWmStore(enterpriseId);
        return ServiceResponse.success(EntityUtil.changeEntityListNew(WmStoreDTO.class, list));
    }

    @Override
    public ServiceResponse<WmStoreDTO> getWmStoreByWmMallStoreId(Integer wmMallStoreId) {
        TabWmStore record = wmStoreService.getByWmMallStoreId(wmMallStoreId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟商城店铺ID错误，查无数据");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(WmStoreDTO.class, record));
    }


    @Override
    public ServiceResponse<Integer> saveOrUpdateWmStoreConfig(WmStoreConfigDTO dto) {
        if (dto.getEnterpriseId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (!WmStoreConfigTypeEnum.isRightCode(dto.getStoreConfigType())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店铺配置类型值非法");
        }
        TabWmStore wmStore = wmStoreService.getByWmMallStoreId(dto.getWmMallStoreId());
        if (wmStore == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟主账号错误，查无数据");
        }
        if (StringUtils.isBlank(dto.getStoreConfigJson())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店铺配置信息不能为空");
        }
        TabWmStoreConfig record = wmStoreConfigService.getWmStoreConfig(dto.getWmMallStoreId(),
                dto.getStoreConfigType());
        if (record == null) {
            //新增
            return ServiceResponse.success(wmStoreConfigService.saveWmStoreConfig(dto));
        }
        dto.setStoreConfigId(record.getStoreConfigId());
        wmStoreConfigService.editWmStoreConfig(dto);
        return ServiceResponse.success(record.getStoreConfigId());
    }

    @Override
    public ServiceResponse<WmMemberCardConfigDTO> getMemberCardConfig(Integer wmMallStoreId) {
        TabWmStore record = wmStoreService.getByWmMallStoreId(wmMallStoreId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟商城店铺ID错误，查无数据");
        }
        TabWmStoreConfig memberCardConfig = wmStoreConfigService.getWmStoreConfig(wmMallStoreId,
                WmStoreConfigTypeEnum.MEMBER_CARD_CONFIG.getCode());
        if (memberCardConfig == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "还未进行" + WmStoreConfigTypeEnum.MEMBER_CARD_CONFIG.getMessage());
        }
        return ServiceResponse.success(JSON.parseObject(memberCardConfig.getStoreConfigJson(), WmMemberCardConfigDTO.class));
    }

    @Override
    public ServiceResponse<WmOrderConfigDTO> getOrderConfig(Integer wmMallStoreId) {
        TabWmStore record = wmStoreService.getByWmMallStoreId(wmMallStoreId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟商城店铺ID错误，查无数据");
        }
        TabWmStoreConfig orderConfig = wmStoreConfigService.getWmStoreConfig(wmMallStoreId,
                WmStoreConfigTypeEnum.ORDER_CONFIG.getCode());
        if (orderConfig == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "还未进行" + WmStoreConfigTypeEnum.ORDER_CONFIG.getMessage());
        }
        return ServiceResponse.success(JSON.parseObject(orderConfig.getStoreConfigJson(), WmOrderConfigDTO.class));
    }

    @Override
    public ServiceResponse<WmStoreConfigDTO> getWmStoreConfig(Integer wmMallStoreId,
            Integer wmStoreConfigType) {
        TabWmStore record = wmStoreService.getByWmMallStoreId(wmMallStoreId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店铺不存在");
        }
        TabWmStoreConfig config = wmStoreConfigService.getWmStoreConfig(wmMallStoreId, wmStoreConfigType);
        if (config == null) {
            if (!WmStoreConfigTypeEnum.isRightCode(wmStoreConfigType)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店铺配置类型值非法");
            }
            //新增
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    WmStoreConfigTypeEnum.getMessageByCode(wmStoreConfigType) + "还未配置");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(WmStoreConfigDTO.class, config));
    }

    @Override
    public ServiceResponse<Set<String>> getWmStoreConfigKey(Integer wmMallStoreId) {
        Set<String> result;
        List<TabWmStoreConfig> wmStoreConfigList = wmStoreConfigService.listWmStoreConfig(wmMallStoreId);
        //基本配置
        result = Optional.ofNullable(wmStoreConfigList).orElse(new ArrayList<>())
                .stream()
                .map(e -> e.getStoreConfigType().toString())
                .collect(Collectors.toSet());
        //积分配置查询会员组接口，默认已配置
        result.add(String.valueOf(WmStoreConfigTypeEnum.INTEGRAL_CONFIG.getCode()));
        //todo 查询是否有门店导购同步记录
        result.add(String.valueOf(WmStoreConfigTypeEnum.STORE_CLERK_CONFIG.getCode()));

        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<Void> saveCouponConfig(WmStoreCouponConfigDTO dto) {
        if (dto.getEnterpriseId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (dto.getWmMallStoreId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟主账号不能为空");
        }
        TabWmStore wmStore = wmStoreService.getByWmMallStoreId(dto.getWmMallStoreId());
        if (wmStore == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店铺不存在");
        }
        if (StringUtils.isBlank(dto.getGicCouponId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "gic卡券为空");
        }
        if (StringUtils.isBlank(dto.getWmCouponId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟卡券为空");
        }
        List<TabWmStoreCouponConfig> gicRecord = wmStoreCouponConfigService
                .getCouponConfig(new CouponConfigQO()
                        .setWmMallStoreId(dto.getWmMallStoreId())
                        .setGicCouponId(dto.getGicCouponId()));
        if (CollectionUtils.isNotEmpty(gicRecord)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "gic卡券重复绑定");
        }
        List<TabWmStoreCouponConfig> wmRecord = wmStoreCouponConfigService
                .getCouponConfig(new CouponConfigQO()
                        .setWmMallStoreId(dto.getWmMallStoreId())
                        .setWmCouponId(dto.getWmCouponId()));
        if (CollectionUtils.isNotEmpty(wmRecord)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟卡券重复绑定");
        }
        wmStoreCouponConfigService.saveCouponConfig(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<WmCouponDTO>> getWmCouponId(Long couponId) {
        List<TabWmStoreCouponConfig> gicRecord = wmStoreCouponConfigService
                .getCouponConfig(new CouponConfigQO()
                        .setGicCouponId(couponId.toString()));
        if (CollectionUtils.isEmpty(gicRecord)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "未绑定");
        }
        List<WmCouponDTO> list = gicRecord.stream().map(e -> new WmCouponDTO()
                    .setWmCouponId(Long.valueOf(e.getWmCouponId()))
                    .setWmMallStoreId(e.getWmMallStoreId())).collect(Collectors.toList());
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<Page<WmStoreCouponConfigDTO>> pageCouponConfig(CouponConfigQO params) {
        com.github.pagehelper.Page<TabWmStoreCouponConfig> page = wmStoreCouponConfigService.pageCouponConfig(params);
        Page<WmStoreCouponConfigDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, WmStoreCouponConfigDTO.class);
        return ServiceResponse.success(resultPage);
    }


    private ServiceResponse validWmMemberCardConfig(WmMemberCardConfigDTO dto) {
        if (dto.getEnterpriseId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (StringUtils.isBlank(dto.getWmMainAccount())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟主账号不能为空");
        }
        TabWmStore wmStore = wmStoreService.getByWmMainAccount(dto.getWmMainAccount());
        if (wmStore == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟主账号错误，查无数据");
        }
        if (dto.getOpenCardMode() == null) {
            //默认达摩开卡
            dto.setOpenCardMode(1);
        }
        //GIC会员归属信息同步微盟 默认开启
        dto.setGicMemberToWm(Optional.ofNullable(dto.getGicMemberToWm()).orElse(1));
        //微盟会员导购关系同步GIC 默认开启
        dto.setWmMemberClerkToGic(Optional.ofNullable(dto.getWmMemberClerkToGic()).orElse(1));
        //微盟会员信息（包括门店关系）同步GIC 默认开启
        dto.setWmMemberToGic(Optional.ofNullable(dto.getWmMemberToGic()).orElse(1));
        return ServiceResponse.success();
    }

    private ServiceResponse<Void> validParam(WmStoreDTO wmStoreDTO) {
        Integer wmMallStoreId = wmStoreDTO.getWmMallStoreId();
        boolean isEdit = wmMallStoreId != null;
        TabWmStore record = null;
        if (wmMallStoreId != null) {
            record = wmStoreService.getByWmMallStoreId(wmMallStoreId);
            if (record == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟商城店铺ID错误，查无数据");
            }
            wmStoreDTO.setEnterpriseId(record.getEnterpriseId());
        }
        //已授权
        boolean isCanEdit = isEdit && record.getAuthStatus().intValue() == 1;

        if (StringUtils.isBlank(wmStoreDTO.getWmMainAccount())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟主账号不能为空");
        }
        if (wmStoreService.validRepeat(wmStoreDTO.getWmMainAccount(), wmMallStoreId, 1)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟主账号已存在");
        }
        if (isCanEdit && !record.getWmMainAccount().equals(wmStoreDTO.getWmMainAccount())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已授权，微盟主账号不能修改");
        }
        if (StringUtils.isBlank(wmStoreDTO.getWmPid())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "PID不能为空");
        }
        if (isCanEdit && !record.getWmPid().equals(wmStoreDTO.getWmPid())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已授权，PID不能修改");
        }
        if (wmStoreService.validRepeat(wmStoreDTO.getWmPid(), wmMallStoreId, 3)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "PID已存在");
        }
        if (StringUtils.isBlank(wmStoreDTO.getWmStoreId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "storeID不能为空");
        }
        if (isCanEdit && !record.getWmStoreId().equals(wmStoreDTO.getWmStoreId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已授权，storeID不能修改");
        }
        if (wmStoreService.validRepeat(wmStoreDTO.getWmStoreId(), wmMallStoreId, 2)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "storeID已存在");
        }
        if (StringUtils.isBlank(wmStoreDTO.getWmPidName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店铺名称不能为空");
        }
        if (isCanEdit && !record.getWmPidName().equals(wmStoreDTO.getWmPidName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已授权，店铺名称不能修改");
        }
        if (wmStoreService.validRepeatStoreName(wmStoreDTO.getWmPidName(), wmMallStoreId,
                wmStoreDTO.getEnterpriseId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店铺名称已存在");
        }
        if (!MallModeEnum.isContain(wmStoreDTO.getMallMode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商城模式值非法");
        }
        //选择智慧零售模式保存后或者有进行过门店同步则不允许切换回微商城模式
        if (isEdit && (record.getMallMode().intValue() == MallModeEnum.SMART_RETAIL.getCode())
                && wmStoreDTO.getMallMode().intValue() != MallModeEnum.SMART_RETAIL.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已进行过智慧零售模式保存，不允许切换回微商城模式");
        }
        Integer storeBrandId = wmStoreDTO.getStoreBrandId();
        if (storeBrandId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店招品牌不能为空");
        }
        if (isCanEdit && record.getStoreBrandId().intValue() != storeBrandId.intValue()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已授权，店招品牌不能修改");
        }
        String wmAppid = wmStoreDTO.getWmAppid();
        if (StringUtil.isBlank(wmAppid)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟小程序APPID不能为空");
        }
        if (isCanEdit && !record.getWmAppid().equals(wmAppid)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已授权，微盟小程序APPID不能修改");
        }
        String memberApplet = wmStoreDTO.getMemberWechat();
        if (StringUtil.isBlank(memberApplet)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "会员小程序不能为空");
        }
        if (isCanEdit && !record.getMemberWechat().equals(memberApplet)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已授权，会员小程序不能修改");
        }
        return ServiceResponse.success();
    }
}
