package com.gic.enterprise.service.outer.impl.pay;

import java.math.BigDecimal;
import java.util.Date;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.constant.BillTypeEnum;
import com.gic.enterprise.constant.BillingPayResultEnum;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.dto.*;
import com.gic.enterprise.entity.TabBillingPayInfo;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.BillingAccountApiService;
import com.gic.enterprise.service.BillingAccountBalanceRecordService;
import com.gic.enterprise.service.BillingPayInfoService;
import com.gic.enterprise.utils.CreateRandomUtils;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/20 15:37
 */
@Component("4")
public class AccountBalanceStrategy implements PayStrategy {

    @Autowired
    private BillingAccountApiService billingAccountApiService;
    @Autowired
    private BillingPayInfoService billingPayInfoService;
    @Autowired
    private BillingAccountBalanceRecordService billingAccountBalanceRecordService;
//    @Autowired
//    private AuthCodeApiService authCodeApiService;

    private Logger logger = LogManager.getLogger(AccountBalanceStrategy.class);

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<BillingPayInfoDTO> savePrePayInfo(OutPayDTO outPayDTO) {
        // 校验是否有余额
        if (validHasNotBalance(outPayDTO)) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "余额不足");
        }
//        if (BuyTypeEnum.BUY_SHORT_MESSAGE_PACKAGE.getCode() != outPayDTO.getBuyType()) {
//            // 如果不是套餐包购买
//            // 校验签名
//            if (!SignUtils.validSign(outPayDTO, outPayDTO.getSign())) {
//                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据被篡改");
//            }
//        }

        // 判断该商户下是否有相同的订单号
        TabBillingPayInfo payInfo = billingPayInfoService.getByOrderSerialNumber(outPayDTO.getEnterpriseId(), outPayDTO.getRequestCode() ,outPayDTO.getOrderSerialNumber());
        if (payInfo != null) {
            return ServiceResponse.success(EntityUtil.changeEntityByJSON(BillingPayInfoDTO.class, payInfo));
        }

        // 生成验证码
//        String authCode = CreateRandomUtils.getStringRandom(4);
        // 保存验证码
//        ServiceResponse<Integer> serviceResponse = saveAuthCode(outPayDTO, authCode);
        // 保存记录
        BillingPayInfoDTO payInfoDTO = savePayInfo(outPayDTO);
        if (payInfoDTO == null) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "余额支付交易保存失败");
        }
//        payInfoDTO.setAuthCode(authCode);

        return ServiceResponse.success(payInfoDTO);
    }

//    private ServiceResponse<Integer> saveAuthCode(OutPayDTO outPayDTO, String authCode) {
//        AuthCodeDTO codeDTO = new AuthCodeDTO();
//        codeDTO.setAuthCode(authCode);
//        codeDTO.setEnterpriseId(outPayDTO.getEnterpriseId());
//        Calendar calendar = Calendar.getInstance();
//        calendar.add(Calendar.DAY_OF_MONTH, 1);
//        codeDTO.setExpirationTime(calendar.getTime());
//        ServiceResponse<Integer> serviceResponse = authCodeApiService.saveAuth(codeDTO);
//        if (!serviceResponse.isSuccess()) {
//            logger.info("验证码保存失败");
//            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
//        }
//        return serviceResponse;
//    }

    /**
     * 是否有余额， 如果没有余额会返回true， 有余额返回false
     */
    private boolean validHasNotBalance(OutPayDTO outPayDTO) {
        ServiceResponse<BillingAccountDTO> accountService = billingAccountApiService.getByEnterpriseId(outPayDTO.getEnterpriseId());
        if (accountService.isSuccess()) {
            BillingAccountDTO accountDTO = accountService.getResult();
            Double balance = accountDTO.getAccountBalance();
            return balance < outPayDTO.getTotalFeePaid();
        }
        return false;
    }

    /**
     * 保存验证信息
     */
    private BillingPayInfoDTO savePayInfo(OutPayDTO outPayDTO) {
        BillingPayInfoDTO payInfoDTO = new BillingPayInfoDTO();
        payInfoDTO.setEnterpriseId(outPayDTO.getEnterpriseId());
        payInfoDTO.setSerialNumber(CreateRandomUtils.getStringRandom(11));
        payInfoDTO.setPayType(PayTypeEnum.BALANCE_PAY.getCode());
        payInfoDTO.setTotalFeePaid(outPayDTO.getTotalFeePaid());
        payInfoDTO.setTimeStart(outPayDTO.getTimeStart());
        payInfoDTO.setResultCode(BillingPayResultEnum.WAIT.getCode());
        payInfoDTO.setInvoiceStatus(GlobalInfo.DATA_STATUS_DELETE);
        payInfoDTO.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        payInfoDTO.setCreateTime(new Date());
        payInfoDTO.setBuyType(outPayDTO.getBuyType());
        payInfoDTO.setOrderSerialNumber(outPayDTO.getOrderSerialNumber());
//        payInfoDTO.setAuthCodeId(authCodeId);
        payInfoDTO.setRequestCode(outPayDTO.getRequestCode());
        payInfoDTO.setProductDescription(outPayDTO.getProductDescription());
        Integer line = billingPayInfoService.savePayInfo(payInfoDTO);
        if (line == 0) {
            logger.info("余额支付交易保存失败, {}", JSON.toJSONString(payInfoDTO));
            // 回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return null;
        }
        return payInfoDTO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<BillingPayInfoDTO> confirmPay(OutPayDTO outPayDTO) {

        // 校验签名
//        if (!SignUtils.validSign(outPayDTO, outPayDTO.getSign())) {
//            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据被篡改");
//        }
        // 判断该商户下订单是否存在
        TabBillingPayInfo payInfo = billingPayInfoService.getByOrderSerialNumber(outPayDTO.getEnterpriseId(), outPayDTO.getRequestCode() ,outPayDTO.getOrderSerialNumber());
        if (payInfo == null) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "未查询到此交易订单，请先调用下单接口");
        }
        // 判断订单是否已经支付
        if (!BillingPayResultEnum.WAIT.getCode().equals(payInfo.getResultCode())) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "交易已支付或已关闭");
        }
        // 判断验证码是否过期 正确
//        AuthCodeDTO authCodeDTO = authCodeApiService.getAuthCode(payInfo.getAuthCodeId()).getResult();
//        if (authCodeDTO == null) {
//            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "验证码已过期或不存在");
//        }
//        if (!StringUtils.equals(authCodeDTO.getAuthCode(), outPayDTO.getAuthCode())) {
//            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "验证码错误");
//        }

        // 判断单据基本信息是否一致
        if (!outPayDTO.getOrderSerialNumber().equals(payInfo.getOrderSerialNumber()) ||
                !outPayDTO.getTotalFeePaid().equals(payInfo.getTotalFeePaid())
                ) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "订单参数错误");
        }
        // 校验是否有余额
        if (validHasNotBalance(outPayDTO)) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "余额不足");
        }

        // 更新交易信息
        ApprovalCallBackDTO dto = new ApprovalCallBackDTO();
        dto.setBuyType(payInfo.getBuyType());
        dto.setEnterpriseId(payInfo.getEnterpriseId());
        dto.setPaySerialNumber(payInfo.getSerialNumber());
        dto.setResultCode(BillingPayResultEnum.SUCCESS.getCode());
        Integer line = billingPayInfoService.updatePayInfoBySerialNumber(dto);
        if (line == 0) {
            logger.info("更新支付信息失败， {}", JSON.toJSONString(dto));
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "更新支付信息失败");
        }
        // 扣费
        ServiceResponse<Integer> serviceResponse = billingAccountApiService.deductAccount(payInfo.getTotalFeePaid(), payInfo.getEnterpriseId());
        if (!serviceResponse.isSuccess()) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "扣费失败");
        }
        // 写明细
        AccountBalanceRecordDTO balanceRecordDTO = new AccountBalanceRecordDTO();
        balanceRecordDTO.setProductName(payInfo.getProductDescription());
        balanceRecordDTO.setPayType(payInfo.getPayType());
        balanceRecordDTO.setProductType(outPayDTO.getBuyProductType());
        balanceRecordDTO.setBillType(BillTypeEnum.CONSUMPTION.getCode());
        balanceRecordDTO.setOrderSerialNumber(outPayDTO.getOrderSerialNumber());
        balanceRecordDTO.setEnterpriseId(outPayDTO.getEnterpriseId());
        balanceRecordDTO.setFee(outPayDTO.getTotalFee());
        balanceRecordDTO.setFeePaid(outPayDTO.getTotalFeePaid());
        if (outPayDTO.getTotalFee() != null && outPayDTO.getTotalFeePaid() != null) {
            balanceRecordDTO.setFeeDiscount(new BigDecimal(outPayDTO.getTotalFee()).subtract(new BigDecimal(outPayDTO.getTotalFeePaid())).doubleValue());
        }
        balanceRecordDTO.setProductName(payInfo.getProductDescription());
        balanceRecordDTO.setPayType(payInfo.getPayType());

        billingAccountBalanceRecordService.save(balanceRecordDTO);
        billingPayInfoService.delOtherPayInfoExceptId(payInfo.getPayInfoId(), payInfo.getEnterpriseId(), payInfo.getOrderSerialNumber());

        return ServiceResponse.success(EntityUtil.changeEntityByOrika(BillingPayInfoDTO.class, payInfo));
    }

    @Override
    public ServiceResponse<Boolean> queryAndUpdatePayStatus(Integer enterpriseId, String serialNumber) {
        logger.info("余额支付没有该方法 queryAndUpdatePayStatus");
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> callBack(ApprovalCallBackDTO resData) {
        logger.info("余额支付没有该方法 callBack");
        return ServiceResponse.success();
    }

}
