package com.gic.enterprise.utils.chart;

import com.gic.enterprise.constant.billing.FeeTypeEnum;
import com.gic.enterprise.dto.HomeStatisticsDTO;
import com.gic.enterprise.qo.HomeStatisticsQO;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
/**
 * 计费中心首页图表
 * @ClassName: FeeChartUtils 
 * @Description:  
 * @author guojuxing 
 * @date 2019/8/23 2:18 PM 
 */
public class FeeChartUtils {
    private static Map getPageMap = new HashMap<>(8);

    static {
        getPageMap.put(FeeTypeEnum.ACCOUNT_BALANCE.getCode(), new BillingAccountBalanceRecord());
//        getPageMap.put(FeeTypeEnum.DOUBLE_CALLING.getCode(), new BillingDoubleCallingRecord());
//        getPageMap.put(FeeTypeEnum.RECORDING_STORAGE.getCode(), new BillingRecordingStorageRecord());
        getPageMap.put(FeeTypeEnum.SMS.getCode(), new BillingSmsRecord());
        getPageMap.put(FeeTypeEnum.SMS_PACKAGE.getCode(), new BillingSmsPackageRecord());
        getPageMap.put(FeeTypeEnum.INTERNATIONAL_SMS.getCode(), new BillingInternationSmsRecord());
//        getPageMap.put(FeeTypeEnum.VOICE_CODE.getCode(), new BillingVoiceCodeRecord());
    }

    /**
     * 查询消费趋势分析图表数据
     * @Title: getFeeChart 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  java.util.List<com.gic.enterprise.dto.HomeStatisticsDTO> 
      */
    public static List<HomeStatisticsDTO> getFeeChart(HomeStatisticsQO params) {
        IFeeChart feeChart = getFeeChartObject(params.getFeeType());
        if (feeChart != null) {
            return feeChart.listHomeStatistics(params);
        }
        return new ArrayList<>();
    }

    /**
     * 获取消费金额
     * @Title: getFee 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param dateType
     * @param type 
     * @return  double 
      */
    public static double getFee(Integer enterpriseId, Integer dateType, int type) {
        IFeeChart feeChart = getFeeChartObject(type);
        if (feeChart != null) {
            return feeChart.getFee(enterpriseId, dateType);
        }
        return 0;
    }

    public static IFeeChart getFeeChartObject(int type) {
        if (getPageMap.containsKey(type)) {
            IFeeChart iFeeChart = (IFeeChart) getPageMap.get(type);
            return iFeeChart;
        }
        return null;
    }
}
