package com.gic.enterprise.web.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.constant.applet.AppletAppStatusEnum;
import com.gic.enterprise.dto.AppletAppDTO;
import com.gic.enterprise.service.AppletAppApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.AppletAppVO;

@RestController
@RequestMapping("/applet-app")
public class AppletAppController {
    private static final Logger LOGGER = LogManager.getLogger(AppletAppController.class);

    @Autowired
    private AppletAppApiService appletAppApiService;

    @RequestMapping("/save")
    public RestResponse save(AppletAppDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(appletAppApiService.save(dto),
                OperationResultUtils.LOG_NEW + "小程序API", dto.getAppName());
    }

    @RequestMapping("/edit")
    public RestResponse edit(AppletAppDTO dto) {
        return OperationResultUtils.operationResult(appletAppApiService.edit(dto),
                OperationResultUtils.LOG_EDIT + "小程序API", dto.getAppName());
    }

    @RequestMapping("/bind")
    public RestResponse bind(Integer appletAppId) {
        ServiceResponse<String> result = appletAppApiService.bind(appletAppId,
                UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(result, "绑定小程序API",
                OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/un-bind")
    public RestResponse unBind(Integer appletAppId) {
        ServiceResponse<String> result = appletAppApiService.unBind(appletAppId);
        return OperationResultUtils.operationResult(result, "解绑小程序API",
                OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/delete")
    public RestResponse delete(Integer appletAppId) {
        ServiceResponse<String> result = appletAppApiService.delete(appletAppId);
        return OperationResultUtils.operationResult(result, OperationResultUtils.LOG_DELETE + "小程序API",
                OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/list")
    public RestResponse save(String search, Integer appStatus, Integer orderCode, Integer currentPage,
            Integer pageSize) {
        if (currentPage == null) {
            currentPage = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        return ResultControllerUtils.commonPageResult(
                appletAppApiService.page(search, enterpriseId, appStatus, orderCode, currentPage, pageSize),
                AppletAppVO.class);
    }

    @RequestMapping("/list-applet-app-status")
    public RestResponse listAppletAppStatus() {
        List<Map<String, Object>> list = new ArrayList<>();
        Map<String, Object> result;
        for (AppletAppStatusEnum type : AppletAppStatusEnum.values()) {
            if (type.getCode() != AppletAppStatusEnum.DELETE.getCode()) {
                result = new HashMap<>(4);
                result.put("key", type.getCode());
                result.put("value", type.getMessage());
                list.add(result);
            }
        }
        return RestResponse.success(list);
    }
}
