package com.gic.enterprise.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.ChannelResourceDTO;
import com.gic.auth.service.ResourceApiService;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.constant.applet.AppletServiceConfigEnum;
import com.gic.enterprise.dto.AppletServiceConfigDTO;
import com.gic.enterprise.qo.AppletServiceConfigQO;
import com.gic.enterprise.service.AppletServiceConfigApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-27 17:46
 */
@RestController
public class AppletServiceConfigController {

    @Autowired
    private AppletServiceConfigApiService appletServiceConfigApiService;
    @Autowired
    private ResourceApiService resourceApiService;

    @RequestMapping("list-applet")
    public RestResponse listApplet(AppletServiceConfigQO appletServiceConfigQO) {
        ChannelResourceDTO channelResourceDTO = resourceApiService.getChannelResourceByUserId(UserDetailUtils.getUserDetail().getUserId()).getResult();
        if (channelResourceDTO == null || CollectionUtils.isEmpty(channelResourceDTO.getAppletConfigIdList())) {
            return RestResponse.success(new Page<>());
        }
        appletServiceConfigQO.setConfigIdList(channelResourceDTO.getAppletConfigIdList());
        ServiceResponse<Page<AppletServiceConfigDTO>> serviceResponse = appletServiceConfigApiService.listApplet(UserDetailUtils.getUserDetail().getEnterpriseId(), appletServiceConfigQO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("list-service")
    public RestResponse listService(AppletServiceConfigQO appletServiceConfigQO) {
        ChannelResourceDTO channelResourceDTO = resourceApiService.getChannelResourceByUserId(UserDetailUtils.getUserDetail().getUserId()).getResult();
        if (channelResourceDTO == null || CollectionUtils.isEmpty(channelResourceDTO.getServiceConfigIdList())) {
            return RestResponse.success(new Page<>());
        }
        appletServiceConfigQO.setConfigIdList(channelResourceDTO.getServiceConfigIdList());
        ServiceResponse<Page<AppletServiceConfigDTO>> serviceResponse = appletServiceConfigApiService.listService(UserDetailUtils.getUserDetail().getEnterpriseId(), appletServiceConfigQO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("sync-applet")
    public RestResponse listApplet() {
        ServiceResponse<Void> serviceResponse = appletServiceConfigApiService.syncApplet(UserDetailUtils.getUserDetail().getEnterpriseId());
        LogUtils.createLog("同步数据", "小程序");
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("sync-service")
    public RestResponse syncServiceConfig() {
        ServiceResponse<Void> serviceResponse = appletServiceConfigApiService.syncServiceConfig(UserDetailUtils.getUserDetail().getEnterpriseId());
        LogUtils.createLog("同步数据", "服务号");
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("del-applet-service")
    public RestResponse delAppletService(Integer appletServiceId) {
        ServiceResponse<AppletServiceConfigDTO> response = appletServiceConfigApiService.getAppletService(appletServiceId);
        AppletServiceConfigDTO appletService = response.getResult();
        if (appletService != null) {
            ServiceResponse<Void> serviceResponse = appletServiceConfigApiService.delAppletService(appletServiceId);
            if (serviceResponse.isSuccess()) {
                LogUtils.createLog("删除" + (AppletServiceConfigEnum.parseCode(appletService.getType())), appletService.getName());
                return ResultControllerUtils.commonResult(serviceResponse);
            }
        }
        return ResultControllerUtils.commonResult(response);
    }

}
