package com.gic.enterprise.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.constant.BillingInitiatorTypeEnum;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.dto.BillingRechargeDTO;
import com.gic.enterprise.dto.BillingRechargeRecordDTO;
import com.gic.enterprise.qo.RechargeRecordQO;
import com.gic.enterprise.service.BillingRechargeApiService;
import com.gic.enterprise.service.BillingRechargeRecordApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.qo.BillingRechargeQO;
import com.gic.finance.dto.TransferAccountsApprovalDTO;
import com.gic.finance.service.TransferAccountsApprovalApiService;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/16 10:52
 */
@RestController
public class BillingRechargeController {

    private Logger logger = LogManager.getLogger(BillingRechargeController.class);

    @Autowired
    private BillingRechargeApiService billingRechargeApiService;
    @Autowired
    private BillingRechargeRecordApiService billingRechargeRecordApiService;
    @Autowired
    private TransferAccountsApprovalApiService transferAccountsApprovalApiService;

    @RequestMapping("/recharge-select-pay")
    public RestResponse saveAndSelectPaymentMethod(@Validated(BillingRechargeQO.SelectPayValid.class) BillingRechargeQO billingRechargeQO) {
//        billingRechargeDTO
        BillingRechargeDTO billingRechargeDTO = EntityUtil.changeEntityByOrika(BillingRechargeDTO.class, billingRechargeQO);
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        billingRechargeDTO.setEnterpriseId(userDetail.getEnterpriseId());
        billingRechargeDTO.setInitiator(UserDetailUtils.getUserDetail().getUserInfo().getLoginType() == 0 ?
                BillingInitiatorTypeEnum.ENTERPRISE.getCode() : BillingInitiatorTypeEnum.OPERATION.getCode());
        billingRechargeDTO.setInitiatorUser(userDetail.getUserId());
        billingRechargeDTO.setInitiatorName(userDetail.getUserInfo().getUserName());
        ServiceResponse<Map<String, Object>> serviceResponse = billingRechargeApiService.saveAndSelectPaymentMethod(billingRechargeDTO);
//        if (serviceResponse.isSuccess()) {
//            LogUtils.createLog("预支付金额：" + billingRechargeQO.getTotalFee(), PayTypeEnum.getMessageByCode(billingRechargeQO.getPayType()));
//        }
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/recharge-confirm-pay")
    public RestResponse confirmPay(@Validated(BillingRechargeQO.ConfirmPayValid.class) BillingRechargeQO billingRechargeQO) {
        //        billingRechargeDTO
        BillingRechargeDTO billingRechargeDTO = EntityUtil.changeEntityByOrika(BillingRechargeDTO.class, billingRechargeQO);
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        billingRechargeDTO.setEnterpriseId(userDetail.getEnterpriseId());
        billingRechargeDTO.setInitiator(UserDetailUtils.getUserDetail().getUserInfo().getLoginType() == 0 ?
                BillingInitiatorTypeEnum.ENTERPRISE.getCode() : BillingInitiatorTypeEnum.OPERATION.getCode());
        billingRechargeDTO.setInitiatorUser(userDetail.getUserId());
        billingRechargeDTO.setInitiatorName(userDetail.getUserInfo().getUserName());
        ServiceResponse<Integer> serviceResponse = billingRechargeApiService.confirmPay(billingRechargeDTO);
        if (serviceResponse.isSuccess()) {
            LogUtils.createLog("充值", "账户余额");
        }
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-recharge-record")
    public RestResponse listRechargeRecord(RechargeRecordQO rechargeRecordQO) {
        rechargeRecordQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (StringUtils.isNotBlank(rechargeRecordQO.getStartTime())) {
            rechargeRecordQO.setStartTime(rechargeRecordQO.getStartTime() + " 00:00:00");
        }
        if (StringUtils.isNotBlank(rechargeRecordQO.getEndTime())) {
            rechargeRecordQO.setEndTime(rechargeRecordQO.getEndTime() + " 23:59:59");
        }
        ServiceResponse<Page<BillingRechargeRecordDTO>> serviceResponse = this.billingRechargeRecordApiService.listRechargeRecord(rechargeRecordQO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/get-recharge-record")
    public RestResponse getRechargeRecord(Integer rechargeRecordId) {
        ServiceResponse<BillingRechargeRecordDTO> serviceResponse = this.billingRechargeRecordApiService.getRechargeRecord(rechargeRecordId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/cancel-recharge-approval")
    public RestResponse cancelRechargeApproval(Integer rechargeRecordId) {
        ServiceResponse<BillingRechargeRecordDTO> record = this.billingRechargeRecordApiService.getRechargeRecord(rechargeRecordId);
        ServiceResponse<Integer> serviceResponse = this.billingRechargeRecordApiService.cancelRechargeApproval(rechargeRecordId);
        if (serviceResponse.isSuccess()) {
            LogUtils.createLog("取消支付，支付金额：" + record.getResult().getTotalFee(), PayTypeEnum.getMessageByCode(record.getResult().getPayType()));
        }
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/bad-account")
    public RestResponse getBadAccount(String serialNumber) {
        ServiceResponse<TransferAccountsApprovalDTO> service = transferAccountsApprovalApiService.getByOrderNumber(serialNumber);
        TransferAccountsApprovalDTO result = service.getResult();
        if (result != null) {
            return RestResponse.success(result.getBadAmount());
        } else {
            logger.info("serialNumber:{}, 查询坏账错误：{}",serialNumber,service.getMessage());
            return RestResponse.success();
        }
    }

    @RequestMapping("/has-recharge")
    public RestResponse hasRecharge() {
        ServiceResponse<Boolean> response = this.billingRechargeApiService.hasRecharge(UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseId());
        return ResultControllerUtils.commonResult(response);
    }


}
