package com.gic.enterprise.web.controller;

import java.util.List;
import java.util.stream.Collectors;

import com.gic.enterprise.dto.data.*;
import com.gic.enterprise.web.vo.data.*;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.gic.commons.util.StringUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.DataConfigApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;

@RestController
@RequestMapping("/data-config")
public class DataConfigController {
    private final static Logger LOGGER = LogManager.getLogger(DataConfigController.class);

    @Autowired
    private DataConfigApiService dataConfigApiService;

    @RequestMapping("/config-member-consume")
    public RestResponse configMemberConsume(DataMemberConsumeConfigDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(dataConfigApiService.configMemberConsume(dto),
                OperationResultUtils.LOG_EDIT + "数据统计配置-会员消费配置",
                UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
    }

    @RequestMapping("/get-member-consume-config")
    public RestResponse getMemberConsumeConfig() {
        return ResultControllerUtils.commonResultOne(dataConfigApiService
                .getMemberConsumeConfigByEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId()), MemberConsumeConfigVO.class);
    }

    @RequestMapping("/config-actually-paid")
    public RestResponse configActuallyPaid(String jsonArr) {
        if (StringUtil.isBlank(jsonArr)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "实付配置内容为空");
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        List<DataActuallyPaidConfigDTO> dtoList = JSON.parseArray(jsonArr, DataActuallyPaidConfigDTO.class);
        dtoList = dtoList.stream().map(e -> e.setEnterpriseId(enterpriseId)).collect(Collectors.toList());
        return OperationResultUtils.operationResult(dataConfigApiService.configActuallyPaid(dtoList),
                OperationResultUtils.LOG_EDIT + "数据统计配置-实付配置",
                UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
    }

    @RequestMapping("/get-actually-paid-config")
    public RestResponse getActuallyPaidConfig() {
        return ResultControllerUtils.commonResult(
                dataConfigApiService.listActuallyPaidConfig(UserDetailUtils.getUserDetail().getEnterpriseId()),
                ActuallyPaidConfigVO.class);
    }

    @RequestMapping("/config-single-effect")
    public RestResponse configSingleEffect(String jsonArr) {
        if (StringUtil.isBlank(jsonArr)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "单效配置内容为空");
        }
        List<DataSingleEffectConfigDTO> dtoList = JSON.parseArray(jsonArr, DataSingleEffectConfigDTO.class);
        dtoList = dtoList.stream().map(e -> e.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId())).collect(Collectors.toList());
        return OperationResultUtils.operationResult(dataConfigApiService.configSingleEffect(dtoList),
                OperationResultUtils.LOG_EDIT + "数据统计配置-单效取值配置",
                UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
    }

    @RequestMapping("/get-single-effect-config")
    public RestResponse getSingleEffectConfig() {
        return ResultControllerUtils.commonResult(
                dataConfigApiService.getSingleEffectConfig(UserDetailUtils.getUserDetail().getEnterpriseId()),
                DataSingleEffectConfigVO.class);
    }

    @RequestMapping("/config-performance")
    public RestResponse configPerformance(DataPerformanceConfigDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(dataConfigApiService.configPerformance(dto),
                OperationResultUtils.LOG_EDIT + "数据统计配置-业绩指标配置",
                UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
    }

    @RequestMapping("/get-performance-config")
    public RestResponse getPerformanceConfig() {
        return ResultControllerUtils.commonResultOne(
                dataConfigApiService.getPerformanceConfig(UserDetailUtils.getUserDetail().getEnterpriseId()),
                PerformanceVO.class);
    }


    @RequestMapping("/config-generation")
    public RestResponse configGeneration(@RequestBody List<DataGenerationConfigDTO> dtoList) {
        return OperationResultUtils.operationResult(dataConfigApiService.saveGenerationConfig(dtoList.stream()
                        .map(e -> e.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId())).collect(Collectors.toList())),
                OperationResultUtils.LOG_EDIT + "数据统计配置-年龄段配置",
                UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
    }

    @RequestMapping("/get-generation-config")
    public RestResponse getGenerationConfig() {
        return ResultControllerUtils.commonResult(
                dataConfigApiService.listGenerationConfig(UserDetailUtils.getUserDetail().getEnterpriseId()),
                GenerationConfigVO.class);
    }
}
