package com.gic.enterprise.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.dto.DownloadReportDTO;
import com.gic.enterprise.qo.DownloadReportQO;
import com.gic.enterprise.service.DownloadReportApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-10-29 20:28
 */
@RestController
public class DownloadReportController {

    @Autowired
    private DownloadReportApiService downloadReportApiService;

    @RequestMapping("/list-download-report")
    public RestResponse listDownloadReport(@Validated({DownloadReportQO.SelectValidView.class}) DownloadReportQO downloadReportQO) {
        downloadReportQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (StringUtils.isNotBlank(downloadReportQO.getStartTime())) {
            downloadReportQO.setStartTime(downloadReportQO.getStartTime() + " 00:00:00");
        }
        if (StringUtils.isNotBlank(downloadReportQO.getEndTime())) {
            downloadReportQO.setEndTime(downloadReportQO.getEndTime() + " 23:59:59");
        }
        ServiceResponse<Page<DownloadReportDTO>> serviceResponse = downloadReportApiService.listDownloadReport(downloadReportQO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/get-download-report-url")
    public RestResponse saveDownloadLogReturnDownloadUrl(Integer downloadReportId) {
        ServiceResponse<String> serviceResponse = this.downloadReportApiService.saveDownloadLogReturnDownloadUrl(downloadReportId, UserDetailUtils.getUserDetail().getUserId());
        ServiceResponse<DownloadReportDTO> downloadReport = this.downloadReportApiService.getDownloadReport(downloadReportId);
        if (serviceResponse.isSuccess()) {
            LogUtils.createLog("下载报表", downloadReport.getResult().getFileName());
        }
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/apply-download-report")
    public RestResponse applyDownloadReport(Integer downloadReportId) {
        ServiceResponse<Void> serviceResponse = this.downloadReportApiService.applyDownloadReport(downloadReportId);
        ServiceResponse<DownloadReportDTO> downloadReport = this.downloadReportApiService.getDownloadReport(downloadReportId);
        if (serviceResponse.isSuccess()) {
            LogUtils.createLog("下载报表", downloadReport.getResult().getFileName());
        }
        return ResultControllerUtils.commonResult(serviceResponse);
    }
}
