package com.gic.enterprise.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.LinkDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.LinkApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.ActivityThemeVO;
import com.gic.mall.api.dto.activity.MallActivityPageSerchDTO;
import com.gic.mall.api.dto.activity.MallActivityThemeDTO;
import com.gic.mall.api.service.MallActivityThemeApiService;
import com.gic.mall.share.api.dto.shop.ShopDTO;
import com.gic.mall.share.api.service.ShopApiService;
import com.gic.member.config.api.dto.AppletCardRelationDTO;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-26 16:29
 */
@RestController
public class LinkController {

    @Autowired
    private LinkApiService linkApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;
    @Autowired
    private ServiceConfigApiService serviceConfigApiService;
    @Autowired
    private ShopApiService shopApiService;
    @Autowired
    private MallActivityThemeApiService mallActivityThemeApiService;

    @RequestMapping("/list-link-data")
    public RestResponse listLinkData(Integer parentId) {
        ServiceResponse<List<LinkDTO>> serviceResponse = linkApiService.listByGic(UserDetailUtils.getUserDetail().getEnterpriseId(), parentId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/tree-link-data")
    public RestResponse treeLinkData(Integer objectType) {
        ServiceResponse<List<LinkDTO>> serviceResponse = linkApiService.treeByGic(UserDetailUtils.getUserDetail().getEnterpriseId(), objectType);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-applet-card")
    public RestResponse listAppletConfigByCardConfigIds(Long cardConfigId) {
        ServiceResponse<List<AppletsConfigSimpleOutputDTO>> serviceResponse = this.appletsConfigApiService.listAppletConfigByCardConfigIds(Collections.singletonList(cardConfigId), UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-applet-service")
    public RestResponse queryCardConfigBindServiceConfig(Long serviceConfigId) {
        ServiceResponse<List<AppletCardRelationDTO>> serviceResponse = this.serviceConfigApiService.queryCardConfigBindServiceConfig(serviceConfigId, UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-mall-activity")
    public RestResponse listMallActivity(String appletId) {
        ServiceResponse<AppletsConfigSimpleOutputDTO> appletResp = appletsConfigApiService.selectAppletsConfigIdByAppId(appletId, UserDetailUtils.getUserDetail().getEnterpriseId());
        if (!appletResp.isSuccess() || appletResp.getResult() == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有对应的小程序");
        }

        ServiceResponse<ShopDTO> shopResp = shopApiService.getShopByAppletId(appletResp.getResult().getAppletConfigId(), UserDetailUtils.getUserDetail().getEnterpriseId(), 1);
        if (!shopResp.isSuccess() || shopResp.getResult() == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有对应的店铺");
        }
        MallActivityPageSerchDTO serchDTO = new MallActivityPageSerchDTO();
        serchDTO.setShopId(shopResp.getResult().getId());
        serchDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        serchDTO.setStatus(4);
        serchDTO.setPageNum(1);
        serchDTO.setPageSize(200);
        ServiceResponse<Page<MallActivityThemeDTO>> serviceResponse = mallActivityThemeApiService.pageActivityTheme(serchDTO);
        if (serviceResponse.isSuccess()) {
            List<MallActivityThemeDTO> themeDTOList = serviceResponse.getResult().getResult();
            List<ActivityThemeVO> resultList = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(themeDTOList)) {
                for (MallActivityThemeDTO mallActivityThemeDTO : themeDTOList) {
                    ActivityThemeVO themeVO = new ActivityThemeVO();
                    themeVO.setId(mallActivityThemeDTO.getId());
                    themeVO.setActivityName(mallActivityThemeDTO.getThemeName());
                    resultList.add(themeVO);
                }
            }
            return RestResponse.success(resultList);
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

}
