package com.gic.enterprise.web.controller.export;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.DataDownloadUtils;
import com.gic.download.utils.DownloadReport;
import com.gic.download.utils.DownloadUtils;
import com.gic.download.utils.ExecutorPoolSingleton;
import com.gic.enterprise.context.RequestContext;
import com.gic.enterprise.dto.RecordingStorageRecordDTO;
import com.gic.enterprise.qo.RecordQO;
import com.gic.enterprise.service.BillingRecordApiService;
import com.gic.enterprise.service.DictApiService;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.qo.ExportQO;
import com.gic.enterprise.web.utils.DataContentUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-11 14:25
 */
@RestController
public class RecordingStorageRecordController extends DownloadUtils {
    private Logger logger = LogManager.getLogger(RecordingStorageRecordController.class);

    @Autowired
    private BillingRecordApiService billingRecordApiService;
    @Autowired
    private DictApiService dictApiService;

    @RequestMapping("/list-recording-storage-field")
    public RestResponse listVoiceCodeField() {
        RecordingStorageFieldEnum[] values = RecordingStorageFieldEnum.values();
        List<Map<String, Object>> arrayList = new ArrayList<>(values.length);
        for (RecordingStorageFieldEnum value : values) {
            Map<String, Object> hashMap = new HashMap<>(4);
            hashMap.put("fieldCode", value.getFieldCode());
            hashMap.put("fieldName", value.getFieldName());
            hashMap.put("needEncrypt", value.getNeedEncrypt());
            arrayList.add(hashMap);
        }
        return RestResponse.success(arrayList);
    }

    @RequestMapping("/export-recording-storage")
    public RestResponse exportRecordingStorageRecord(HttpServletRequest request, RecordQO recordQO, ExportQO exportQO) {
        recordQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Integer> serviceResponse = billingRecordApiService.countRecordingStorageRecord(recordQO);


        List<String> fieldCodeList = exportQO.getFieldCodeList();
        List<String> fieldNameList = new ArrayList<>(fieldCodeList.size());
        for (String fieldCode: fieldCodeList) {
            String fieldName = RecordingStorageFieldEnum.valueOf(fieldCode).getFieldName();
            fieldNameList.add(fieldName);
        }

        // 开始下载
        String dataContent = DataContentUtils.getDataContent();

        // 生成入参数据
        DownloadReport report = new DownloadReport();
        //申请人ID
        report.setApplyUserId(UserDetailUtils.getUserDetail().getUserId());
        //数据源（频道页），用  "-" 隔开上级和本级
        report.setDataContent(dataContent);
        //待下载数据的数据总量
        report.setDataCount(serviceResponse.getResult());
        //报告类型  1：完整数据， 2：脱敏数据， 3：二维码
        report.setDataType(exportQO.getDataType());
        //数据源超链接，用于点击跳转对应频道页面
        report.setDataUrl((String) RequestContext.getContext().getRequest().getAttribute("moduleMenuUrl"));
        report.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        //待下载文件名
        report.setFileName(exportQO.getFileName());
        //待下载数据的结束时间
        if (StringUtils.isNotBlank(recordQO.getEndTime())) {
            report.setReportEndTime(DateUtil.strToDate(recordQO.getEndTime(), DateUtil.FORMAT_DATETIME_19));
        }
        //待下载数据的开始时间
        if (StringUtils.isNotBlank(recordQO.getStartTime())) {
            report.setReportStartTime(DateUtil.strToDate(recordQO.getStartTime(), DateUtil.FORMAT_DATETIME_19));
        }
        //调用生成方法，获取reportId
        Integer reportId = DataDownloadUtils.createDownloadReport(report);

        String path = request.getSession().getServletContext().getRealPath("/excel/csv/collaborator/file/");
        String userName = UserDetailUtils.getUserDetail().getUserInfo().getUserName();
        ExecutorPoolSingleton.getInstance().executeTask(() -> {
            DownloadDataLoader<RecordingStorageRecordDTO> loader = new DownloadDataLoader<RecordingStorageRecordDTO>() {
                @Override
                protected List<RecordingStorageRecordDTO> getDownloadData(Integer pageNum) throws Exception {
                    //调用分页查询接口
                    recordQO.setCurrentPage(pageNum);
                    recordQO.setPageSize(1000);
                    ServiceResponse<Page<RecordingStorageRecordDTO>> result = billingRecordApiService.listRecordingStorageRecord(recordQO);
                    return result.getResult().getResult();
                }
            };
            try {
                //参数说明：excelExtensionCode，Excel后缀名code ，枚举类：ExcelExtensionEnum, Arrays.asList("phone")这个phone是需要加密的字段
                download(path, reportId, exportQO.getFileName(), exportQO.getExcelExtension(), fieldNameList, fieldCodeList, loader, Arrays.asList("callerPhone", "calledPhone"), null);
            } catch (Exception e) {
                logger.warn("下载错误", e);
            }
        });
        return RestResponse.success(reportId);
    }

    private enum RecordingStorageFieldEnum{
        deductionTime("deductionTime", "扣费时间", 0),
        paySerialNumber("paySerialNumber", "支付流水号", 0),
        storageDuration("storageDuration", "录音存储时长", 0),
        callerName("callerName", "主叫姓名", 0),
        callerPhone("callerPhone", "主叫号码", 1),
        calledPhone("calledPhone", "被叫号码", 1),
        storeName("storeName", "服务门店", 0),
        storeGroupName("storeGroupName", "门店分组", 0),
        callTime("callTime", "呼叫时间", 0),
        voiceTime("voiceTime", "通话时间", 0),
        appName("appName", "应用", 0),
        remark("remark", "说明", 0),
        fee("fee", "费用", 0),
        ;
        private String fieldCode;
        private String fieldName;
        private Integer needEncrypt;

        RecordingStorageFieldEnum(String fieldCode, String fieldName, Integer needEncrypt) {
            this.fieldCode = fieldCode;
            this.fieldName = fieldName;
            this.needEncrypt = needEncrypt;
        }

        public String getFieldCode() {
            return fieldCode;
        }

        public String getFieldName() {
            return fieldName;
        }

        public Integer getNeedEncrypt() {
            return needEncrypt;
        }
    }
}
