package com.gic.enterprise.web.utils;

import com.gic.commons.util.PropertiesVal;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.WriterException;
import com.google.zxing.common.BitMatrix;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

public class QrcodeUtils {
    private static final Logger log = LogManager.getLogger(QrcodeUtils.class);
    private static final int width = 1280;
    private static final int height = 1280;
    private static final int QRCOLOR = 0xFF000000; // 默认是黑色
    private static final int BGWHITE = 0xFFFFFFFF; // 背景颜色

    public static byte[] createQrcode(String url, String logoPic, String logoName) {
        byte[] imageByte = null;
        String format = "png";// 图像类型
        Map<EncodeHintType, Object> hints = new HashMap<EncodeHintType, Object>();
        hints.put(EncodeHintType.CHARACTER_SET, "UTF-8");
        try {
            BufferedImage bim = getQR_CODEBufferedImage(url, BarcodeFormat.QR_CODE, width, height, hints);
            imageByte = addLogo_QRCode(bim, getPicByte(logoPic), logoName, format);
            return imageByte;
        } catch (Exception e) {
            log.error(e.getMessage());
            return imageByte;
        }
    }

    public static byte[] getPicByte(String imgUrl) {
        if(StringUtils.isBlank(imgUrl)){
            return null;
        }
        HttpURLConnection conn = null;
        try {
            URL url = new URL(imgUrl);
            try {
                conn = (HttpURLConnection) url.openConnection();
                conn.setRequestMethod("GET");
                InputStream in = conn.getInputStream();
                return IOUtils.toByteArray(in);
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 生成LOGO二维码图片
     *
     * @param bim
     *            图片对象
     * @param logoPic
     *            logo图片
     * @param logoName
     *            logo名称
     * @param format
     *            图片后缀名
     * @return
     */
    private static byte[] addLogo_QRCode(BufferedImage bim, byte[] logoPic, String logoName, String format) {
        try {
            /**
             * 读取二维码图片，并构建绘图对象
             */
            BufferedImage image = bim;
            Graphics2D g = image.createGraphics();
            if (StringUtils.isNotEmpty(logoName)) {
                g.setColor(Color.BLACK);
                g.setFont(new Font("宋体", Font.BOLD, 50)); // 字体、字型、字号

                // int _height=outg.getFontMetrics().getHeight();
                log.info("logoName1:"+logoName);
                int _width = g.getFontMetrics().stringWidth(logoName);
                log.info("logoName2:"+logoName);
                g.drawString(logoName, (width - _width) / 2, height - 60); // 画文字
                // outg.dispose();
                image.flush();
                // image = outImage;
            }

            if (logoPic != null) {
                /**
                 * 读取Logo图片
                 */
                try {
                    BufferedImage logo = ImageIO.read(new ByteArrayInputStream(logoPic));
                    /**
                     * 设置logo的大小,本人设置为二维码图片的20%,因为过大会盖掉二维码
                     */
                    int widthLogo = (int) Math.floor(logo.getWidth(null) > image.getWidth() * 1.5 / 10
                            ? (image.getWidth() * 1.5 / 10) : logo.getWidth(null));
                    int heightLogo = (int) Math.floor(logo.getHeight(null) > image.getHeight() * 1.5 / 10
                            ? (image.getHeight() * 1.5 / 10) : logo.getWidth(null));
                    /**
                     * logo放在中心
                     */
                    int x = (int) Math.floor((image.getWidth() - widthLogo) / 2);
                    int y = (int) Math.floor((image.getHeight() - heightLogo) / 2);
                    /**
                     * logo放在右下角
                     */
                    // 开始绘制图片
                    g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
                    g.drawImage(logo, x, y, widthLogo, heightLogo, null);
                    // g.drawRoundRect(x,y,widthLogo,heightLogo,15,15);
                    g.drawRect(x, y, widthLogo, heightLogo);
                    // logo边框大小
                    g.setStroke(new BasicStroke(2));
                    // logo边框颜色
                    g.setColor(Color.WHITE);
                    g.drawRect(x, y, widthLogo, heightLogo);
                    // g.dispose();
                    logo.flush();
                    image.flush();
                } catch (Exception e) {
                    e.printStackTrace();
                    log.error(e.getMessage());
                }
            }

            g.dispose();
            image.flush();
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            baos.flush();
            ImageIO.write(image, format, baos);
            return baos.toByteArray();
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage());
        }
        return null;
    }

    /**
     * 生成二维码bufferedImage图片
     *
     * @param content
     *            编码内容
     * @param barcodeFormat
     *            编码类型
     * @param width
     *            图片宽度
     * @param height
     *            图片高度
     * @param hints
     *            设置参数
     * @return
     */
    private static BufferedImage getQR_CODEBufferedImage(String content, BarcodeFormat barcodeFormat, int width, int height,
                                                  Map<EncodeHintType, ?> hints) {
        MultiFormatWriter multiFormatWriter = null;
        BitMatrix bm = null;
        BufferedImage image = null;
        try {
            multiFormatWriter = new MultiFormatWriter();
            // 参数顺序分别为：编码内容，编码类型，生成图片宽度，生成图片高度，设置参数
            bm = multiFormatWriter.encode(content, barcodeFormat, width, height, hints);
            int w = bm.getWidth();
            int h = bm.getHeight();
            image = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);

            // 开始利用二维码数据创建Bitmap图片，分别设为黑（0xFFFFFFFF）白（0xFF000000）两色
            for (int x = 0; x < w; x++) {
                for (int y = 0; y < h; y++) {
                    image.setRGB(x, y, bm.get(x, y) ? QRCOLOR : BGWHITE);
                }
            }
        } catch (WriterException e) {
            e.printStackTrace();
        }
        return image;
    }
}
