package com.gic.operation.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.mall.share.api.dto.PayConfigDTO;
import com.gic.mall.share.api.service.PayConfigApiService;
import com.gic.operation.web.vo.apppay.AppVO;
import com.gic.operation.web.vo.apppay.EnterpriseVO;
import com.gic.operation.web.vo.apppay.PayConfigVO;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 支付配置
 * @ClassName: AppPayController 
 * @Description:  
 * @author guojuxing 
 * @date 2020/2/21 9:10 AM 
 */
@RestController
@RequestMapping("/app-pay")
public class AppPayController {
    @Autowired
    private PayConfigApiService payConfigApiService;

    @Autowired
    private EnterpriseApiService enterpriseApiService;

    /**
     * @Title: list 
     * @Description:
     * @author guojuxing
     * @param pageNum
     * @param pageSize
     * @param search 商户/公司名称
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/list")
    public RestResponse list(@RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
            @RequestParam(value = "pageSize", defaultValue = "20") Integer pageSize, String search) {
        ServiceResponse<Page<PayConfigDTO>> list = payConfigApiService.getGicBusinessPayConfig(pageNum, pageSize,
                search);
        return ResultControllerUtils.commonPageResult(list, PayConfigVO.class);
    }

    @RequestMapping("/save")
    public RestResponse save(PayConfigDTO payConfigDTO) {
        //1gic特邀商户
        payConfigDTO.setPayConfigType(1);
        //小程序
        payConfigDTO.setAppType(1);
        if (payConfigDTO.getEnterpriseId() == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (payConfigDTO.getAppletConfigId() == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "小程序配置ID不能为空");
        }
        if (StringUtils.isBlank(payConfigDTO.getSubMchId())) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "字商户号不能为空");
        }
        return ResultControllerUtils.commonResult(payConfigApiService.savePayConfig(payConfigDTO));
    }

    @RequestMapping("/list-app")
    public RestResponse listApp(Integer enterpriseId) {
        return ResultControllerUtils.commonResult(payConfigApiService.getApplets(enterpriseId, 1), AppVO.class);
    }

    @RequestMapping("/delete")
    public RestResponse save(Long configId, Integer enterpriseId) {
        return ResultControllerUtils.commonResult(payConfigApiService.deletePayConfig(configId, enterpriseId));
    }

    @RequestMapping("/list-enterprise-by-name")
    public RestResponse listEnterpriseByName(String search) {
        return ResultControllerUtils.commonResult(enterpriseApiService.listEnterpriseByName(search),
                EnterpriseVO.class);
    }
}
