package com.gic.operation.web.controller;

import java.util.*;

import com.gic.auth.qo.MenuOperationItemListQO;
import com.gic.open.api.dto.ServePropDTO;
import com.gic.open.api.service.ServeApiService;
import com.gic.operation.web.vo.MenuOperationItemListVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.*;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.dto.MenuItemDTO;
import com.gic.auth.qo.AuthItemSaveQO;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.MenuItemApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.operation.web.vo.MenuDetailVO;
import com.gic.operation.web.vo.MenuOperationItemVO;

@RestController
public class MenuController {

    private final static Logger LOGGER = LogManager.getLogger(MenuController.class);
    @Autowired
    private MenuApiService menuApiService;
    @Autowired
    private MenuItemApiService menuItemApiService;
    @Autowired
    private ServeApiService serveApiService;

    @RequestMapping("/sync-product-menu")
    public RestResponse syncProductMenu(String menuCode) {
        return RestResponse.success();
    }


    /**
     * 新增GIC页面
     * @Title: saveGICPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/save-gic-page")
    public RestResponse saveGICPage(MenuDTO menuDTO) {
        return ResultControllerUtils.commonResult(menuApiService.saveGICPage(menuDTO));
    }

    @RequestMapping("/update-gic-page")
    public RestResponse updateGICPage(MenuDTO menuDTO) {
        return ResultControllerUtils.commonResult(menuApiService.updateGICPage(menuDTO));
    }

    @RequestMapping("/delete-menu")
    public RestResponse deleteMenu(Integer menuId) {
        return ResultControllerUtils.commonResult(menuApiService.deleteMenu(menuId));
    }

    /**
     * 新增gic子页面
     * @Title: saveGICChildPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/save-gic-child-page")
    public RestResponse saveGICChildPage(MenuDTO menuDTO) {
        return ResultControllerUtils.commonResult(menuApiService.saveGICChildPage(menuDTO));
    }

    @RequestMapping("/update-gic-child-page")
    public RestResponse updateGICChildPage(MenuDTO menuDTO) {
        return ResultControllerUtils.commonResult(menuApiService.updateGICChildPage(menuDTO));
    }

    /**
     * 编辑应用页面
     * @Title: updateAppPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/update-app-page")
    public RestResponse updateAppPage(MenuDTO menuDTO) {
        return ResultControllerUtils.commonResult(menuApiService.updateAppPage(menuDTO));
    }

    @RequestMapping("/save-app-child-page")
    public RestResponse saveAppChildPage(MenuDTO menuDTO) {
        return ResultControllerUtils.commonResult(menuApiService.saveAppChildPage(menuDTO));
    }

    @RequestMapping("/update-app-child-page")
    public RestResponse updateAppChildPage(MenuDTO menuDTO) {
        return ResultControllerUtils.commonResult(menuApiService.updateAppChildPage(menuDTO));
    }

    /**
     * 查询列表数据
     * @Title: updateAppPage 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-menu-tree")
    public RestResponse listMenuTree(MenuListQO params) {
        LOGGER.info("菜单列表查询参数：{}", JSON.toJSONString(params));
        //逐级展开的方式，所以需要传ID，如果不传，默认查询第一层级，即level = 1
        //        if (params.getMenuId() == null) {
        //            params.setLevel(1);
        //        }
        long serviceBefore = System.currentTimeMillis();
        if (params.getIsGIC() != null && params.getIsGIC() == MenuProjectConstants.APP_PROJECT_CODE) {
            //如果是查询app应用类别，必须实时更新最新应用数据
            params.setNeedUpdateAppMenu(true);
        }
        ServiceResponse<List<MenuDTO>> result = menuApiService.listMenuTree(params);
        long serviceAfter = System.currentTimeMillis();
        LOGGER.info("menuApiService.listMenuTree权限菜单服务调用时间:{}", serviceAfter - serviceBefore);
        if (result.isSuccess()) {
            if (StringUtils.isBlank(params.getSearch())) {
                List<Map<String, Object>> resultInfo = changeListToTree(MenuLevelConstants.FIRST_LEVEL_PARENT_ID,
                        result.getResult());
                LOGGER.info("changeListToTree权限菜单数据递归时间:{}", System.currentTimeMillis() - serviceAfter);
                return RestResponse.success(resultInfo);
            } else {
                return RestResponse.success(addAuthItemSignAndChildrenSignToList(result.getResult()));
            }
        }
        return EnterpriseRestResponse.failure(result);
    }

    /**
     * 查询新增操作项的时候，获取的页面树
     * @Title: listMenuPageTree 
     * @Description: 
      * @author guojuxing 
     * @param params 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-menu-page-tree")
    public RestResponse listMenuPageTree(MenuListQO params) {
        LOGGER.info("菜单列表查询参数：{}", JSON.toJSONString(params));
        params.setMenuType(MenuTypeEnum.PAGE.getCode());
        ServiceResponse<List<MenuDTO>> result = menuApiService.listMenuTree(params);
        if (result.isSuccess()) {
            return RestResponse
                    .success(changePageListToTree(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, result.getResult()));
        }
        return EnterpriseRestResponse.failure(result);
    }

    /**
     * 查询页面下面的操作项列表数据
     * @Title: listMenuRole 
     * @Description:
      * @author guojuxing
     * @param parentId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-menu-item")
    public RestResponse listMenuItem(Integer parentId) {
        ServiceResponse<List<MenuDTO>> result = menuApiService.listMenuItemByParentId(parentId);
        if (result.isSuccess()) {
            List<MenuDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                List<MenuOperationItemVO> voList = new ArrayList<>(list.size());
                for (MenuDTO menuDTO : list) {
                    //仅超管可用的过滤
                    if (menuDTO.getAdminOnlySign().intValue() != MenuOperationItemConstants.ADMIN_ONLY) {
                        voList.add(EntityUtil.changeEntityNew(MenuOperationItemVO.class, menuDTO));
                    }
                }
                return RestResponse.success(voList);
            }
        }

        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("/set-menu-sort")
    public RestResponse listMenuRole(Integer id, Integer sort) {
        return ResultControllerUtils.commonResult(menuApiService.setSort(id, sort));
    }

    @RequestMapping("/get-menu-version-by-id")
    public RestResponse getMenuVersionById(Integer id) {
        ServiceResponse<MenuDTO> result = menuApiService.getDetailById(id);
        if (result.isSuccess()) {
            return RestResponse.success(getMenuVersionMap(result.getResult()));
        }
        return EnterpriseRestResponse.failure(result);
    }

    /**
     * 新增页面的时候，应用版本下拉数据
     * @Title: getMenuVersionByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/get-menu-version-by-parent-id")
    public RestResponse getMenuVersionByParentId(Integer parentId) {
        if (parentId == null) {
            //如果为空，则是gic新增页面
            Map<String, String> map = menuApiService.getGicDefaultVersion().getResult();
            return RestResponse.success(map);
        }
        ServiceResponse<MenuDTO> result = menuApiService.getDetailById(parentId);
        if (result.isSuccess()) {
            return RestResponse.success(getMenuVersionMap(result.getResult()));
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("/get-menu-detail-by-id")
    public RestResponse getMenuDetailById(Integer id) {
        ServiceResponse<MenuDTO> result = menuApiService.getDetailById(id);
        if (result.isSuccess()) {
            MenuDTO dto = result.getResult();
            MenuDetailVO vo = EntityUtil.changeEntityNew(MenuDetailVO.class, dto);
            if (dto.getMenuType().intValue() == MenuTypeEnum.ROLE.getCode()) {
                //权限项
                List<MenuItemDTO> menuItemDTOList = menuItemApiService.listMenuItemByMenuId(id).getResult();
                if (CollectionUtils.isNotEmpty(menuItemDTOList)) {
                    StringBuilder itemIds = new StringBuilder();
                    for (MenuItemDTO menuItemDTO : menuItemDTOList) {
                        itemIds.append(menuItemDTO.getItemId()).append(SignConstants.COMMA);
                    }
                    String tempItemIds = itemIds.toString();
                    vo.setOperationItemIds(tempItemIds.substring(0, tempItemIds.length() - 1));
                }
            } else {
                if (StringUtils.isNotBlank(dto.getMenuVersion())) {
                    String menuVersion = dto.getMenuVersion().substring(1, dto.getMenuVersion().length() - 1);
                    vo.setMenuVersion(menuVersion.replaceAll(SignConstants.UNDERLINE, SignConstants.COMMA));
                }
                vo.setMap(getMenuVersionMap(result.getResult()));
            }
            return RestResponse.success(vo);
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("/save-operation-item")
    public RestResponse saveOperationItem(MenuDTO menuDTO) {
        return ResultControllerUtils.commonResult(menuApiService.saveOperationItem(menuDTO));
    }

    @RequestMapping("/update-operation-item")
    public RestResponse updateOperationItem(MenuDTO menuDTO) {
        return ResultControllerUtils.commonResult(menuApiService.updateOperationItem(menuDTO));
    }

    @RequestMapping("/save-auth-item")
    public RestResponse saveAuthItem(AuthItemSaveQO params) {
        return ResultControllerUtils.commonResult(menuApiService.saveAuthItem(params));
    }

    @RequestMapping("/update-auth-item")
    public RestResponse updateAuthItem(AuthItemSaveQO params) {
        return ResultControllerUtils.commonResult(menuApiService.updateAuthItem(params));
    }

    @RequestMapping("/list-auth-item-all")
    public RestResponse updateAuthItemAll(MenuOperationItemListQO params) {
        return ResultControllerUtils.commonPageResult(menuApiService.pageMenuOperationItem(params),
                MenuOperationItemListVO.class);
    }

    /**
     * 查询页面下的权限项列表数据
     * @Title: listAuthItem 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-auth-item")
    public RestResponse listAuthItem(Integer menuId) {
        return ResultControllerUtils.commonResult(menuApiService.listAuthItemByMenuId(menuId));
    }

    private Map<String, String> getMenuVersionMap(MenuDTO dto) {
        Map<String, String> map = new HashMap<>(16);
        if (dto.getProject().equals(MenuProjectConstants.DEFAULT_PROJECT) && dto.getLevel() == 1) {
            //如果为空，则是gic新增页面
            map = menuApiService.getGicDefaultVersion().getResult();
        } else {
            String menuVersion = dto.getMenuVersion();
            String menuVersionName = dto.getMenuVersionName();
            if (StringUtils.isNotBlank(menuVersion)) {
                String[] menuVersionArr = menuVersion.substring(1, menuVersion.length() - 1)
                        .split(SignConstants.UNDERLINE);
                String[] menuVersionNameArr = menuVersionName.split(SignConstants.SLASH);
                for (int i = 0, len = menuVersionArr.length; i < len; i++) {
                    if (StringUtils.isBlank(menuVersionNameArr[i])) {
                        continue;
                    }
                    map.put(menuVersionArr[i], menuVersionNameArr[i]);
                }
            }
        }
        return map;
    }

    public static List<Map<String, Object>> changeListToTree(int storeGroupId, List<MenuDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (int j = 0, length = list.size(); j < length; j++) {
                MenuDTO menuDTO = list.get(j);
                if (storeGroupId == menuDTO.getParentId()) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("menuId", menuDTO.getMenuId());
                    data.put("menuName", menuDTO.getMenuName());
                    data.put("menuUrl", menuDTO.getMenuUrl());
                    data.put("project", menuDTO.getProject());
                    data.put("sort", menuDTO.getSort());
                    data.put("menuCode", menuDTO.getMenuCode());
                    data.put("parentCode", menuDTO.getParentCode());
                    data.put("parentId", menuDTO.getParentId());
                    data.put("menuVersionName", menuDTO.getMenuVersionName());
                    if (menuDTO.getLevel().intValue() == MenuLevelConstants.FIRST_LEVEL) {
                        data.put("seq", j + 1);
                    }
                    if (menuDTO.getMenuType().intValue() == MenuTypeEnum.ROLE.getCode()) {
                        //如果是权限项
                        data.put("isAuthItem", true);
                    } else {
                        data.put("isAuthItem", false);
                    }
                    List<Map<String, Object>> children = changeListToTree(menuDTO.getMenuId(), list);
                    if (children == null || children.size() < 1) {
                        data.put("children", null);
                        data.put("hasAuthItem", false);
                    } else {
                        for (int i = 0, len = children.size(); i < len; i++) {
                            children.get(i).put("seq", i + 1);
                        }
                        data.put("children", children);
                        data.put("hasAuthItem", hasAuthItem(children));
                    }
                    result.add(data);
                }
            }
        }
        return result;
    }

    /**
     * 是权限项标识/是否有子菜单
     * @param list
     * @return
     */
    public List<Map<String, Object>> addAuthItemSignAndChildrenSignToList(List<MenuDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            List<Integer> parentMenuIdList = new ArrayList<>(list.size());
            for (MenuDTO menuDTO : list) {
                parentMenuIdList.add(menuDTO.getMenuId());
            }
            Set<String> authItemMap = new HashSet<>(16);
            Set<String> childrenMap = new HashSet<>(16);
            ServiceResponse<List<MenuDTO>> listByParentIdListResult = menuApiService
                    .listByParentMenuIdList(parentMenuIdList);
            if (listByParentIdListResult.isSuccess()) {
                List<MenuDTO> listByParentIdList = listByParentIdListResult.getResult();
                if (CollectionUtils.isNotEmpty(listByParentIdList)) {
                    for (MenuDTO child : listByParentIdList) {
                        int menuType = child.getMenuType().intValue();
                        if (menuType == MenuTypeEnum.ROLE.getCode()) {
                            authItemMap.add(child.getParentId().toString());
                        } else if (menuType == MenuTypeEnum.PAGE.getCode()) {
                            childrenMap.add(child.getParentId().toString());
                        }
                    }
                }
            }
            for (int j = 0, length = list.size(); j < length; j++) {
                MenuDTO menuDTO = list.get(j);
                Map<String, Object> data = new HashMap<>(16);
                data.put("menuId", menuDTO.getMenuId());
                data.put("menuName", menuDTO.getMenuName());
                data.put("menuUrl", menuDTO.getMenuUrl());
                data.put("project", menuDTO.getProject());
                data.put("sort", menuDTO.getSort());
                data.put("menuCode", menuDTO.getMenuCode());
                data.put("parentCode", menuDTO.getParentCode());
                data.put("parentId", menuDTO.getParentId());
                data.put("menuVersionName", menuDTO.getMenuVersionName());
                data.put("seq", j + 1);
                //是否拥有子集，用于 展开按钮的显示
                data.put("children", childrenMap.contains(menuDTO.getMenuId().toString()));
                //是否有权限项
                data.put("hasAuthItem", authItemMap.contains(menuDTO.getMenuId().toString()));
                result.add(data);
            }
        }
        return result;
    }

    public static List<Map<String, Object>> changePageListToTree(int storeGroupId, List<MenuDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (int j = 0, length = list.size(); j < length; j++) {
                MenuDTO menuDTO = list.get(j);
                if (storeGroupId == menuDTO.getParentId()) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("menuId", menuDTO.getMenuId());
                    data.put("menuName", menuDTO.getMenuName());
                    data.put("menuCode", menuDTO.getMenuCode());
                    data.put("parentCode", menuDTO.getParentCode());
                    data.put("parentId", menuDTO.getParentId());
                    List<Map<String, Object>> children = changePageListToTree(menuDTO.getMenuId(), list);
                    if (children == null || children.size() < 1) {
                        data.put("children", null);
                    } else {
                        data.put("children", children);
                    }
                    result.add(data);
                }
            }
        }
        return result;
    }

    /**
     * 是否拥有权限项
     * @Title: hasAuthItem 
     * @Description:
      * @author guojuxing
     * @param children 
     * @return  boolean 
      */
    private static boolean hasAuthItem(List<Map<String, Object>> children) {
        for (Map<String, Object> map : children) {
            if ((boolean) map.get("isAuthItem")) {
                return true;
            }
        }
        return false;
    }
}
