package com.gic.operation.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.PlatformBrandDTO;
import com.gic.enterprise.dto.PlatformBrandRefDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.PlatformBrandApiService;
import com.gic.operation.web.qo.PageQO;
import com.gic.operation.web.qo.PlatformBrandQO;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class PlatformBrandController {

    private static final Logger LOGGER = LoggerFactory.getLogger(PlatformBrandController.class);

    @Autowired
    private PlatformBrandApiService platformBrandApiService;

    @RequestMapping("/brand-save")
    public RestResponse saveBrand(@Validated({ PlatformBrandQO.SavePlatformBrandValid.class }) PlatformBrandQO platformBrandQO) {
        PlatformBrandDTO platformBrandDTO = new PlatformBrandDTO();
        platformBrandDTO.setPlatformBrandName(platformBrandQO.getPlatformBrandName());
        platformBrandDTO.setPlatformBrandCategoryCode(platformBrandQO.getPlatformBrandCategoryCode());
        platformBrandDTO.setPlatformBrandId(platformBrandQO.getPlatformBrandId());
        ServiceResponse<Integer> serviceResponse = platformBrandApiService.saveOrUpdateBrandFromOperation(platformBrandDTO);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/brand-del")
    public RestResponse delBrand(@Validated({ PlatformBrandQO.DelPlatformBrandValid.class }) PlatformBrandQO platformBrandQO) {
        ServiceResponse<Integer> serviceResponse = platformBrandApiService.delBrandById(platformBrandQO.getPlatformBrandId());
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/brand-list")
    public RestResponse listBrand(PlatformBrandQO platformBrandQO, PageQO pageQO) {
        ServiceResponse<Page<PlatformBrandDTO>> serviceResponse = platformBrandApiService.listPlatformBrand(EntityUtil.changeEntityByOrika(PlatformBrandDTO.class, platformBrandQO), pageQO.getCurrentPage(), pageQO.getPageSize());
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }
    @RequestMapping("/brand-ref-list")
    public RestResponse listBrandRef(PlatformBrandQO platformBrandQO, PageQO pageQO) {
        ServiceResponse<Page<PlatformBrandRefDTO>> serviceResponse = platformBrandApiService.listByPlatformBrandRef(platformBrandQO.getPlatformBrandName(), pageQO.getCurrentPage(), pageQO.getPageSize());
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/brand-merge")
    public RestResponse mergeBrand(String fromBrandId, String toBrandName) {
        if (fromBrandId == null || StringUtils.isBlank(toBrandName)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ErrorCode.PARAMETER_ERROR.getMsg());
        }
        ServiceResponse<Integer> serviceResponse = platformBrandApiService.mergeBrandRef(fromBrandId, toBrandName);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }


}
